@@ -68,7 +68,16 @@ module FileOption = struct
           let get_value name conv = conv (List.assoc name assocs) in
           let network = try get_value "file_network" value_to_string
             with _ -> "Donkey" in
-          let network = network_find_by_name network in
+          let network = 
+            try network_find_by_name network with e ->
+                lprintf_nl ()
+		  "Error %s for network %s while parsing file %s"
+		    (Printexc2.to_string e)
+		    network
+		    (get_value "file_filename" value_to_string);
+		lprintf_nl () "This core is lacking support for network %s, exiting" network;
+                exit_properly 70
+          in
           let file_state = 
             try
               get_value "file_state" value_to_state 
@@ -92,9 +101,9 @@ module FileOption = struct
           set_file_state file file_state;       
           if !verbose then
           (match file_state with
-              FileDownloading -> lprintf_nl () "New downloading file";
-            | FileDownloaded -> lprintf_nl () "New downloaded file";
-            | _ -> lprintf_nl () "New file with other state"
+              FileDownloading -> lprintf_nl () "New downloading file - check complete";
+            | FileDownloaded -> lprintf_nl () "New downloaded file - check complete";
+            | _ -> lprintf_nl () "New file with other state - check complete"
           );
           
           (try
@@ -641,7 +650,7 @@ let sharing_only_directory = {
     sharing_directories = false;
     sharing_extensions = [];
     sharing_recursive = false;
-    sharing_minsize = Int64.of_int 1;
+    sharing_minsize = Int64.one;
     sharing_maxsize = Int64.max_int;
   }  
   
@@ -650,7 +659,7 @@ let sharing_incoming_directories = {
     sharing_directories = true;
     sharing_extensions = [];
     sharing_recursive = false;
-    sharing_minsize = Int64.of_int 1;
+    sharing_minsize = Int64.one;
     sharing_maxsize = Int64.max_int;
   }  
   
@@ -659,7 +668,7 @@ let sharing_incoming_files = {
     sharing_directories = false;
     sharing_extensions = [];
     sharing_recursive = false;
-    sharing_minsize = Int64.of_int 1;
+    sharing_minsize = Int64.one;
     sharing_maxsize = Int64.max_int;
   }
 
@@ -668,7 +677,7 @@ let sharing_directories = {
       sharing_directories = true;
       sharing_extensions = [];
       sharing_recursive = false;
-      sharing_minsize = Int64.of_int 1;
+      sharing_minsize = Int64.one;
       sharing_maxsize = Int64.max_int;
     }
   
@@ -703,7 +712,7 @@ let sharing_strategies = define_option searches_section
       sharing_directories = false;
       sharing_extensions = [];
       sharing_recursive = true;
-      sharing_minsize = Int64.of_int 1;
+      sharing_minsize = Int64.one;
       sharing_maxsize = Int64.max_int;
     };
     
@@ -820,7 +829,18 @@ module SharedDirectoryOption = struct
   
 let shared_directories = 
   define_option CommonOptions.path_section ["shared_directories" ] 
-  "Directories where files will be shared"
+  "                Incoming and shared directories.
+   At least two entries have to be present here, one with strategy
+   incoming_files and one with strategy incoming_directories.
+   Both entries can point to the same directory.
+   If one of the two strategies is missing, MLDonkey will create a default
+   directory with its entry here.
+   Finished BT multifile downloads are committed to the first directory
+   with strategy incoming_directories. Other downloads are committed
+   to the first directory with the strategy incoming_files.
+   If more than one directory has one of the incoming_* strategies
+   it will be ignored on commit, but they are shared nonetheless.
+   Other strategies can be found in searches.ini, section customized_sharing."
     (list_option SharedDirectoryOption.t) 
   [
     {
@@ -844,7 +864,7 @@ let shared_directories =
   ]
 
 
-let incoming_files () =
+let search_incoming_files () =
   try
     List.find (fun s -> s.shdir_strategy = "incoming_files") 
     !!shared_directories
@@ -860,7 +880,13 @@ let incoming_files () =
         shared_directories =:= s :: !!shared_directories;
         s
 
-let incoming_directories () =
+let incoming_files () =
+  let dir = search_incoming_files () in
+    Unix2.safe_mkdir dir.shdir_dirname;
+    Unix2.can_write_to_directory dir.shdir_dirname;
+    dir
+
+let search_incoming_directories () =
   try
     List.find (fun s -> s.shdir_strategy = "incoming_directories") 
     !!shared_directories
@@ -876,6 +902,12 @@ let incoming_directories () =
       shared_directories =:= s :: !!shared_directories;
       s
 
+let incoming_directories () =
+  let dir = search_incoming_directories () in
+    Unix2.safe_mkdir dir.shdir_dirname;
+    Unix2.can_write_to_directory dir.shdir_dirname;
+    dir
+
 let _ =
 (* Check the definition of the incoming_files and incoming_directories in
 shared_directories *)
@@ -971,7 +1003,7 @@ let backup_tar archive files =
 	if size > Sys.max_string_length then
 	  begin
 	    Tar.close_out otar;
-	    failwith (Printf.sprintf "Tar: file %s too big" arg)
+	    failwith (Printf.sprintf "Tar: file %s too big, limit %d byte" arg Sys.max_string_length)
 	  end;
 	let header = 
 	  { Tar.t_name = arg;
@@ -999,12 +1031,12 @@ let backup_tar archive files =
 	  let error = (Printexc2.to_string e) in
 	    if error =
 	      "Gzip.Error(\"error during compression\")"
-	      && Autoconf.system = "windows"
-	      && arg = "fasttrack.ini" then
+	      && Autoconf.windows && arg = "fasttrack.ini" then
               (* for whatever reason this error is raised on Windows,
                  but fasttrack.ini is stored correctly *)
-	      if !verbose_hidden_errors then
+	      if !verbose then
 	        lprintf_nl () "Tar: Windows specific pseudo error %s in %s" error arg
+	      else ()
 	    else
 	      begin
 		if !failed_files = "" then
@@ -1056,3 +1088,254 @@ let backup_options () =
     with e -> lprintf_nl () "Exception %s while options backup" (Printexc2.to_string e); raise e
   end;
   lprintf_nl () "Options backup as %s correctly saved" format
+             
+let buildinfo html buf =
+  let s =
+  (
+        "MLNet Multi-Network p2p client version " ^ Autoconf.current_version
+      ^ (if Autoconf.scm_version <> "" then "\nSCM version info: " ^ Autoconf.scm_version else "")
+      ^ "\nNetworks: " ^ !networks_string
+      ^ "\nOcaml version: " ^ Sys.ocaml_version
+      ^ "\nBuild on: " ^ Autoconf.build_system ^ " (" ^ Unix2.endianness () ^ ")"
+      ^ (if Autoconf.glibc_version = "" then "" 
+          else
+            let real_glibc_version = MlUnix.glibc_version_num () in
+            if real_glibc_version = "" || real_glibc_version = Autoconf.glibc_version 
+              then " with glibc " ^ Autoconf.glibc_version
+            else 
+              Printf.sprintf " (Warning: glibc version mismatch, %s present on your system, MlDonkey was compiled with %s)"
+              real_glibc_version Autoconf.glibc_version)  
+      ^ (if Autoconf.configure_arguments <> "" then
+           "\nConfigure arguments: " ^ Autoconf.configure_arguments else "")
+      ^ (if !patches_string <> "" then "\n" ^ !patches_string else "")
+      ^ "\nFeatures:"
+          ^ (if BasicSocket.has_threads () then " threads" else " no-threads")
+          ^ (let s = Zlib.zlib_version_num () in Printf.sprintf " zlib%s" (if s <> "" then "-" ^ s else ""))
+          ^ (if Autoconf.has_bzip2 then
+	       begin
+	         let s =
+	           (* catch the exception in the case Bzip2.bzlib_version_num returns an empty string *)
+	           try
+	             List.hd(String2.split_simplify
+		   (Misc2.bzlib_version_num ()) ',') 
+		   with e -> ""
+		 in
+	         Printf.sprintf " bzip2%s" (if s <> "" then "-" ^ s else "")
+	       end
+	     else " no-bzip2")
+          ^ (if Autoconf.has_gd && Autoconf.has_gd_png && Autoconf.has_gd_jpg then
+	       Printf.sprintf " gd(jpg/png%s)"
+	         (let s = CommonGraphics.png_version_num () in
+		    if s <> "" then "-" ^ s else "")
+	     else "")
+          ^ (if Autoconf.has_gd && Autoconf.has_gd_png && not Autoconf.has_gd_jpg then
+	       Printf.sprintf " gd(png%s)"
+	         (let s = CommonGraphics.png_version_num () in
+		    if s <> "" then "-" ^ s else "")
+	     else "")
+	  ^ (if Autoconf.has_gd && not Autoconf.has_gd_png && Autoconf.has_gd_jpg then " gd(jpg)" else "")
+	  ^ (if not Autoconf.has_gd then " no-gd" else "")
+          ^ (if Autoconf.has_iconv then " iconv" else " no-iconv")
+          ^ (if Autoconf.check_bounds then " check-bounds" else " no-check-bounds")
+  )
+  in
+  if html then
+    begin
+      Printf.bprintf buf "\\<div class=\\\"cs\\\"\\>";
+      html_mods_table_header buf "versionTable" "results" [];
+      Printf.bprintf buf "\\<tr\\>";
+      html_mods_td buf [ ("", "srh", "Buildinfo"); ];
+      Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-1\\\"\\>";
+      html_mods_td buf [ ("", "sr", Str.global_replace (Str.regexp "\n") "\\<br\\>" s); ];
+      Printf.bprintf buf "\\</tr\\>\\</table\\>\\</div\\>\\</div\\>";
+    end
+  else
+    Printf.bprintf buf "Buildinfo:\n%s\n" s
+  
+let runinfo html buf =
+  let s =
+  (
+        "Enabled Networks: " 
+      ^   (if Autoconf.donkey = "yes" && !!enable_donkey then " Donkey" else "")
+      ^   (if Autoconf.donkey = "yes" && !!enable_overnet then " Overnet" else "")
+      ^   (if Autoconf.donkey = "yes" && !!enable_kademlia then " Kademlia" else "")             
+      ^   (if Autoconf.bittorrent = "yes" && !!enable_bittorrent then " BitTorrent" else "")
+      ^   (if Autoconf.fasttrack = "yes" && !!enable_fasttrack then " Fasttrack" else "")
+      ^   (if Autoconf.gnutella = "yes" && !!enable_gnutella then " Gnutella" else "")
+      ^   (if Autoconf.gnutella2 = "yes" && !!enable_gnutella2 then " G2" else "")
+      ^   (if Autoconf.filetp = "yes" && !!enable_fileTP then " FileTP" else "")
+      ^  "\nServer usage: " ^ (if !!enable_servers then "enabled" else "disabled (you are not able to connect to ED2K Servers)")
+      ^  (if !Geoip.active then "\nThis product includes GeoLite data created by MaxMind, available from http://maxmind.com/" else "")
+      ^  (let r1,r2 = Ip_set.block_stats () in if r1 = 0 then
+	    "\nIP blocking disabled" else Printf.sprintf "\nIP blocking enabled: %d ranges loaded - optimized to %d" r1 r2)
+      ^ (let uname = Unix32.uname () in
+          if uname <> "" then Printf.sprintf "\nSystem info: %s" uname else "")
+      ^ "\nUptime: " ^ Date.time_to_string (last_time () - start_time) "verbose"
+      ^ " - Language: " ^ Charset.default_language
+      ^ " - locale: " ^ Charset.locstr
+      ^ " - UTC offset: " ^ Rss_date.mk_timezone (Unix.time ())
+      ^ "\n max_string_length: " ^ string_of_int Sys.max_string_length
+      ^ " - word_size: " ^ string_of_int Sys.word_size
+      ^ " - max_array_length: " ^ string_of_int Sys.max_array_length
+      ^ "\n max file descriptors: " ^ string_of_int (Unix2.c_getdtablesize ())
+      ^ " - max useable file size: " ^ 
+	    (match Unix2.c_sizeofoff_t () with
+	     | 4 -> "2GB"
+	     |  _ -> Printf.sprintf "2^%d-1 bits (do the maths ;-p)" ((Unix2.c_sizeofoff_t () *8)-1)
+	     )
+  )    
+  in
+  if html then
+    begin
+      Printf.bprintf buf "\\<div class=\\\"cs\\\"\\>";
+      html_mods_table_header buf "versionTable" "results" [];
+      Printf.bprintf buf "\\<tr\\>";
+      html_mods_td buf [ ("", "srh", "Runinfo"); ];
+      Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-1\\\"\\>";
+      html_mods_td buf [ ("", "sr", Str.global_replace (Str.regexp "\n") "\\<br\\>" s); ];
+      Printf.bprintf buf "\\</tr\\>\\</table\\>\\</div\\>\\</div\\>";
+    end
+  else
+    Printf.bprintf buf "Runinfo:\n%s\n" s
+
+let diskinfo html buf =
+  let list = ref [] in
+  ignore (search_incoming_files ());
+  ignore (search_incoming_directories ());
+  List.iter (fun dir ->
+     list := (dir.shdir_dirname, (Printf.sprintf "shared (%s)" dir.shdir_strategy))
+     :: !list) !!shared_directories;
+  list := (!!temp_directory, "temp/downloading") :: !list;
+  list := (Sys.getcwd (), "core/ini files") :: !list;
+
+  let len_dir = ref 9 in
+  let len_strategy = ref 29 in (* "shared (incoming_directories)" *)
+  List.iter ( fun (dir, strategy) ->
+    len_dir := maxi !len_dir (String.length dir);
+    len_strategy := maxi !len_strategy (String.length strategy)
+  ) !list;
+  let fill_dir = String.make (!len_dir - 9) ' ' in
+  let fill_dir_line = String.make (!len_dir - 9) '-' in
+  let fill_strategy = String.make (!len_strategy - 4) ' ' in
+  let fill_strategy_line = String.make (!len_strategy - 4) '-' in
+  let counter = ref 0 in
+  if html then
+      html_mods_table_header buf "sharesTable" "shares" [
+       ( "0", "srh", "Directory", "Directory" ) ;
+       ( "0", "srh", "Directory type", "Type" ) ;
+       ( "1", "srh ar", "HDD used", "used" ) ;
+       ( "1", "srh ar", "HDD free", "free" ) ;
+       ( "1", "srh ar", "% free", "% free" ) ;
+       ( "0", "srh", "Filesystem", "FS" ) ]
+  else
+    begin
+      Printf.bprintf buf "Diskinfo:\n";
+      Printf.bprintf buf "Directory%s|Type%s|    used|    free|%%free|Filesystem\n"
+        fill_dir fill_strategy;
+      Printf.bprintf buf "---------%s+----%s+--------+--------+-----+----------\n"
+        fill_dir_line fill_strategy_line;
+    end;
+  List.iter (fun (dir, strategy) ->
+	incr counter;
+	let diskused =
+    	  match Unix32.diskused dir with
+	  | None -> Printf.sprintf "---"
+	  | Some du -> size_of_int64 du
+	in
+	let diskfree =
+    	  match Unix32.diskfree dir with
+	  | None -> Printf.sprintf "---"
+	  | Some df -> size_of_int64 df
+	in
+	let percentfree =
+    	  match Unix32.percentfree dir with
+	  | None -> Printf.sprintf "---"
+	  | Some p -> Printf.sprintf "%d%%" p
+	in
+	let filesystem = Unix32.filesystem dir in
+	if html then
+	  begin
+	    Printf.bprintf buf "\\<tr class=\\\"%s\\\"\\>
+	\\<td class=\\\"sr\\\"\\>%s\\</td\\>
+	\\<td class=\\\"sr\\\"\\>%s\\</td\\>
+	\\<td class=\\\"sr ar\\\"\\>%s\\</td\\>
+	\\<td class=\\\"sr ar\\\"\\>%s\\</td\\>
+	\\<td class=\\\"sr ar\\\"\\>%s\\</td\\>
+	\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\</tr\\>"
+	    (if !counter mod 2 == 0 then "dl-1" else "dl-2")
+	    dir strategy diskused diskfree percentfree filesystem
+	  end
+	else
+	  Printf.bprintf buf "%-*s|%-*s|%8s|%8s|%5s|%-s\n"
+	    (maxi !len_dir (!len_dir - String.length dir)) dir
+	    (maxi !len_strategy (!len_strategy - String.length strategy)) strategy
+	    diskused diskfree percentfree filesystem
+    	) !list;
+  if html then
+    Printf.bprintf buf "\\</table\\>\\</td\\>\\<tr\\>\\</table\\>\\</div\\>"
+
+let _ =
+  option_hook ip_blocking (fun _ ->
+    (try
+      Ip_set.bl := if !!ip_blocking <> "" then
+                     Ip_set.load !!ip_blocking
+                   else Ip_set.bl_empty
+    with _ -> ());
+    CommonServer.check_blocked_servers ()
+  );
+  option_hook max_opened_connections (fun _ ->
+  if !verbose then lprintf_nl ()
+    "checking max_opened_connections = %d for validity" !!max_opened_connections;
+(* original code from ./src/config/unix/MlUnix.ml
+let max_all_sockets = getdtablesize ()
+let max_sockets = max (max_all_sockets - 100) (max_all_sockets / 2)
+let max_filedescs = (max_all_sockets - max_sockets) / 2 *)
+
+  (* ulimit open files. minimum 150, most systems have 1024 *)
+  let max_all_sockets = Unix2.c_getdtablesize () in
+
+  (* old max_sockets code: max (150 - 100) (150 / 2),
+     minimum number of max_opened_connections *)
+  let min_conns = 75 in
+
+  if min_conns > !!max_opened_connections then begin
+    lprintf_nl () "max_opened_connections is set too low (%d), raising to %d"
+      !!max_opened_connections min_conns;
+    max_opened_connections =:= min_conns
+  end;
+
+  let total_files = (* maximum number of files in use at the same time *)
+    (maxi (List.length !!files) !!max_concurrent_downloads) + !!max_upload_slots + 20 (* ini files etc. *)
+  in
+
+  let wanted_socks = !!max_opened_connections + total_files in
+
+  if max_all_sockets < wanted_socks then
+    if max_all_sockets < total_files + min_conns then (* check if ulimit is enough to allow total_files + min_conns *)
+      begin
+        lprintf_nl () "only %d file descriptors available, raise ulimit open files to at least %d"
+          max_all_sockets wanted_socks;
+        lprintf_nl () "FD info: max_opened_connections %d, number of (possible) concurrent downloads %d, = %d fd needed"
+          !!max_opened_connections total_files wanted_socks;
+        CommonGlobals.exit_properly 71
+      end
+  else
+    begin
+      let new_max_opened_connections =
+        maxi (max_all_sockets - total_files) (max_all_sockets / 2)
+      in
+      lprintf_nl () "max_opened_connections is set too high (%d), reducing to %d"
+        !!max_opened_connections new_max_opened_connections;
+      max_opened_connections =:= new_max_opened_connections;
+    end;
+
+  if !verbose then lprintf_nl ()
+    "max_opened_connections %d, total_files %d, max_concurrent_downloads %d, !!files %d"
+      !!max_opened_connections total_files !!max_concurrent_downloads (List.length !!files);
+
+  TcpBufferedSocket.set_max_opened_connections
+    (fun _ -> !!max_opened_connections);
+
+  Unix32.max_cache_size := total_files - 20;
+  calc_real_max_indirect_connections ()
+)
