@@ -21,6 +21,9 @@ val load : unit -> unit
 val save : unit -> unit
 val save_sources : unit -> unit
 val backup_options : unit -> unit
+val buildinfo : bool -> Buffer.t -> unit
+val runinfo : bool -> Buffer.t -> unit
+val diskinfo : bool -> Buffer.t -> unit
   
 val done_files :  CommonTypes.file list Options.option_record
 val files :  CommonTypes.file list Options.option_record
