@@ -102,76 +102,6 @@ let version () =
   Printf.sprintf "MLNet %s: Multi-Network p2p client (%s)"
     Autoconf.current_version !networks_string
 
-
-let buildinfo () =
-  (
-    "MLNet Multi-Network p2p client version " ^ Autoconf.current_version
-      ^ (if Autoconf.scm_version <> "" then "\nSCM version info: " ^ Autoconf.scm_version else "")
-      ^ "\nNetworks: " ^ !networks_string
-      ^ "\nOcaml version: " ^ Autoconf.ocaml_version
-      ^ "\nBuild on: " ^ Autoconf.build_system
-          ^ (if Autoconf.glibc_version <> "" then " with glibc " ^ Autoconf.glibc_version else "")
-	  ^ (let real_glibc_version = MlUnix.glibc_version_num () in
-	       if real_glibc_version <> Autoconf.glibc_version
-	         && real_glibc_version <> "" then
-		   Printf.sprintf " (DANGER: glibc %s present on system)" real_glibc_version else "")
-      ^ (if Autoconf.configure_arguments <> "" then "\nConfigure arguments: " ^ Autoconf.configure_arguments else "")
-      ^ (if !patches_string <> "" then "\n" ^ !patches_string else "")
-      ^ "\nFeatures:"
-          ^ (if BasicSocket.has_threads () then " threads" else " no-threads")
-          ^ (if Autoconf.has_zlib then
-	       begin
-	         let s = Zlib.zlib_version_num () in
-	           Printf.sprintf " zlib%s" (if s <> "" then "-" ^ s else "")
-	       end
-	     else " no-zlib")
-          ^ (if Autoconf.has_bzip2 then
-	       begin
-	         let s =
-	           (* catch the exception in the case Bzip2.bzlib_version_num returns an empty string *)
-	           try
-	             List.hd(String2.split_simplify
-		   (Bzip2.bzlib_version_num ()) ',') 
-		   with e -> ""
-		 in
-	         Printf.sprintf " bzip2%s" (if s <> "" then "-" ^ s else "")
-	       end
-	     else " no-bzip2")
-          ^ (if Autoconf.has_gd && Autoconf.has_gd_png && Autoconf.has_gd_jpg then
-	       Printf.sprintf " gd(jpg/png%s)"
-	         (let s = Gd.png_version_num () in
-		    if s <> "" then "-" ^ s else "")
-	     else "")
-          ^ (if Autoconf.has_gd && Autoconf.has_gd_png && not Autoconf.has_gd_jpg then
-	       Printf.sprintf " gd(png%s)"
-	         (let s = Gd.png_version_num () in
-		    if s <> "" then "-" ^ s else "")
-	     else "")
-	  ^ (if Autoconf.has_gd && not Autoconf.has_gd_png && Autoconf.has_gd_jpg then " gd(jpg)" else "")
-	  ^ (if not Autoconf.has_gd then " no-gd" else "")
-          ^ (if Autoconf.has_iconv then " iconv" else " no-iconv")
-          ^ (if Autoconf.check_bounds then " check-bounds" else " no-check-bounds")
-          ^ " " ^ Autoconf.sha1_version
-      ^ "\nLanguage: " ^ Charset.default_language
-      ^ " - locale: " ^ Charset.locstr
-      ^ "\n max_string_length: " ^ string_of_int Sys.max_string_length
-      ^ " - word_size: " ^ string_of_int Sys.word_size
-      ^ " - max_array_length: " ^ string_of_int Sys.max_array_length
-      ^ "\n max file descriptors: " ^ string_of_int (Unix2.c_getdtablesize ())
-      ^ " - max useable file size: " ^ 
-	    (match Unix2.c_sizeofoff_t () with
-	     | 4 -> "2GB"
-
-	     |  _ -> Printf.sprintf "2^%d-1 bits (do the maths ;-p)" ((Unix2.c_sizeofoff_t () *8)-1)
-
-(*
-	     | _ -> Printf.sprintf "%s"
-	       ((*size_of_int64*)(string_of_float(((2. ** (float_of_int((Unix2.c_sizeofoff_t () * 8)-1))) -. 1.)/. 8.)))
-*)
-
-	     ))
-
-
 (* Should we try to find another port when we cannot bind to the one set
 in an option, and then change the option accordingly. ?> *)
 let find_other_port = ref false
@@ -214,14 +144,6 @@ let find_port server_name bind_addr port_option handler =
 let one_day = 3600. *. 24.
 let half_day = one_day /. 2.
 
-let printf_char c =
-  if !verbose then
-    (lprint_char c)
-
-let printf_string c =
-  if !verbose then
-    (lprint_string c)
-
 let minutes25 = 25 * 60
 
 let new_connection_control () = {
@@ -255,6 +177,9 @@ let connection_next_try cc =
 let connection_can_try cc =
   connection_next_try cc < last_time ()
 
+let connection_was_tried cc =
+  cc.control_last_try > 0
+
 let print_control c =
   lprintf_nl "Connection Control: ok = %d seconds ago, state = %d, last tried = %d seconds ago, delay = %d, next in %d seconds"
     (last_time () - c.control_last_ok) c.control_state (last_time () - c.control_last_try) c.control_min_reask (connection_next_try c - last_time ())
@@ -278,17 +203,24 @@ let upload_control = TcpBufferedSocket.create_write_bandwidth_controler
 
 let download_control = TcpBufferedSocket.create_read_bandwidth_controler
   "Download"
-    (!!max_hard_download_rate * 2048) (* TODO: changed from 1024 to 2048 because of bug *)
+    (!!max_hard_download_rate * 1024)
 
+let payload_bandwidth = ref 0.
 
 let _ =
   option_hook max_hard_upload_rate (fun _ ->
       TcpBufferedSocket.change_rate upload_control
-        (!!max_hard_upload_rate * 1024));
+        (!!max_hard_upload_rate * 1024);
+      payload_bandwidth :=
+	float_of_int (if !!max_hard_upload_rate = 0 then 
+	   10000 * 1024
+         else 
+	   maxi (!!max_hard_upload_rate * 1024) 1024) *. 0.90;
+  );
   option_hook max_hard_download_rate (fun _ ->
       let rate = !!max_hard_download_rate in
       TcpBufferedSocket.change_rate download_control
-        (rate * 2048)) (* TODO: changed from 1024 to 2048 because of bug *)
+        (rate * 1024))
 
 let udp_write_controler = UdpSocket.new_bandwidth_controler upload_control
 
@@ -507,8 +439,6 @@ let rec bprint_tags buf tags =
       (string_of_tag_value tag.tag_value);
       bprint_tags buf tags
 
-let aborted_download = ref (None : int option)
-
 (* let searches = ref ([] : search list) *)
 
 let core_included = ref false
@@ -537,109 +467,10 @@ let log_chat_message i num n s =
       (Printexc2.to_string e) !messages_log;
 
   while (Fifo.length chat_message_fifo) > !!html_mods_max_messages do
-    let foo = Fifo.take chat_message_fifo in ()
+    ignore(Fifo.take chat_message_fifo)
   done
 let last_message_log = ref 0
 
-let html_mods_commands buf n c l =
-    (* Name Class List *)
-    Printf.bprintf buf "\\<div class=\\\"%s\\\"\\>\\<table id=\\\"%s\\\"
-    name=\\\"%s\\\" class=\\\"%s\\\" cellspacing=0 cellpadding=0
-    width=\\\"100pc\\\"\\>\\<tbody\\>\\<tr\\>"
-    c n n c;
-    List.iter (fun (w,x,y,z)  ->
-     (* Class Title Onclick Value *)
-     Printf.bprintf buf "\\<td class=\\\"%s\\\"
-     title=\\\"%s\\\" onMouseOver=\\\"mOvr(this,'mOvr1');\\\" onMouseOut=\\\"mOut(this);\\\"
-     onClick=\\\"%s\\\" \\>%s\\</td\\>"
-     w x y z;
-    ) l;
-    Printf.bprintf buf "\\</tr\\>\\</tbody\\>\\</table\\>\\</div\\>"
-
-(*
-  html_mods_commands buf "commandsTable" "commands" [
-    ( "bu bbig", "Extend search to more servers and view results", "mSub('output','vr');", "Extend search" ) ;
-*)
-
-
-let html_mods_table_header buf n c l =
-    (* Name Class List *)
-    Printf.bprintf buf "\\<div class=\\\"%s\\\"\\>\\<table id=\\\"%s\\\" name=\\\"%s\\\" class=\\\"%s\\\" cellspacing=0 cellpadding=0\\>"
-    c n n c;
-    if List.length l > 0 then begin
-        Printf.bprintf buf "\\<tr\\>";
-        List.iter (fun (w,x,y,z)  ->
-         (* Sort Class Title Value *)
-         Printf.bprintf buf "\\<td onClick=\\\"_tabSort(this,%s);\\\" class=\\\"%s\\\" title=\\\"%s\\\"\\>%s\\</td\\>"
-         w x y z;
-        ) l;
-        Printf.bprintf buf "\\</tr\\>"
-      end
-(* Add colspan functionality to html_mods_table_header *)
-
-let html_mods_table_header_colspan buf n c l =
-    (* Name Class List *)
-    Printf.bprintf buf "\\<div class=\\\"%s\\\"\\>\\<table id=\\\"%s\\\" name=\\\"%s\\\" class=\\\"%s\\\" cellspacing=0 cellpadding=0\\>\\<tr\\>"
-    c n n c;
-    List.iter (fun (v,w,x,y,z)  ->
-     (* Sort Class Title Value *)
-     Printf.bprintf buf "\\<td colspan=%s onClick=\\\"_tabSort(this,%s);\\\" class=\\\"%s\\\" title=\\\"%s\\\"\\>%s\\</td\\>"
-     v w x y z;
-    ) l;
-    Printf.bprintf buf "\\</tr\\>"
-
-let html_mods_table_no_header buf n c l =
-    (* 1 row * n cols *)
-    (* Name Class List *)
-    Printf.bprintf buf "\\<div class=\\\"%s\\\"\\>\\<table id=\\\"%s\\\" name=\\\"%s\\\" class=\\\"%s\\\" cellspacing=0 cellpadding=0\\>\\<tr\\>"
-    c n n c;
-    List.iter (fun (t,c,d)  ->
-    (* Title Class Value *)
-     Printf.bprintf buf "\\<td class=\\\"%s\\\" %s\\>%s\\</td\\>"
-     c (if t <> "" then "title=\\\"" ^ t ^ "\\\"" else "") d;
-    ) l;
-    Printf.bprintf buf "\\</tr\\>\\</table\\>\\</div\\>"
-
-let html_mods_table_one_row buf n c l =
-    (* 1 row * n cols *)
-    (* Name Class List *)
-    Printf.bprintf buf "\\<div class=\\\"%s\\\"\\>\\<table id=\\\"%s\\\" name=\\\"%s\\\" class=\\\"%s\\\" cellspacing=0 cellpadding=0\\>\\<tr\\>"
-    c n n c;
-    List.iter (fun (t,c,d)  ->
-    (* Title Class Value *)
-     Printf.bprintf buf "\\<td class=\\\"%s\\\" %s\\>%s\\</td\\>"
-     c (if t <> "" then "title=\\\"" ^ t ^ "\\\"" else "") d;
-    ) l;
-    Printf.bprintf buf "\\</tr\\>\\</table\\>\\</div\\>"
-
-let html_mods_table_one_col buf n c l =
-    (* n rows * 1 col *)
-    (* Name Class List *)
-    Printf.bprintf buf "\\<div class=\\\"%s\\\"\\>\\<table id=\\\"%s\\\" name=\\\"%s\\\" class=\\\"%s\\\" cellspacing=0 cellpadding=0\\>\\<tr\\>"
-    c n n c;
-    List.iter (fun (t,c,d)  ->
-    (* Title Class Value *)
-     Printf.bprintf buf "\\<tr\\>\\<td class=\\\"%s\\\" %s\\>%s\\</td\\>\\</tr\\>"
-     c (if t <> "" then "title=\\\"" ^ t ^ "\\\"" else "") d;
-    ) l;
-    Printf.bprintf buf "\\</tr\\>\\</table\\>\\</div\\>"
-
-let html_mods_td buf l =
-    (* List *)
-    List.iter (fun (t,c,d)  ->
-    (* Title Class Value *)
-     Printf.bprintf buf "\\<td class=\\\"%s\\\" %s\\>%s\\</td\\>"
-     c (if t <> "" then "title=\\\"" ^ t ^ "\\\"" else "") d;
-    ) l
-
-let html_mods_counter = ref true
-
-let html_mods_cntr () =
-    html_mods_counter := not !html_mods_counter;
-    if !html_mods_counter then 1 else 2
-
-let html_mods_cntr_init () =
-  html_mods_counter := true
 
 
 let debug_clients = ref Intset.empty
@@ -694,22 +525,22 @@ let sd_udp_upload_rate = ref 0
 let sd_tcp_upload_rate = ref 0
 let sd_control_upload_rate = ref 0
 
-let bandwidth_samples = Fifo2.create ()
-let short_delay_bandwidth_samples = Fifo2.create ()
+let bandwidth_samples = Fifo.create ()
+let short_delay_bandwidth_samples = Fifo.create ()
 
 let nmeasures = 6
 let dummy_sample = Array.make nmeasures 0.
 
 let trimto n samples =
-  let len = ref (Fifo2.length samples) in
+  let len = ref (Fifo.length samples) in
   while !len > n do
-    ignore (Fifo2.take samples);
+    ignore (Fifo.take samples);
     decr len
   done
 
 let last samples =
   try
-    Fifo2.read samples
+    Fifo.head samples
   with _ ->
       (last_time (), dummy_sample)
 
@@ -726,7 +557,7 @@ let update_link_stats () =
 
   let put time sample samples =
     assert (Array.length sample = nmeasures);
-    Fifo2.put samples (time, sample) in
+    Fifo.put samples (time, sample) in
 
   let last_count_time, last_sample =
     last bandwidth_samples in
@@ -774,10 +605,10 @@ let update_link_stats () =
 let history_size = 720
 let history_h_size = 720
 
-let upload_history = Fifo2.create ()
-let download_history = Fifo2.create ()
-let upload_h_history = Fifo2.create ()
-let download_h_history = Fifo2.create ()
+let upload_history = Fifo.create ()
+let download_history = Fifo.create ()
+let upload_h_history = Fifo.create ()
+let download_h_history = Fifo.create ()
 
 let upload_usage () =
   !udp_upload_rate + !control_upload_rate
@@ -789,45 +620,45 @@ let download_usage () =
   !udp_download_rate + !control_download_rate
 
 let update_download_history () =
-  Fifo2.put download_history (download_usage ());
-  let len = ref (Fifo2.length download_history) in
+  Fifo.put download_history (download_usage ());
+  let len = ref (Fifo.length download_history) in
   while !len > history_size do
-    ignore (Fifo2.take download_history);
+    ignore (Fifo.take download_history);
     decr len
   done
 
 let update_upload_history () =
-  Fifo2.put upload_history (upload_usage ());
-  let len = ref (Fifo2.length upload_history) in
+  Fifo.put upload_history (upload_usage ());
+  let len = ref (Fifo.length upload_history) in
   while !len > history_size do
-    ignore (Fifo2.take upload_history);
+    ignore (Fifo.take upload_history);
     decr len
   done
 
 let update_h_download_history () =
-  Fifo2.put download_h_history ((List.fold_left (+) 0 (Fifo2.to_list download_history)) / ((Fifo2.length download_history)));
-  let len = ref (Fifo2.length download_h_history) in
+  Fifo.put download_h_history ((List.fold_left (+) 0 (Fifo.to_list download_history)) / ((Fifo.length download_history)));
+  let len = ref (Fifo.length download_h_history) in
   while !len > history_h_size do
-    ignore (Fifo2.take download_h_history);
+    ignore (Fifo.take download_h_history);
     decr len
   done
 
 let update_h_upload_history () =
-  Fifo2.put upload_h_history ((List.fold_left (+) 0 (Fifo2.to_list upload_history)) / ((Fifo2.length upload_history)));
-  let len = ref (Fifo2.length upload_h_history) in
+  Fifo.put upload_h_history ((List.fold_left (+) 0 (Fifo.to_list upload_history)) / ((Fifo.length upload_history)));
+  let len = ref (Fifo.length upload_h_history) in
   while !len > history_h_size do
-    ignore (Fifo2.take upload_h_history);
+    ignore (Fifo.take upload_h_history);
     decr len
   done
 
 let detected_link_capacity link =
-  List.fold_left maxi 0 (Fifo2.to_list link)
+  List.fold_left maxi 0 (Fifo.to_list link)
 
 let detected_uplink_capacity () =
-  List.fold_left maxi 0 (Fifo2.to_list upload_history)
+  List.fold_left maxi 0 (Fifo.to_list upload_history)
 
 let detected_downlink_capacity () =
-  List.fold_left maxi 0 (Fifo2.to_list download_history)
+  List.fold_left maxi 0 (Fifo.to_list download_history)
 
 
 let new_tag name v =
@@ -879,11 +710,6 @@ let for_string_tag tag f =
   | Pair _ -> ()
   | Uint16 _ | Uint8 _ -> ()
 
-(* Name,FrameHeight *)
-let html_mods_styles = ref
-  [| ("Green",46) ; ("Tang",46); ("L.Blue",46);
-   ("L.Purple",46); ("Monochrome",46); ("Corona",46); ("Coronax",46); ("Construction",46); |]
-
 let partial_chunk c =
   match c with
     '0' | '1' -> true
@@ -1015,9 +841,6 @@ module CanBeCompressed = struct
   end
     *)
 
-let ip_reachable ip =
-  !!allow_local_network || Ip.reachable ip
-
 let do_if_connected tcp_connection f =
   match tcp_connection with
     Connection sock -> f sock
