@@ -20,6 +20,7 @@
 open AnyEndian
 open LittleEndian
 open Int64ops
+open Md4
 open Misc
 open Printf2
 open CommonOptions
@@ -31,7 +32,6 @@ open CommonClient
 open CommonServer
 open CommonNetwork
 open CommonOptions
-open CommonTypes
 open CommonFile
 open CommonGlobals
 open CommonSearch
@@ -40,6 +40,16 @@ open CommonServer
 open CommonTypes
 open CommonComplexOptions
 
+(* prints a new logline with date, module and starts newline *)
+let lprintf_nl () =
+  lprintf "%s[cInt] "
+    (log_time ()); lprintf_nl2
+
+(* prints a new logline with date, module and does not start newline *)
+let lprintf_n () =
+  lprintf "%s[cInt] "
+    (log_time ()); lprintf
+
 (*************  ADD/REMOVE FUNCTIONS ************)
 let check_forbidden_chars (uc : Charset.uchar) =
   match uc with
@@ -53,7 +63,7 @@ let check_forbidden_chars (uc : Charset.uchar) =
   | 60  (* '<'  *)
   | 62  (* '>'  *)
   | 124 (* '|'  *)
-  | 37  (* '%'  *) when Autoconf.system = "windows" -> 95 (* '_' *)
+  | 37  (* '%'  *) when Autoconf.windows -> 95 (* '_' *)
   | _ -> uc
 
 let canonize_basename name =
@@ -75,10 +85,12 @@ let canonize_basename name =
         Charset.add_uchar buf uc'
       end
   done;
-  Charset.to_locale (Buffer.contents buf)
+  if Autoconf.windows then
+    Charset.to_locale (Buffer.contents buf)
+  else
+    Buffer.contents buf
 
 let file_commited_name incoming_dir file =
-  let network = file_network file in
   let best_name = file_best_name file in
   (try Unix2.safe_mkdir incoming_dir with _ -> ());
   let new_name =
@@ -96,6 +108,45 @@ let file_commited_name incoming_dir file =
     else new_name in
   new_name
 
+let script_for_file file incoming new_name =
+  let info = file_info file in
+  let temp_name = file_disk_name file in
+  let file_id = Filename.basename temp_name in
+  let size = Int64.to_string (file_size file) in
+  let duration =
+    string_of_int ((BasicSocket.last_time ()) - info.G.file_age)
+  in
+  let network = network_find_by_num info.G.file_network in
+  let filename = Filename.basename new_name in
+  let ed2k_hash =
+    if info.G.file_md4 = Md4.null then "" else
+       Printf.sprintf "ed2k://|file|%s|%s|%s|/"
+        (Url.encode filename)
+        (Int64.to_string (file_size file))
+        (Md4.to_string info.G.file_md4)
+  in
+  begin try
+  MlUnix.fork_and_exec !!file_completed_cmd
+      [|  (* keep those for compatibility *)
+      "";
+      file_id; (* $1 *)
+      size; (* $2 *)
+      filename (* $3 *)
+    |]
+     ~vars:[("TEMPNAME",  temp_name);
+	    ("FILEID",    file_id);
+	    ("FILESIZE",  size);
+	    ("FILENAME",  filename);
+	    ("FILEHASH",  string_of_uids info.G.file_uids);
+	    ("DURATION",  duration);
+	    ("INCOMING",  incoming);
+	    ("NETWORK",   network.network_name);
+	    ("ED2K_HASH", ed2k_hash)]
+  with e -> 
+      lprintf_nl () "Exception %s while executing %s"
+        (Printexc2.to_string e) !!file_completed_cmd
+  end
+
 (********
 These two functions 'file_commit' and 'file_cancel' should be the two only
 functions in mldonkey able to destroy a file, the first one by moving it,
@@ -124,12 +175,17 @@ let file_commit file =
         if Unix2.is_directory file_name then
           Unix2.safe_mkdir new_name;
         (try
+(*          the next line really moves the file *)
             set_file_disk_name file new_name;
+
+            if !!file_completed_cmd <> "" then
+	      script_for_file file incoming.shdir_dirname new_name;
+
             if Unix2.is_directory new_name then
-              Unix.chmod new_name (Misc.int_of_octal_string !!create_dir_mask);
+              Unix2.chmod new_name (Misc.int_of_octal_string !!create_dir_mask);
             let best_name = file_best_name file in
             Unix32.destroy (file_fd file);
-            if !verbose_files then lprintf_nl "[cInt] file_commit: destroyed";
+            if !verbose then lprintf_nl () " file_commit: destroyed";
             if Unix2.is_directory file_name then Unix2.remove_all_directory file_name;
             let impl = as_file_impl file in
 
@@ -144,7 +200,7 @@ anymore. *)
                       incoming.shdir_dirname incoming.shdir_priority
                       best_name new_name);
               with e ->
-                  lprintf_nl "[cInt] Exception %s while trying to share commited file"
+                  lprintf_nl () "Exception %s while trying to share commited file"
                     (Printexc2.to_string e);
             end;
 
@@ -152,7 +208,7 @@ anymore. *)
             done_files =:= List2.removeq file !!done_files;
             files =:= List2.removeq file !!files;
 
-            if !verbose_files then lprintf_nl "[cInt]file_commit: going to secondaries...";
+            if !verbose then lprintf_nl () "file_commit: going to secondaries...";
             List.iter (fun file ->
 (* Commit the file first, and share it after... *)
                 try
@@ -163,10 +219,10 @@ anymore. *)
                   files =:= List2.removeq file !!files;
 
                 with e ->
-                    lprintf_nl "[cInt] Exception %s in file_commit secondaries" (Printexc2.to_string e);
+                    lprintf_nl () "Exception %s in file_commit secondaries" (Printexc2.to_string e);
             ) secondary_files
           with e ->
-              lprintf_nl "[cInt] Exception in file_commit: %s" (Printexc2.to_string e))
+              lprintf_nl () "Exception in file_commit: %s" (Printexc2.to_string e))
     | _ -> assert false
 
 let file_cancel file =
@@ -182,31 +238,20 @@ let file_cancel file =
             impl.impl_file_ops.op_file_cancel impl.impl_file_val;
             files =:= List2.removeq file !!files;
           with e ->
-              lprintf_nl "[cInt] Exception %s in file_cancel" (Printexc2.to_string e);
+              lprintf_nl () "Exception %s in file_cancel" (Printexc2.to_string e);
       ) subfiles;
-      (try
-          let fd = file_fd file in
-          if fd != Unix32.bad_fd then Unix32.remove (file_fd file)
-        with e ->
-            lprintf_nl "[cInt]Sys.remove %s exception %s"
-              (file_disk_name file)
-            (Printexc2.to_string e); );
-      Unix32.destroy (file_fd file);
+      try
+	let fd = file_fd file in
+	(try
+           Unix32.remove fd
+         with e ->
+           lprintf_nl () "Sys.remove %s exception %s"
+             (file_disk_name file)
+             (Printexc2.to_string e));
+	Unix32.destroy fd
+      with Not_found -> ()
   with e ->
-      lprintf_nl "[cInt] Exception in file_cancel: %s" (Printexc2.to_string e)
-
-let time_to_string time =
-  let days = time / 60 / 60 / 24 in
-  let rest = time - days * 60 * 60 * 24 in
-  let hours = rest / 60 / 60 in
-  let rest = rest - hours * 60 * 60 in
-  let minutes = rest / 60 in
-  let seconds = rest - minutes * 60 in
-    if days > 0
-    then Printf.sprintf " %dd " days
-    else if hours > 0
-    then Printf.sprintf " %dh %dm " hours minutes
-    else Printf.sprintf " %dm " minutes
+      lprintf_nl () "Exception in file_cancel: %s" (Printexc2.to_string e)
 
 let mail_for_completed_file file =
   if !!mail <> "" then
@@ -214,43 +259,49 @@ let mail_for_completed_file file =
     let info = file_info file in
     let line1 = "mldonkey has completed the download of:\r\n\r\n" in
 
-    let line2 = Printf.sprintf "\r\nFile: %s\r\nSize: %Ld bytes\r\nHash: %s\r\nFile was downloaded in%s\r\n"
+    let line2 = Printf.sprintf "\r\nFile: %s\r\nSize: %Ld bytes\r\nHash: %s\r\nFile was downloaded in %s\r\n"
       (file_best_name file)
       (file_size file)
       (string_of_uids info.G.file_uids)
-      (let age = (BasicSocket.last_time ()) - info.G.file_age in time_to_string age)
+      (let age = (BasicSocket.last_time ()) - info.G.file_age in Date.time_to_string age "verbose")
     in
 
-    let line3 = if (file_comment file) <> "" then
+    let line3 = if (file_comment file) = "" then "" else
         Printf.sprintf "\r\nComment: %s\r\n" (file_comment file)
-      else
-        Printf.sprintf "";
     in
 
     let subject = if !!filename_in_subject then
-        Printf.sprintf "[mldonkey] file received - %s"
-        (file_best_name file)
+        Printf.sprintf "[mldonkey] file received - %s" (file_best_name file)
       else
-        Printf.sprintf "mldonkey, file received";
+        Printf.sprintf "mldonkey, file received"
     in
 
-    let line4 = if !!url_in_mail <> "" then
-        Printf.sprintf "\r\n<%s/%s>\r\n" !!url_in_mail (Url.encode (file_best_name file))
+    let incoming =
+      if Unix2.is_directory (file_disk_name file) then
+        incoming_directories ()
       else
-        Printf.sprintf "";
+        incoming_files ()
+    in
+
+    let line4 = if !!url_in_mail = "" then "" else
+      Printf.sprintf "\r\n<%s/%s/%s>\r\n" !!url_in_mail incoming.shdir_dirname (Url.encode (file_best_name file))
+    in
+
+    let line5 = if !!auto_commit then "" else
+        Printf.sprintf "\r\nauto_commit is disabled, file is not committed to incoming"
     in
 
     let mail = {
         M.mail_to = !!mail;
         M.mail_from = !!mail;
         M.mail_subject = subject;
-        M.mail_body = line1 ^ line2 ^ line3 ^ line4;
+        M.mail_body = line1 ^ line2 ^ line3 ^ line4 ^ line5;
       } in
     M.sendmail !!smtp_server !!smtp_port !!add_mail_brackets mail
 
 let chat_for_completed_file file =
   CommonChat.send_warning_for_downloaded_file (file_best_name file)
-
+  
 let file_completed (file : file) =
   try
     let impl = as_file_impl file in
@@ -259,27 +310,17 @@ let file_completed (file : file) =
         done_files =:= file :: !!done_files;
         update_file_state impl FileDownloaded;
         let file_name = file_disk_name file in
-        let file_id = Filename.basename file_name in
         ignore (CommonShared.new_shared "completed" 0 (
             file_best_name file ) file_name);
         (try mail_for_completed_file file with e ->
-              lprintf_nl "[cInt] Exception %s in sendmail" (Printexc2.to_string e);
+              lprintf_nl () "Exception %s in sendmail" (Printexc2.to_string e);
               );
         if !!CommonOptions.chat_warning_for_downloaded then
           chat_for_completed_file file;
 
-        if !!file_completed_cmd <> "" then begin
-            MlUnix.fork_and_exec  !!file_completed_cmd
-              [|
-              file_name;
-              file_id;
-              Int64.to_string (file_size file);
-              file_best_name file
-            |]
-          end
       end
   with e ->
-      lprintf_nl "[cInt] Exception in file_completed: %s" (Printexc2.to_string e)
+      lprintf_nl () "Exception in file_completed: %s" (Printexc2.to_string e)
 
 let file_add impl state =
   try
@@ -301,19 +342,24 @@ let file_add impl state =
         update_file_state impl state
       end
   with e ->
-      lprintf_nl "[cInt] Exception in file_add: %s" (Printexc2.to_string e)
+      lprintf_nl () "[cInt] Exception in file_add: %s" (Printexc2.to_string e)
 
 let server_remove server =
+  begin
+    match server_state server with
+      NotConnected _ -> ()
+    | _ -> server_disconnect server
+  end;
   try
     let impl = as_server_impl server in
     if impl.impl_server_state <> RemovedHost then begin
         set_server_state server RemovedHost;
         (try impl.impl_server_ops.op_server_remove impl.impl_server_val
           with _ -> ());
-        servers =:= Intmap.remove (server_num server) !!servers;
+        servers =:= Intmap.remove (server_num server) !!servers
       end
   with e ->
-      lprintf_nl "[cInt] Exception in server_remove: %s" (Printexc2.to_string e)
+      lprintf_nl () "[cInt] Exception in server_remove: %s" (Printexc2.to_string e)
 
 let server_add impl =
   let server = as_server impl in
@@ -352,7 +398,7 @@ let friend_remove c =
       end
 
   with e ->
-      lprintf_nl "[cInt] Exception in friend_remove: %s" (Printexc2.to_string e)
+      lprintf_nl () "Exception in friend_remove: %s" (Printexc2.to_string e)
 
 let contact_add c =
   let impl = as_client_impl c in
@@ -373,8 +419,24 @@ let contact_remove c =
         impl.impl_client_ops.op_client_clear_files impl.impl_client_val
       end
   with e ->
-      lprintf_nl "[cInt] Exception in contact_remove: %s" (Printexc2.to_string e)
+      lprintf_nl () "Exception in contact_remove: %s" (Printexc2.to_string e)
 
+let exit_counter = ref 0
+let exit_timer = ref false
+
+let rec clean_exit n =
+  let can_exit = networks_for_all (fun n -> network_clean_exit n) in
+  if can_exit || (!exit_counter > !!shutdown_timeout) then 
+    exit_properly n
+  else 
+    if not !exit_timer then begin
+      exit_timer := true;
+      add_timer 1. (fun _ ->
+        incr exit_counter;
+        exit_timer := false;
+        clean_exit n;
+      );
+    end
 
 let time_of_sec sec =
   let hours = sec / 60 / 60 in
@@ -566,7 +628,7 @@ let send_custom_query user buf query args =
           if maxsize = "" then raise Not_found;
           let maxsize = Int64.of_string maxsize in
           let unit = Int64.of_string unit in
-          QHasMaxVal (Field_Size, Int64.mul maxsize unit)
+          QHasMaxVal (Field_Size, maxsize ** unit)
 
       | Q_FORMAT _ ->
           let format = get_arg "format" in
@@ -700,7 +762,7 @@ let all_active_network_opfile_network_names () =
        !names
 
 let apply_on_fully_qualified_options name f =
-  if !verbose then lprintf_nl "[cInt]Change option %s" name;
+  if !verbose then lprintf_nl () "Change option %s" name;
   let rec iter prefix opfile =
     let args = simple_options prefix opfile in
     List.iter (fun o ->
@@ -723,7 +785,7 @@ let apply_on_fully_qualified_options name f =
               false
             with Exit -> true
         )) then begin
-        lprintf_nl "[cInt]Could not set option %s" name;
+        lprintf_nl () "Could not set option %s" name;
         raise Not_found
       end
   with Exit -> ()
@@ -781,7 +843,7 @@ let keywords_of_query query =
           | _ -> ()
         end
     | QNone ->
-        lprintf_nl "[cInt]start_search: QNone in query";
+        lprintf_nl () "start_search: QNone in query";
         ()
   in
   iter query;
