@@ -29,13 +29,14 @@ open Unix
 let bin_dir = Filename.dirname Sys.argv.(0)
 
 let hidden_dir_prefix =
-  if Autoconf.system = "windows" then "" else "."
+  if Autoconf.windows then "" else "."
 
 let config_dir_basename = hidden_dir_prefix ^ "mldonkey"
 
 let home_dir =
   match Autoconf.system with
-  | "windows" -> "."
+  | "cygwin"
+  | "mingw" -> "."
   | _ -> Filename.concat (try Sys.getenv "HOME" with _ -> ".") config_dir_basename
 
 let installer_ini = create_options_file (Filename.concat home_dir
@@ -85,7 +86,7 @@ let file_basedir =
   if Sys.file_exists (Filename.concat (Sys.getcwd ()) "downloads.ini") then
     "."
   else
-    if Autoconf.system = "windows" && file_basedir_pre <> home_dir then
+    if Autoconf.windows && file_basedir_pre <> home_dir then
       match String2.split file_basedir_pre ':' with
       | drive :: directory :: _ ->
           Unix.chdir (drive ^ ":\\");
@@ -93,12 +94,27 @@ let file_basedir =
       | _ -> lprintf "Please provide an absolute path in MLDONKEY_DIR like d:\\mldonkey, exiting...\n"; exit 2
     else file_basedir_pre
 
+let exit_message file = Printf.sprintf
+"\nThis means another MLDonkey process could still be working
+in this directory. Please shut it down before starting
+a new instance here. If you are sure no other process uses
+this directory delete %s and restart the core.\n" file
+
+let exit_message_dev file = Printf.sprintf
+"\n/dev/%s does not exist, please create it, exiting...
+If you are using a chroot environment, create it inside the chroot.\n" file
+
 let _ =
   lprintf_nl "Starting MLDonkey %s ... " Autoconf.current_version;
-  lprintf_nl "Language %s, locale %s"
-    Charset.default_language Charset.locstr;
+  lprintf_nl "Language %s, locale %s, ulimit for open files %d"
+    Charset.default_language Charset.locstr (Unix2.c_getdtablesize ());
   lprintf_nl "MLDonkey is working in %s" file_basedir;
 
+  let ulof = Unix2.c_getdtablesize () in
+  if ulof < 150 then begin
+    lprintf_nl "ulimit for open files is set to %d, at least 150 is required, exiting..." ulof;
+    exit 2
+  end;
   (try
      Unix2.safe_mkdir file_basedir
    with e ->
@@ -108,16 +124,73 @@ let _ =
   Unix2.can_write_to_directory file_basedir;
   Unix.chdir file_basedir;
 
-  if (String2.starts_with (Filename.basename Sys.argv.(0))  "mlnet")
-    then if Sys.file_exists "mlnet.pid"
+  if (String2.starts_with (Filename.basename Sys.argv.(0)) "mlnet")
+    && not Autoconf.windows && not (Autoconf.system = "morphos") && not (Sys.file_exists "/dev/urandom") then begin
+      lprintf "%s" (exit_message_dev "urandom");
+      if Autoconf.system = "hpux" then
+        lprintf_nl "For HP-UX get urandom support from http://www.josvisser.nl/hpux11-random";
+      exit 2
+    end;  
+
+  if (String2.starts_with (Filename.basename Sys.argv.(0)) "mlnet")
+    && not Autoconf.windows && not (Sys.file_exists "/dev/null") then begin
+      lprintf "%s" (exit_message_dev "null");
+      exit 2
+    end;  
+
+  Unix2.can_write_to_directory (Filename2.temp_directory ());
+
+  if (String2.starts_with (Filename.basename Sys.argv.(0)) "mlnet")
+    then
+      let pid_filename =
+        Printf.sprintf "%s.pid" (Filename.basename Sys.argv.(0)) in
+      let config_space = "config_files_space.tmp" in
+      if Sys.file_exists pid_filename || Sys.file_exists config_space
       then begin
-        lprintf_nl "\n\n\nPID file mlnet.pid exists in %s"
-        file_basedir;
-        lprintf_nl "This means another MLDonkey process could still be working";
-        lprintf_nl "with this directory. Please shut it down before starting";
-        lprintf_nl "a new instance here. If you are sure no other process uses";
-        lprintf_nl "this directory delete mlnet.pid and restart the core.";
-        exit 2
+        if Sys.file_exists pid_filename then
+          lprintf_nl "PID file %s exists."
+	    (Filename.concat file_basedir pid_filename)
+	else
+	  if Sys.file_exists config_space then begin
+            lprintf_nl "%s exists." (Filename.concat file_basedir config_space);
+	    lprintf "%s" (exit_message config_space);
+	    if Autoconf.windows then
+	      begin
+	        lprintf_nl "waiting 10 seconds to exit...";
+		Unix.sleep 10
+	      end;
+	    exit 2
+	    end;
+        let pid =
+          try
+            let pid_ci = open_in pid_filename in
+            let pid = int_of_string (input_line pid_ci) in
+            close_in pid_ci;
+            pid
+          with _ ->
+            lprintf_nl "But it couldn't be read to check if the process still exists.";
+            lprintf_nl "To avoid doing any harm, MLDonkey will now stop.";
+            exit 2
+        in
+          try
+            Unix.kill pid 0;
+	    lprintf "%s" (exit_message pid_filename);
+            exit 2
+          with
+            (* stalled pid file, disregard it *)
+            | Unix.Unix_error (Unix.ESRCH, _, _) ->
+	       (lprintf_nl "Removing stalled file %s " pid_filename;
+	       try Sys.remove pid_filename with _ -> ())
+	    | e -> 
+	      lprintf "%s" (exit_message pid_filename);
+	      if Autoconf.system = "mingw" then lprintf_nl
+	        "can not check for stalled pid file because Unix.kill is not implemented on MinGW";
+	      lprintf_nl "Exception %s, exiting..." (Printexc2.to_string e);
+	      if Autoconf.system = "mingw" then begin
+	        lprintf_nl "waiting 10 seconds to exit...";
+		Unix.sleep 10;
+	      end;
+	      exit 2
       end;
 
   let filename =
@@ -180,6 +253,8 @@ let empty_password user =
 
 let ip_list_option = list_option Ip.option
 
+let int_list_option = list_option int_option
+
 let allow_browse_share_option = define_option_class "Integer"
     (fun v ->
       match v with
@@ -205,6 +280,16 @@ let _ =
   (fun s ->
       let list = String2.tokens s in
       List.map (fun ip -> Ip.of_string ip) list
+  );
+  Options.set_string_wrappers int_list_option
+    (fun list ->
+      List.fold_left (fun s i ->
+          Printf.sprintf "%s %s" (string_of_int i) s
+      ) "" (List.rev list)
+  )
+  (fun s ->
+      let list = String2.tokens s in
+      List.map (fun i -> int_of_string i) list
   )
 
 let is_not_spam = ref (fun _ -> true)
@@ -288,7 +373,8 @@ let allowed_ips = define_option current_section ["allowed_ips"]
   "list of IP address allowed to connect to the core via telnet/GUI/WEB
 for internal command set: list separated by spaces
 example for internal command: set allowed_ips \"127.0.0.1 192.168.1.2\"
-or for editing the ini-file: allowed_ips = [ \"127.0.0.1\"; \"192.168.1.2\";]
+or for editing the ini-file: list separated by semi-colon
+example for ini-file: allowed_ips = [ \"127.0.0.1\"; \"192.168.1.2\";]
 wildcard=255 ie: use 192.168.0.255 for 192.168.0.*"
     ip_list_option [Ip.localhost]
 
@@ -340,7 +426,37 @@ may have spaces (put it in quotation then)."
     "exit", "q";
   ]
 
-
+let verbosity = define_expert_option current_section ["verbosity"]
+  "A space-separated list of keywords. Each keyword triggers
+  printing information on the corresponding messages:
+  verb : verbose mode (interesting not only for coders)
+  mc : debug client messages
+  mr|raw : debug raw messages
+  mct : debug emule clients tags
+  ms : debug server messages
+  sm : debug source management
+  net : debug net
+  gui : debug gui
+  file : debug file handling
+  do : some download warnings
+  up : some upload warnings
+  unk : unknown messages
+  ov : overnet
+  loc : debug source research/master servers
+  share: debug sharing
+  md4 : md4 computation
+  connect : debug connections
+  udp : udp messages
+  ultra|super : debug supernode
+  swarming : debug swarming
+  hc : http_client messages
+  hs : http_server messages
+  act : debug activity
+  bw : debug bandwidth
+  redir : debug redirector
+  unexp : debug unexpected messages
+"
+    string_option ""
 
 
 (*************************************************************************)
@@ -361,16 +477,23 @@ let max_hard_download_rate = define_option current_section ["max_hard_download_r
   The limit will apply on all your connections (clients and servers) and both
 control and data messages." int_option 50
 
-
 let max_opened_connections = define_option current_section
     ["max_opened_connections"] "Maximal number of opened connections"
-  int_option (min MlUnix.max_sockets 200)
+  int_option 200
 
+let max_indirect_connections = define_option current_section
+    ["max_indirect_connections"]
+  "Amount of indirect connections in percent (min 30, max 70) of max_opened_connections"
+  int_option 20
 
 let max_upload_slots = define_option current_section ["max_upload_slots"]
     "How many slots can be used for upload"
     int_option 5
 
+let friends_upload_slot = define_option current_section ["friends_upload_slot"]
+    "Set aside a single reserved slot to upload to friends"
+    bool_option true
+
 let dynamic_slots = define_option current_section ["dynamic_slots"]
     "Set this to true if you want to have dynamic upload slot allocation (experimental)" bool_option false
 
@@ -383,7 +506,7 @@ let loop_delay = define_expert_option current_section
   ["loop_delay"]
 "The delay in milliseconds to wait in the event loop. Can be decreased to
 increase the bandwidth usage, or increased to lower the CPU usage."
-  int_option 5
+  int_option 20
 
 let nolimit_ips = define_option current_section ["nolimit_ips"]
     ~desc: "No-limit IPs"
@@ -621,7 +744,7 @@ let html_vd_chunk_graph_style = define_expert_option current_section
     ["html_vd_chunk_graph_style"] "Change style of chunk graph" int_option 0
 
 let html_vd_chunk_graph_max_width = define_expert_option current_section
-    ["html_vd_chunk_graph_max_width"] "Change max width of chunk graph" int_option 880
+    ["html_vd_chunk_graph_max_width"] "Change max width of chunk graph" int_option 200
 
 let html_mods_show_pending = define_expert_option current_section
     ["html_mods_show_pending"] "Whether to display the pending slots in uploaders command" bool_option true
@@ -678,21 +801,23 @@ let force_client_ip = define_option current_section ["force_client_ip"]
     ourself. Don't set this option to true if you have dynamic IP."
     bool_option false
 
-let web_infos = define_option current_section
-    ["web_infos"] "A list of lines to download on the WEB: each line has
+let web_infos = define_option current_section ["web_infos"]
+    "A list of lines to download on the WEB: each line has
     the format: (kind, period, url), where kind is either
-    'server.met' for a server.met file containing ed2k server, or
+    'server.met' for a server.met file (also in gz/bz2/zip format)
+                 containing ed2k server, or
     'comments.met' for a file of comments, or
     'guarding.p2p' for a blocklist file (also in gz/bz2/zip format), or
     'ocl' for file in the ocl format containing overnet peers, or
     'contact.dat' for an contact.dat file containing overnet peers,
     and period is the period between updates (in hours),
+    a period of zero means the file is only loaded once on startup,
     and url is the url of the file to download.
     IMPORTANT: Put the URL and the kind between quotes.
     EXAMPLE:
  web_infos = [
-  (\"server.met\", 96, \"http://www.primusnet.ch/users/komintern/ed2k/min/server.met\");
-  (\"guarding.p2p\", 96, \"http://homepage.ntlworld.com/tim.leonard1/guarding.p2p\");
+  (\"server.met\", 0, \"http://www.gruk.org/server.met.gz\");
+  (\"guarding.p2p\", 96, \"http://www.bluetack.co.uk/config/antip2p.txt\");
   (\"ocl\", 24, \"http://members.lycos.co.uk/appbyhp2/FlockHelpApp/contact-files/contact.ocl\");
   (\"contact.dat\", 168, \"http://download.overnet.org/contact.dat\");
  ]
@@ -702,27 +827,37 @@ let web_infos = define_option current_section
   [
     ("guarding.p2p", 96,
       "http://www.bluetack.co.uk/config/antip2p.txt");
-    ("server.met", 24,
-      "http://ocbmaurice.dyndns.org/pl/slist.pl/server.met?download/server-best.met");
+    ("server.met", 0,
+      "http://www.gruk.org/server.met.gz");
     ("contact.dat", 168,
       "http://download.overnet.org/contact.dat");
+    ("geoip.dat", 0,
+      "http://www.maxmind.com/download/geoip/database/GeoIP.dat.gz");
   ]
 
 let rss_feeds = define_expert_option current_section ["rss_feeds"]
     "URLs of RSS feeds"
     (list_option Url.option) []
 
+let ip_blocking_descriptions =
+  define_expert_option current_section ["ip_blocking_descriptions"] "Keep IP blocking ranges descriptions in memory" bool_option true
+
 let ip_blocking = define_expert_option current_section ["ip_blocking"]
     "IP blocking list filename (peerguardian format), can also be in gz/bz2/zip format
   Zip files must contain either a file named guarding.p2p or guarding_full.p2p."
     string_option ""
 
+let geoip_dat = define_expert_option current_section ["geoip_dat"]
+    "Location of GeoIP.dat (Get one from http://www.maxmind.com/download/geoip/database/)"
+    string_option ""
+
 let _ =
-  option_hook ip_blocking (fun _ ->
+  option_hook ip_blocking_descriptions (fun _ ->
+    Ip_set.store_blocking_descriptions := !!ip_blocking_descriptions
+  );
+  option_hook geoip_dat (fun _ ->
     try
-      Ip_set.bl := if !!ip_blocking <> "" then
-      	             Ip_set.load !!ip_blocking
-        	   else Ip_set.bl_empty
+      Geoip.init (Geoip.unpack !!geoip_dat);
     with _ -> ()
   )
 
@@ -847,7 +982,12 @@ let emulate_sparsefiles = define_expert_option current_section
 let max_concurrent_downloads = define_option current_section
     ["max_concurrent_downloads"]
   "The maximal number of files in Downloading state (other ones are Queued)"
-    int_option 60
+    int_option 50
+
+let sources_per_chunk =
+  define_expert_option current_section ["sources_per_chunk"]
+    "How many sources to use to download each chunk"
+    int_option 3
 
   (*
 let delete_original = define_option current_section ["delete_original"]
@@ -858,11 +998,24 @@ let delete_original = define_option current_section ["delete_original"]
 let max_recover_gap = define_option current_section ["max_recover_zeroes_gap"]
   "The maximal length of zero bytes between non-zero bytes in a file that
 should be interpreted as downloaded during a recovery"
-    int64_option (Int64.of_int 16)
+    int64_option 16L
 
 let file_completed_cmd = define_option current_section
-    ["file_completed_cmd"] "A command that is called when a file is completely
-    downloaded. Arguments are: <file_name on disk> <md4> <size>"
+    ["file_completed_cmd"] "A command that is called when a file is committed, does not work on MinGW.
+    Arguments are (kept for compatability):
+      $1 - temp file name, without path
+      $2 - file size
+      $3 - filename of the committed file
+    Also these environment variables can be used (prefered way):
+      $TEMPNAME  - temp file name, including path
+      $FILEID    - same as $1
+      $FILESIZE  - same as $2
+      $FILENAME  - same as $3
+      $FILEHASH  - internal hash
+      $DURATION  - download duration
+      $INCOMING  - directory used for commit
+      $NETWORK   - network used for downloading
+      $ED2K_HASH - ed2k hash if MD4 is known"
     string_option ""
 
 let file_started_cmd = define_option current_section
@@ -937,6 +1090,10 @@ let create_dir_mask = define_option current_section ["create_dir_mask"]
     "New directories in incoming_directories are created with these rights"
     string_option "755"
 
+let create_file_sparse = define_option current_section ["create_file_sparse"]
+    "Create new files as sparse, only valid on MinGW for files on NTFS drives"
+    bool_option true
+
 let previewer = define_expert_option current_section ["previewer"]
   "Name of program used for preview (first arg is local filename, second arg
     is name of file as searched on eDonkey" string_option
@@ -977,10 +1134,10 @@ ones in allowed_commands"
     bool_option false
 
 let allow_browse_share = define_option current_section ["allow_browse_share"]
-  "Allow others to browse our share list (0: none, 1: friends only, 2: everyone" allow_browse_share_option 2
+  "Allow others to browse our share list (0: none, 1: friends only, 2: everyone" allow_browse_share_option 1
 
 let messages_filter = define_option current_section ["messages_filter"]
-    "Regexp of messages to filter out, example: string1\\|string2\\|string3" string_option "Your client is connecting too fast"
+    "Regexp of messages to filter out, example: string1|string2|string3" string_option "Your client is connecting too fast"
 
 
 
@@ -1077,6 +1234,10 @@ let backup_options_format = define_option current_section ["backup_options_forma
     "Define the format of the archive, zip or tar.gz are valid."
     string_option "tar.gz"
 
+let shutdown_timeout = define_option current_section ["shutdown_timeout"]
+    "The maximum time in seconds to wait for networks to cleanly shutdown."
+    int_option 3
+
   (*
 let password = define_option current_section ["password"]
   "The password to access your client from the GUI (setting it disables
@@ -1114,9 +1275,6 @@ let utf8_filename_conversions = define_expert_option current_section
     "The conversions to apply on Unicode characters"
     (list_option (tuple2_option (int_option, utf8_option))) []
 
-let client_timeout = define_expert_option current_section ["client_timeout"]
-  "Timeout on client connections when not queued" float_option 40.
-
 let interface_buffer = define_expert_option current_section ["interface_buffer"]
   "The size of the buffer between the client and its GUI. Can be useful
 to increase when the connection between them has a small bandwith" int_option
@@ -1234,6 +1392,10 @@ let minor_heap_size = define_expert_option current_section
     ["minor_heap_size"] "Size of the minor heap in kB"
     int_option 32
 
+let relevant_queues = define_expert_option current_section ["relevant_queues"]
+    "The source queues to display in source lists (see 'sources' command)"
+    int_list_option [0;1;2;3;4;5;6;8;9;10]
+
 let min_reask_delay = define_expert_option current_section ["min_reask_delay"]
   "The minimal delay between two connections to the same client (in seconds)"
     int_option 600
@@ -1285,6 +1447,15 @@ let compaction_overhead = define_expert_option current_section
     "The percentage of free memory before a compaction is triggered"
     int_option 25
 
+let space_overhead = define_expert_option current_section
+    ["space_overhead"]
+    "The major GC speed is computed from this parameter. This is the memory
+    that will be \"wasted\" because the GC does not immediatly collect 
+    unreachable blocks. It is expressed as a percentage of the memory used
+    for live data. The GC will work more (use more CPU time and collect 
+    blocks more eagerly) if space_overhead is smaller."
+    int_option 80
+
 let max_displayed_results = define_expert_option current_section
     ["max_displayed_results"]
     "Maximal number of results displayed for a search"
@@ -1292,7 +1463,7 @@ let max_displayed_results = define_expert_option current_section
 
 let options_version = define_expert_option current_section ["options_version"]
     "(internal option)"
-    int_option 4
+    int_option 9
 
 
 (*************************************************************************)
@@ -1312,45 +1483,16 @@ let log_size =
   define_expert_option current_section ["log_size"]
     "size of log in number of records" int_option 300
 
+let log_file_size = define_expert_option current_section ["log_file_size"]
+   "Maximum size of log_file in MB" int_option 2
+
 let log_file = define_expert_option current_section ["log_file"]
   "The file in which you want mldonkey to log its debug messages. If you
   set this option, mldonkey will log this info in the file until you use the
 'close_log' command. The log file may become very large. You can
     also enable logging in a file after startup using the 'log_file' command."
-  string_option ""
+  string_option "mlnet.log"
 
-let verbosity = define_expert_option current_section ["verbosity"]
-  "A space-separated list of keywords. Each keyword triggers
-  printing information on the corresponding messages:
-  mc : debug client messages
-  mr|raw : debug raw messages
-  mct : debug emule clients tags
-  ms : debug server messages
-  verb : debug other
-  sm : debug source management
-  net : debug net
-  do : some download warnings
-  up : some upload warnings
-  unk : unknown messages
-  ov : overnet
-  loc : debug source research
-  share: debug sharing
-  md4 : md4 computation
-  connect : debug connections
-  udp : udp messages
-  ultra|super : debug supernode
-  swarming : debug swarming
-  hc : http_client messages
-  hs : http_server messages
-  act : debug activity
-  bw : debug bandwidth
-  tor : debug .torrent loading
-  file : debug file handling
-  redir : debug redirector
-  unexp : debug unexpected messages
-  hid : print hidden errors messages
-"
-    string_option ""
 
 
 
@@ -1418,8 +1560,20 @@ on the top right corner of this window!
 Instead use the kill command in Telnet or HTML,
 the kill function of a GUI or CTRL+C.\n\n"
 
+let real_max_indirect_connections = ref 0
+
+let calc_real_max_indirect_connections () =
+  real_max_indirect_connections :=
+    !!max_opened_connections * !!max_indirect_connections / 100
+
 let _ =
-  let in_hook = ref false in
+  option_hook max_indirect_connections (fun _ ->
+    begin
+      if !!max_indirect_connections > 70 then max_indirect_connections =:= 70
+      else if !!max_indirect_connections < 30 then max_indirect_connections =:= 30
+    end;
+    calc_real_max_indirect_connections ()
+  );
   option_hook global_login (fun _ ->
       let len = String.length !!global_login in
       let prefix = "mldonkey_" in
@@ -1433,16 +1587,23 @@ let _ =
   option_hook log_file (fun _ ->
       if !!log_file <> "" then
         try
-          let oc = open_out !!log_file in
+	  if Sys.file_exists !!log_file then
+	    if (Unix32.getsize !!log_file)
+	     > (Int64ops.megabytes !!log_file_size) then begin
+	      Sys.remove !!log_file;
+              lprintf_nl "Logfile %s reset: bigger than %d MB" !!log_file !!log_file_size
+	    end;
+          let oc = open_out_gen [Open_creat; Open_wronly; Open_append] 0o644 !!log_file in
           lprintf_to_file := true;
-          if Autoconf.system = "windows" then lprintf "%s" win_message;
-          lprintf_nl "Logging in %s" !!log_file;
+          if Autoconf.system = "cygwin" then lprintf "%s" win_message;
+          lprintf_nl "Logging in %s" ( Filename.concat file_basedir !!log_file);
           log_to_file oc;
+	  lprintf_nl "Started logging..."
         with e ->
             lprintf_nl "Exception %s while opening log file: %s"
               (Printexc2.to_string e) !!log_file
       else
-      if !lprintf_to_file then begin
+        if !lprintf_to_file then begin
           lprintf_to_file := false;
           close_log ()
         end
@@ -1459,6 +1620,10 @@ let _ =
       let gc_control = Gc.get () in
       Gc.set { gc_control with Gc.max_overhead = !!compaction_overhead };
   );
+  option_hook space_overhead (fun _ ->
+      let gc_control = Gc.get () in
+      Gc.set { gc_control with Gc.space_overhead = !!space_overhead };
+  );
   option_hook web_infos (fun _ ->
       List.iter (fun remove ->
           if List.mem remove !!web_infos then
@@ -1498,7 +1663,7 @@ let _ =
         (!!minor_heap_size * 1024) };
   );
   option_hook client_buffer_size (fun _ ->
-      TcpBufferedSocket.max_buffer_size := maxi 1000000 !!client_buffer_size
+      TcpBufferedSocket.max_buffer_size := maxi 10000000 !!client_buffer_size
   );
   if Autoconf.has_gd then
     option_hook html_mods_vd_gfx_png (fun _ ->
@@ -1524,11 +1689,8 @@ let verbose_udp = ref false
 let verbose_supernode = ref false
 let verbose_swarming = ref false
 let verbose_activity = ref false
-let verbose_torrent = ref false
-let verbose_files = ref false
 let verbose_redirector = ref false
 let verbose_unexpected_messages = ref false
-let verbose_hidden_errors = ref false
 
 let set_all v =
 
@@ -1540,6 +1702,7 @@ let set_all v =
   BasicSocket.debug := v;
   TcpServerSocket.debug := v;
   UdpSocket.debug := v;
+  Unix32.verbose := v;
   verbose_download := v;
   verbose_upload := v;
   verbose_unknown_messages := v;
@@ -1554,11 +1717,8 @@ let set_all v =
   Http_client.verbose := v;
   Http_server.verbose := v;
   verbose_activity := v;
-  verbose_torrent  := v;
-  verbose_files := v;
   verbose_redirector := v;
-  verbose_unexpected_messages := v;
-  verbose_hidden_errors := v
+  verbose_unexpected_messages := v
 
 let _ =
   option_hook verbosity (fun _ ->
@@ -1574,6 +1734,8 @@ let _ =
           | "verb" -> verbose := true
           | "sm" -> incr verbose_sources
           | "net" -> BasicSocket.debug := true; TcpServerSocket.debug := true; UdpSocket.debug := true
+          | "file" -> Unix32.verbose := true
+          | "gui" -> GuiProto.verbose_gui_decoding := true
           | "do" -> verbose_download := true
           | "up" -> verbose_upload := true
           | "unk" -> verbose_unknown_messages := true
@@ -1589,11 +1751,8 @@ let _ =
           | "hs" -> Http_server.verbose := true
           | "act" -> verbose_activity := true
           | "bw" -> incr BasicSocket.verbose_bandwidth
-          | "tor" -> verbose_torrent := true
-          | "file" -> verbose_files := true
           | "redir" -> verbose_redirector := true
           | "unexp" -> verbose_unexpected_messages := true
-          | "hid" -> verbose_hidden_errors := true
 
           | "all" ->
 
@@ -1614,10 +1773,39 @@ let _ =
       BasicSocket.socket_keepalive := !!socket_keepalive
   )
 
+(* convert "|" to "\|" and "\|" to "|" *)
+let quote_unquote_bars m =
+  let len = String.length m in
+  let result = Buffer.create len in
+  let rec aux i =
+    if i = len then
+      Buffer.contents result
+    else match m.[i] with
+      | '|' -> 
+	  Buffer.add_string result "\\|";
+	  aux (i+1)
+      | '\\' -> 
+	  aux_escaped (i+1)
+      | _ -> 
+	  Buffer.add_char result m.[i];
+	  aux (i+1)
+  and aux_escaped i =
+    if i = len then begin
+      Buffer.add_char result '\\';
+      Buffer.contents result
+    end else match m.[i] with
+      | '|' -> 
+	  Buffer.add_char result '|';
+	  aux (i+1)
+      | _ ->
+	  Buffer.add_char result '\\';
+	  aux i
+  in aux 0
+	    
 let _ =
   option_hook messages_filter (fun _ ->
       is_not_spam := if !!messages_filter <> "" then
-        let r = Str.regexp_case_fold !!messages_filter in
+        let r = Str.regexp_case_fold (quote_unquote_bars !!messages_filter) in
         (fun s -> try
               ignore (Str.search_forward r s 0);
               false
@@ -1712,4 +1900,38 @@ let rec update_options () =
         ];
       update 4
 
+  | 4 ->
+      web_infos_remove
+        [
+          ("server.met", 24,
+            "http://ocbmaurice.dyndns.org/pl/slist.pl/server.met?download/server-best.met");
+        ];
+      web_infos =:= !!web_infos @ [
+          ("server.met", 0,
+            "http://www.gruk.org/server.met.gz");
+        ];
+      update 5
+
+  | 5 ->
+      if !!max_indirect_connections > 50 then
+          max_indirect_connections =:= 20;
+      update 6
+
+  | 6 ->
+      (* it's more natural to use | instead of \| for simple case *) 
+      messages_filter =:= quote_unquote_bars !!messages_filter;
+      update 7
+
+  | 7 ->
+      (* update to 20 because of dynamic_loop_delay patch *)
+      loop_delay =:= 20;
+      update 8
+
+  | 8 ->
+      web_infos =:= !!web_infos @ [
+          ("geoip.dat", 0,
+            "http://www.maxmind.com/download/geoip/database/GeoIP.dat.gz");
+        ];
+      update 9
+
   | _ -> ()
