@@ -25,6 +25,16 @@ open Options
 open CommonUser
 open CommonTypes
 
+(* prints a new logline with date, module and starts newline *)
+let lprintf_nl () =
+  lprintf "%s[cSe] "
+    (log_time ()); lprintf_nl2
+
+(* prints a new logline with date, module and does not start newline *)
+let lprintf_n () =
+  lprintf "%s[cSe] "
+    (log_time ()); lprintf
+
 module G = GuiTypes
 
 type 'a server_impl = {
@@ -48,6 +58,7 @@ and 'a server_ops = {
     mutable op_server_query_users : ('a -> unit);
     mutable op_server_find_user : ('a -> string -> unit);
     mutable op_server_cid : ('a -> Ip.t);
+    mutable op_server_low_id : ('a -> bool);
     mutable op_server_set_preferred : ('a -> bool -> unit);
     mutable op_server_rename : ('a -> string -> unit);
   }
@@ -55,7 +66,7 @@ and 'a server_ops = {
 let ni n m =
   let s = Printf.sprintf "Server.%s not implemented by %s"
       m n.network_name in
-  lprintf_nl "%s" s;
+  lprintf_nl () "%s" s;
   s
 
 let fni n m =  failwith (ni n m)
@@ -80,12 +91,38 @@ let dummy_server_impl = {
 
 let dummy_server = as_server dummy_server_impl
 
+let impl_server_info impl =
+  let module T = GuiTypes in
+  {
+    T.server_num = impl.impl_server_num;
+    T.server_state = impl.impl_server_state;
+
+    T.server_network = 0;
+    T.server_addr = Ip.addr_of_ip Ip.null;
+    T.server_port = 0;
+    T.server_realport = 0;
+    T.server_score = 0;
+    T.server_tags = [];
+    T.server_nusers = 0L;
+    T.server_nfiles = 0L;
+    T.server_name = "";
+    T.server_description = "";
+    T.server_users = None;
+    T.server_banner = "";
+    T.server_preferred = false;
+    T.server_version = "";
+    T.server_max_users = 0L;
+    T.server_soft_limit = 0L;
+    T.server_hard_limit = 0L;
+    T.server_lowid_users = 0L;
+    T.server_ping = 0;
+  }
 
 let server_num s =
   let s = as_server_impl s in
   s.impl_server_num
 
-module H = Weak2.Make(struct
+module H = Weak.Make(struct
       type t = server
       let hash s = Hashtbl.hash (server_num s)
 
@@ -104,7 +141,6 @@ let _ =
   )
 
 let server_must_update s =
-(*  lprintf "server_must_update ?\n";  *)
   let impl = as_server_impl s in
   if impl.impl_server_update <> 0 then
     CommonEvent.add_event (Server_info_event s);
@@ -116,7 +152,6 @@ let server_must_update_state s =
     begin
       impl.impl_server_update <- - impl.impl_server_update;
       CommonEvent.add_event (Server_info_event s);
-(*      lprintf "server_must_update YES\n";  *)
     end
 
 let server_update_num impl =
@@ -154,6 +189,10 @@ let server_cid s =
   let s = as_server_impl s in
   s.impl_server_ops.op_server_cid s.impl_server_val
 
+let server_low_id s =
+  let s = as_server_impl s in
+  s.impl_server_ops.op_server_low_id s.impl_server_val
+
 let server_set_preferred s b =
   let s = as_server_impl s in
   s.impl_server_ops.op_server_set_preferred s.impl_server_val b
@@ -177,6 +216,7 @@ let new_server_ops network =
       op_server_query_users = (fun _ -> ni_ok network "query_users");
       op_server_users = (fun _ -> fni network "users");
       op_server_cid = (fun _ -> fni network "cid");
+      op_server_low_id = (fun _ -> fni network "low_id");
       op_server_set_preferred = (fun _ _ -> fni network "server_set_preferred");
       op_server_rename = (fun _ _ -> fni network "server_rename");
     } in
@@ -186,34 +226,36 @@ let new_server_ops network =
   s
 
 let check_server_implementations () =
-  lprintf_nl "\n----- Methods not implemented for CommonServer ----\n";
+  lprintf_nl () "----- Methods not implemented for CommonServer ----";
   List.iter (fun (c, cc) ->
       let n = c.op_server_network.network_name in
-      lprintf_nl "\n  Network %s\n" n;
+      lprintf_nl ()"  Network %s" n;
       if c.op_server_remove == cc.op_server_remove then
-        lprintf_nl "op_server_remove";
+        lprintf_nl () "op_server_remove";
       if c.op_server_to_option == cc.op_server_to_option then
-        lprintf_nl "op_server_to_option";
+        lprintf_nl () "op_server_to_option";
       if c.op_server_info == cc.op_server_info then
-        lprintf_nl "op_server_info";
+        lprintf_nl () "op_server_info";
       if c.op_server_sort == cc.op_server_sort then
-        lprintf_nl "op_server_sort";
+        lprintf_nl () "op_server_sort";
       if c.op_server_connect == cc.op_server_connect then
-        lprintf_nl "op_server_connect";
+        lprintf_nl () "op_server_connect";
       if c.op_server_disconnect == cc.op_server_disconnect then
-        lprintf_nl "op_server_disconnect";
+        lprintf_nl () "op_server_disconnect";
       if c.op_server_find_user == cc.op_server_find_user then
-        lprintf_nl "op_server_find_user";
+        lprintf_nl () "op_server_find_user";
       if c.op_server_query_users == cc.op_server_query_users then
-        lprintf_nl "op_server_query_users";
+        lprintf_nl () "op_server_query_users";
       if c.op_server_users == cc.op_server_users then
-        lprintf_nl "op_server_users";
+        lprintf_nl () "op_server_users";
       if c.op_server_cid == cc.op_server_cid then
-        lprintf_nl "op_server_cid";
+        lprintf_nl () "op_server_cid";
+      if c.op_server_low_id == cc.op_server_low_id then
+        lprintf_nl () "op_server_low_id";
       if c.op_server_rename == cc.op_server_rename then
-        lprintf_nl "op_server_rename";
+        lprintf_nl () "op_server_rename";
       if c.op_server_set_preferred == cc.op_server_set_preferred then
-        lprintf_nl "op_server_set_preferred";
+        lprintf_nl () "op_server_set_preferred";
   ) !servers_ops;
   lprint_newline ()
 
@@ -238,10 +280,6 @@ let server_state c =
   let impl = as_server_impl c in
   impl.impl_server_state
 
-let server_num c =
-  let impl = as_server_impl c in
-  impl.impl_server_num
-
 let set_server_state c state =
   let impl = as_server_impl c in
   if impl.impl_server_state <> state then begin
@@ -282,24 +320,24 @@ let servers_get_all () =
 
 let servers_by_num = ()
 
-(*
-type server_info = {
-    server_num : int;
-    server_network : int;
-
-    mutable server_ip : Ip.t;
-    mutable server_port : int;
-    mutable server_score : int;
-    mutable server_tags : CommonTypes.tag list;
-    mutable server_nusers : int;
-    mutable server_nfiles : int;
-    mutable server_state : host_state;
-    mutable server_name : string;
-    mutable server_description : string;
-    mutable server_users : int list option;
-  }
-    *)
-
+let check_blocked_servers () =
+  try
+    server_iter (fun s ->
+      if server_blocked s then
+        begin
+          let impl = as_server_impl s in
+          let info = server_info s in
+          (match impl.impl_server_state with
+             NotConnected _ -> ()
+         | _ -> server_disconnect s;
+              lprintf_nl () "Disconnected server %s (%s:%d), IP is now blocked"
+                info.G.server_name
+                (Ip.string_of_addr info.G.server_addr)
+                info.G.server_port);
+      end;
+      server_must_update s)
+  with e ->
+    lprintf_nl () "Exception in check_blocked_servers: %s" (Printexc2.to_string e)
 
 let server_banner s o =
   let buf = o.conn_buf in
@@ -309,18 +347,24 @@ let server_banner s o =
 
 let server_print_html_header buf ext =
     html_mods_table_header buf "serversTable" (Printf.sprintf "servers%s" ext) [
-		( "1", "srh", "Server number", "#" ) ;
-		( "0", "srh", "Connect|Disconnect", "C/D" ) ;
-		( "0", "srh", "Remove", "Rem" ) ;
+    ( "1", "srh", "Server number", "#" ) ;
+    ( "0", "srh", "Connect|Disconnect", "C/D" ) ;
+    ( "0", "srh", "Remove", "Rem" ) ;
     ( "0", "srh", "Preferred", "P" ) ;
-		( "0", "srh", "[Hi]gh or [Lo]w ID", "ID" ) ;
-		( "0", "srh", "Network name", "Network" ) ;
-		( "0", "srh", "Connection status", "Status" ) ;
-		( "0", "srh br", "IP address", "IP address" ) ;
-		( "1", "srh ar", "Number of connected users", "Users" ) ;
-		( "1", "srh ar br", "Number of files indexed on server", "Files" ) ;
-		( "0", "srh", "Server name", "Name" ) ;
-		( "0", "srh", "Server details", "Details" ) ]
+    ( "0", "srh", "[Hi]gh or [Lo]w ID", "ID" ) ;
+    ( "0", "srh", "Network name", "Network" ) ;
+    ( "0", "srh", "Connection status", "Status" ) ;
+    ( "0", "srh br", "IP address", "IP address" ) ;
+    ( "1", "srh ar", "Number of connected users", "Users" ) ;
+    ( "1", "srh ar br", "Max number of users", "MaxUsers" ) ;
+    ( "1", "srh ar br", "LowID users", "LowID" ) ;
+    ( "1", "srh ar br", "Number of files indexed on server", "Files" ) ;
+    ( "1", "srh ar", "Soft file limit", "Soft" ) ;
+    ( "1", "srh ar br", "Hard file limit", "Hard" ) ;
+    ( "0", "srh ar br", "Ping (ms)", "Ping" ) ;
+    ( "0", "srh", "Server version", "Version" ) ;
+    ( "0", "srh", "Server name", "Name" ) ;
+    ( "0", "srh", "Server details", "Details" ) ]
 
 let server_print s o =
   let impl = as_server_impl s in
@@ -328,14 +372,14 @@ let server_print s o =
   try
     let info =
       try server_info s with e ->
-          lprintf "Exception %s in server_info (%s)\n"
+          lprintf_nl () "Exception %s in server_info (%s)\n"
             (Printexc2.to_string e) n.network_name;
           raise e in
 
     let buf = o.conn_buf in
-	
-	if use_html_mods o then begin
-	let snum = (server_num s) in
+  
+  if use_html_mods o then begin
+  let snum = (server_num s) in
 
     Printf.bprintf buf "
     \\<tr class=\\\"dl-%d\\\"\\>
@@ -349,33 +393,40 @@ let server_print s o =
     \\<td class=\\\"sr br\\\"\\>%s:%s\\</td\\>
     \\<td class=\\\"sr ar\\\"\\>%Ld\\</td\\>
     \\<td class=\\\"sr ar br\\\"\\>%Ld\\</td\\>
+    \\<td class=\\\"sr ar br\\\"\\>%Ld\\</td\\>
+    \\<td class=\\\"sr ar br\\\"\\>%Ld\\</td\\>
+    \\<td class=\\\"sr ar\\\"\\>%Ld\\</td\\>
+    \\<td class=\\\"sr ar br\\\"\\>%Ld\\</td\\>
+    \\<td class=\\\"sr ar br\\\"\\>%d\\</td\\>
+    \\<td class=\\\"sr br\\\"\\>%s\\</td\\>
     \\<td class=\\\"sr\\\"\\>%s\\</td\\>
     \\<td width=\\\"100%%\\\" class=\\\"sr\\\"\\>%s\\</td\\>\\</tr\\>\n"
     (html_mods_cntr ())
-	  (
+    (
         Printf.sprintf "%s"
         (match impl.impl_server_state with
         Connected _ -> Printf.sprintf "title=\\\"Server Banner\\\"
-						onMouseOver=\\\"mOvr(this);\\\"
-						onMouseOut=\\\"mOut(this);\\\"
-						onClick=\\\"location.href='submit?q=server_banner+%d'\\\"" snum
+            onMouseOver=\\\"mOvr(this);\\\"
+            onMouseOut=\\\"mOut(this);\\\"
+            onClick=\\\"location.href='submit?q=server_banner+%d'\\\"" snum
         | _ -> "")
-	  )
-	  snum
-      (if server_blocked s then
-        Printf.sprintf "\\<TD class=\\\"srb\\\"----\\</TD\\>"
-       else
+    )
+    snum
+      (
+        if server_blocked s && (match impl.impl_server_state with
+         NotConnected _ -> true
+             | _ -> false) then "\\<td class=\\\"srb\\\"\\>blk\\</td\\>" else
         Printf.sprintf
         "\\<TD class=\\\"srb\\\" onMouseOver=\\\"mOvr(this);\\\"
         onMouseOut=\\\"mOut(this);\\\" title=\\\"Connect|Disconnect\\\"
         onClick=\\\"parent.fstatus.location.href='submit?q=%s+%d'\\\"\\>%s\\</TD\\>"
-      (match impl.impl_server_state with
-        NotConnected _ -> "c"
-      | _ -> "x")
-      snum
-      (match impl.impl_server_state with
-        NotConnected _ -> "Conn"
-      | _ -> "Disc")
+        (match impl.impl_server_state with
+           NotConnected _ -> "c"
+         | _ -> "x")
+        snum
+        (match impl.impl_server_state with
+           NotConnected _ -> "Conn"
+         | _ -> "Disc")
       )
       (
         Printf.sprintf
@@ -401,17 +452,12 @@ let server_print s o =
       )
       (if n.network_name = "Donkey" then
          begin
-           let donkey_low_id ip =
-             match Ip.to_ints ip with
-             | _, _, _, 0 -> true
-             | _ -> false
-           in
            match impl.impl_server_state with
            | Connected _ ->
                begin
                  let cid = (server_cid s) in
                  let (label,shortlabel,our_ip) =
-                   if not (donkey_low_id cid) then
+                   if not (server_low_id s) then
                      ("HighID","Hi",
                       (if !!set_client_ip <> cid then
                          Printf.sprintf "(clientIP: %s)"
@@ -435,41 +481,48 @@ let server_print s o =
        else "\\>"
       )
       n.network_name
-      (if server_blocked s then
-	"IP blocked"
-      else
-	(string_of_connection_state impl.impl_server_state))
+      (match impl.impl_server_state with
+        NotConnected _ -> if server_blocked s then "IP blocked"
+        else (string_of_connection_state impl.impl_server_state)
+      | _ -> (string_of_connection_state impl.impl_server_state))
       (Ip.string_of_addr info.G.server_addr)
       (Printf.sprintf "%s%s"
        (string_of_int info.G.server_port)
-       (if info.G.server_realport <> 0 then
-          "(" ^ (string_of_int info.G.server_realport) ^ ")" else ""))
+       (if info.G.server_realport <> 0 
+          then "(" ^ (string_of_int info.G.server_realport) ^ ")" 
+          else ""))
       info.G.server_nusers
+      info.G.server_max_users
+      info.G.server_lowid_users
       info.G.server_nfiles
+      info.G.server_soft_limit
+      info.G.server_hard_limit
+      info.G.server_ping
+      info.G.server_version
       info.G.server_name
       info.G.server_description
 
-	end
+  end
    else
-	begin
+  begin
 
         Printf.bprintf buf "[%-10s%5d] %15s:%-10s %s\n%45sUsers:%-8Ld Files:%-8Ld State:%s\n"
           (n.network_name)
           (server_num s)
           (Ip.string_of_addr info.G.server_addr)
           (Printf.sprintf "%s%s"
-	   (string_of_int info.G.server_port)
-           (if info.G.server_realport <> 0 then
-	      "(" ^ (string_of_int info.G.server_realport) ^ ")" else ""))
+          (string_of_int info.G.server_port)
+          (if info.G.server_realport <> 0 
+            then "(" ^ (string_of_int info.G.server_realport) ^ ")" 
+            else ""))
           (info.G.server_name) ("")
           (info.G.server_nusers)
           (info.G.server_nfiles)
-          (if server_blocked s then
-	    "IP blocked"
-          else
-	    (string_of_connection_state impl.impl_server_state));
+          (if server_blocked s 
+            then "IP blocked" 
+            else (string_of_connection_state impl.impl_server_state));
       end;
 
   with e ->
-      lprintf "Exception %s in CommonServer.server_print\n"
+      lprintf_nl () "Exception %s in CommonServer.server_print"
         (Printexc2.to_string e)
