@@ -18,6 +18,7 @@
 *)
 
 open Options
+open Printf2
 open Md4 
 
 type activity = {
@@ -396,6 +397,7 @@ type result_info = {
     mutable result_tags : tag list;
     mutable result_comment : string;
     mutable result_done : bool;
+    mutable result_force : bool;
 
     mutable result_time : int;
     mutable result_modified : bool;
@@ -461,6 +463,7 @@ type network = {
     mutable op_network_save_sources : (unit -> unit);
     mutable op_network_load_complex_options : (unit -> unit);
     mutable op_network_enable : (unit -> unit);
+    mutable op_network_update_options : (unit -> unit);
     mutable op_network_disable : (unit -> unit);
     
     mutable op_network_server_of_option : 
@@ -475,7 +478,7 @@ type network = {
     mutable op_network_share : (
       string -> string -> int64 -> unit);
     mutable op_network_private_message : (string -> string -> unit);
-    mutable op_network_parse_url : (string -> bool);
+    mutable op_network_parse_url : (string -> string * bool);
     mutable op_network_connect_servers : (unit -> unit);
     
     mutable op_network_search : (search -> Buffer.t -> unit);
@@ -491,6 +494,7 @@ type network = {
     
     mutable op_network_download : (result_info -> file);
     mutable op_network_display_stats : (Buffer.t -> ui_conn -> unit);
+    mutable op_network_clean_exit : (unit -> bool);
   }
 
 and   ui_user = {
@@ -672,13 +676,12 @@ let ogg_tag_to_string ogg_tag =
 
 let print_ogg_infos ogg_infos =
   List.iter (fun stream ->
-    Printf.printf "-== %s info - stream number: %d ==-\n"
+    lprintf "-== %s info - stream number: %d ==-\n"
       (stream_type_to_string stream.stream_type) stream.stream_no;
     List.iter (fun tag ->
-      Printf.printf "  %s\n" (ogg_tag_to_string tag)
+      lprintf "  %s\n" (ogg_tag_to_string tag)
     ) stream.stream_tags
-  ) !ogg_infos;
-  flush stdout
+  ) !ogg_infos
 
 type format =
   AVI of avi_info
@@ -697,7 +700,7 @@ and history_result = {
 
 and location_kind = 
   Known_location of Ip.t * int
-| Indirect_location of string * Md4.t
+| Indirect_location of string * Md4.t * Ip.t * int
   
 (*
 We should add information about what has already been sent to the GUI to
@@ -841,9 +844,37 @@ type arg_kind =
 | Arg_three of (string -> string -> string -> arg_handler)
 
     
-let  string_of_kind kind =
+let string_of_kind kind =
   try
     match kind with
     | Known_location (ip,port) -> Ip.to_string ip
-    | _ -> "firewalled"
+    | Indirect_location (server_ip, server_port, ip, port) -> Ip.to_string ip
   with _ -> ""
+
+let string_of_kind_geo kind =
+  try
+    match kind with
+    | Known_location (ip,port) -> 
+        let cc,cn = Geoip.get_country ip in
+        (Ip.to_string ip),cc,cn
+    | Indirect_location (server_ip, server_port, ip, port) ->
+        let cc,cn = Geoip.get_country ip in
+        (Ip.to_string ip),cc,cn
+  with _ -> "","X","Country error"
+
+type brand_stat = {
+  mutable brand_seen : int;
+  mutable brand_banned : int;
+  mutable brand_filerequest : int;
+  mutable brand_download : Int64.t;
+  mutable brand_upload : Int64.t;
+}
+
+let dummy_stats =
+  {
+    brand_seen = 0;
+    brand_banned = 0;
+    brand_filerequest = 0;
+    brand_download = 0L;
+    brand_upload = 0L;
+  }
