@@ -33,6 +33,16 @@ open TcpBufferedSocket
 open CommonGlobals
 open CommonOptions
 
+(* prints a new logline with date, module and starts newline *)
+let lprintf_nl () =
+  lprintf "%s[cUp] "
+    (log_time ()); lprintf_nl2
+
+(* prints a new logline with date, module and does not start newline *)
+let lprintf_n () =
+  lprintf "%s[cUp] "
+    (log_time ()); lprintf
+
 (*
 PROBLEMS: most of the time, users won't share their files on all networks.
 We should provide a different directory than incoming/, where files
@@ -52,7 +62,7 @@ Default would be: share all files greater than 1 MB in incoming/ on Edonkey.
 
 *******************************************************************)
 
-let ed2k_block_size = Int64.of_int 9728000
+let ed2k_block_size = 9728000L
 let tiger_block_size = Int64.of_int (1024 * 1024)
 
 type shared_file = {
@@ -217,6 +227,7 @@ let network = CommonNetwork.new_network "GS" "Global Shares"
 let _ =
   network.op_network_connected <- (fun _ -> false);
   network.op_network_is_enabled <- (fun _ -> raise IgnoreNetwork);
+  network.op_network_update_options <- (fun _ -> raise IgnoreNetwork);
   network.op_network_info <- (fun _ -> raise Not_found);
   network.op_network_info <- (fun n ->
       {
@@ -240,11 +251,9 @@ let shareds_by_uid = Hashtbl.create 13
 let shareds_by_id = Hashtbl.create 13
 
 let add_by_uid uid sh =
-  let urn = Uid.to_string uid in
   Hashtbl.add shareds_by_uid uid sh
 
 let find_by_uid uid =
-  let urn = Uid.to_string uid in
   Hashtbl.find shareds_by_uid uid
 
 module SharedFilesIndex = IndexedSharedFiles.MakeIndex (struct
@@ -265,6 +274,15 @@ let shareds_counter = ref 1
 let shared_counter = ref (Int64.zero)
 let shared_files = Hashtbl.create 13
 
+let _ =
+  Heap.add_memstat "CommonUploads" (fun level buf ->
+   Printf.bprintf buf "  infos_by_name: %d\n" (Hashtbl.length infos_by_name);
+   Printf.bprintf buf "  shareds_by_uid: %d\n" (Hashtbl.length shareds_by_uid);
+   Printf.bprintf buf "  shareds_by_id: %d\n" (Hashtbl.length shareds_by_id);
+   Printf.bprintf buf "  shared_files: %d\n" (Hashtbl.length shared_files);
+  )
+
+
 let new_shared_dir dirname = {
     shared_dirname = dirname;
     shared_files = [];
@@ -431,7 +449,7 @@ let rec start_job_for sh (wanted_id, handler) =
             CommonHasher.compute_sha1 (Unix32.filename sh.shared_fd)
             zero info.shared_size (fun job ->
                 if job.CommonHasher.job_error then begin
-                    lprintf_nl "[cUp] Error during hashing of %s" info.shared_fullname;
+                    lprintf_nl () "Error during hashing of %s" info.shared_fullname;
                     current_job := None;
                   end else
                   begin
@@ -491,9 +509,9 @@ computation ??? *)
     | MD5EXT ->
         let md5ext =
           try
-            let fd = Unix32.create_rw info.shared_fullname in
-            let file_size = Unix32.getsize64 fd false in
-            let len64 = min (Int64.of_int 307200) file_size in
+            let fd = Unix32.create_ro info.shared_fullname in
+            let file_size = Unix32.getsize64 fd in
+            let len64 = min 307200L file_size in
             let len = Int64.to_int len64 in
             let s = String.create len in
             Unix32.read fd zero s 0 len;
@@ -512,7 +530,6 @@ computation ??? *)
     | ED2K ->
         let size = info.shared_size in
         let chunk_size = ed2k_block_size  in
-        let nhashes = Int64.to_int (size // chunk_size) + 1 in
         let rec iter pos hashes =
           if pos < size then
             try
@@ -520,7 +537,7 @@ computation ??? *)
                 pos (min (size -- pos) chunk_size)
               (fun job ->
                   if job.CommonHasher.job_error then begin
-                      lprintf_nl "[cUp] Error during hashing of %s" info.shared_fullname;
+                      lprintf_nl () "Error during hashing of %s" info.shared_fullname;
                       current_job := None;
                     end else begin
 
@@ -533,7 +550,6 @@ computation ??? *)
           let list = List.rev hashes in
           let ed2k = md4_of_list list in
           let uid = Uid.create (Ed2k (ed2k)) in
-          let urn = Uid.to_string uid in
           info.shared_md4s <- Array.of_list list;
           info.shared_uids <- uid :: info.shared_uids;
           IndexedSharedFiles.update_result sh.shared_info info;
@@ -549,14 +565,13 @@ computation ??? *)
 
           let size = info.shared_size in
           let chunk_size = tiger_block_size in
-          let nhashes = Int64.to_int (size // chunk_size) + 1 in
           let rec iter pos hashes =
             if pos < size then
                 CommonHasher.compute_tiger info.shared_fullname
                   pos (min (size -- pos) chunk_size)
               (fun job ->
                     if job.CommonHasher.job_error then begin
-                      lprintf_nl "[cUp] Error during hashing of %s"
+                      lprintf_nl () "Error during hashing of %s"
                       info.shared_fullname;
                         current_job := None;
                       end else begin
@@ -593,7 +608,7 @@ let shared_files_timer _ =
             [] ->  waiting_shared_files := tail;
           | uid :: tail ->
               if !verbose_share then
-                lprintf_nl "[cUp] shared_files_timer: starting job";
+                lprintf_nl () "shared_files_timer: starting job";
               sh.shared_uids_wanted <- tail;
               current_job := Some sh;
               start_job_for sh uid
@@ -695,7 +710,7 @@ let add_shared full_name codedname size =
 
       SharedFilesIndex.add sh.shared_info;
       add_shared_file shared_tree sh (String2.split codedname '/');
-      shared_counter := Int64.add !shared_counter size;
+      shared_counter := !shared_counter ++ size;
 (*      lprintf "Total shared : %Ld\n" !shared_counter; *)
       sh
 
@@ -729,112 +744,73 @@ let upload_clients = (Fifo.create () : client Fifo.t)
 let (pending_slots_map : client Intmap.t ref) = ref Intmap.empty
 (* let (pending_slots_fifo : int Fifo.t)  = Fifo.create () *)
 
-let remaining_bandwidth = ref 0
-let total_bandwidth = ref 0
-let complete_bandwidth = ref 0
-let counter = ref 1
-let sent_bytes = Array.create 10 0
+let packet_size = 10240
+
+(* two seconds max of streaming ahead *)
+let streaming_amount () = 
+  int_of_float (!CommonGlobals.payload_bandwidth *. 2.0)
+let streaming_left = ref (streaming_amount ())
+let streaming_time = (ref None : float option ref)
+
 let has_upload = ref 0
 let upload_credit = ref 0
 
 
 let can_write_len sock len =
   let bool1 = can_write_len sock len in
-  let upload_rate =
-        (if !!max_hard_upload_rate = 0 then 10000 else !!max_hard_upload_rate)
-    * 1024 in
-  let bool2 =
-    (
-(*      lprintf "upload_rate %d -> %d\n" upload_rate
-        (upload_rate * (Fifo.length upload_clients)); *)
-(* changed 2.5.24
-Don't put in a socket more than 10 seconds of upload.
-  *)
-    not_buffer_more sock (upload_rate * 10  (* * (Fifo.length upload_clients) *) ))
-  in
+(* changed 2.5.24: Don't put in a socket more than 10 seconds of upload. *)
+(* ...since the socket shares available bandwidth with others, 
+   that test has no physical meaning. Should it simply be removed ? *)
+  let bool2 = not_buffer_more sock (int_of_float !CommonGlobals.payload_bandwidth * 10) in
 (*  lprintf "can_write_len %b %b\n" bool1 bool2; *)
   let b = bool1 && bool2 in
 (*  if not b then
-lprintf "bool1 %b len %d bool2 %b upload %d\n" bool1 len bool2 upload_rate;
-  *)
+      lprintf "bool1 %b len %d bool2 %b upload %d\n" 
+        bool1 len bool2 upload_rate; *)
   b
 
-let upload_to_one_client () =
-(*  lprintf "upload_to_one_client %d %d\n" (Fifo.length upload_clients)
-  !remaining_bandwidth;
-Fifo.iter (fun c -> lprintf "   client %d\n" (client_num c))
-upload_clients; *)
-  if !remaining_bandwidth < 10000 then begin
-      let c = Fifo.take upload_clients in
-      client_can_upload c  !remaining_bandwidth
-    end else
-  let per_client =
-    let len = Fifo.length upload_clients in
-    if len * 10000 < !remaining_bandwidth then
-(* Each client in the Fifo can receive 10000 bytes.
-Divide the bandwidth between the clients
-*)
-      (!remaining_bandwidth / 10000 / len) * 10000
-    else mini 10000 !remaining_bandwidth in
-  let c = Fifo.take upload_clients in
-  client_can_upload c per_client
-
-let rec fifo_uploads n =
-  if n>0 && !remaining_bandwidth > 0 then
-    begin
-      upload_to_one_client ();
-      fifo_uploads (n-1)
-    end
-
-let rec next_uploads () =
-(*  lprintf "next_uploads %d\n" !remaining_bandwidth; *)
-  let old_remaining_bandwidth = !remaining_bandwidth in
-  let len = Fifo.length upload_clients in
-  fifo_uploads len;
-  if !remaining_bandwidth < old_remaining_bandwidth then
-    next_uploads ()
-
 let next_uploads () =
-  sent_bytes.(!counter-1) <- sent_bytes.(!counter-1) - !remaining_bandwidth;
+
+  let rec next_uploads_aux () =
+    let rec next_uploads_round n =
+      let upload_to_one_client max_amount =
+        let c = Fifo.take upload_clients in
+        client_can_upload c max_amount
+(* it's up to client_can_upload to put the client back into the Fifo *)
+      in
+
+(*  lprintf "next_uploads %d %d\n" 
+      (Fifo.length upload_clients) !streaming_left;
+    Fifo.iter (fun c -> 
+      lprintf "   client %d\n" (client_num c)
+    ) upload_clients; *)
+      if n>0 && 
+        not (Fifo.empty upload_clients) && 
+        !streaming_left > 0 then begin
+          upload_to_one_client packet_size;
+          next_uploads_round (n-1)
+      end in
+
+(* stop if no uploader could take anything during the last round *)
+    let old_streaming_left = !streaming_left in
+    next_uploads_round (Fifo.length upload_clients);
+    if !streaming_left < old_streaming_left then
+      next_uploads_aux () in
+
   (*
   if !verbose_upload then begin
-      lprintf "Left %d\n" !remaining_bandwidth;
+      lprintf "streaming_left %d\n" !streaming_left;
     end; *)
-  complete_bandwidth := !complete_bandwidth + !remaining_bandwidth;
-  incr counter;
-  if !counter = 11 then begin
-      counter := 1;
-      total_bandwidth :=
-      (if !!max_hard_upload_rate = 0 then 10000 * 1024
-        else (maxi (!!max_hard_upload_rate - 1) 1) * 1024 );
-      complete_bandwidth := !total_bandwidth;
-(*      lprintf "Init to %d\n" !total_bandwidth;  *)
-      remaining_bandwidth := 0
-    end;
-
-  let last_sec = ref 0 in
-  for i = 0 to 9 do
-    last_sec := !last_sec + sent_bytes.(i)
-  done;
-
-(*  if !verbose_upload then begin
-      lprintf "last sec: %d/%d (left %d)\n" !last_sec !total_bandwidth
-        (!total_bandwidth - !last_sec);
- (*
-      for i = 0 to 9 do
-        lprintf "    last[%d] = %d\n" i  sent_bytes.(i)
-      done; *)
-
-    end; *)
-  remaining_bandwidth := mini (mini (mini
-        (maxi (!remaining_bandwidth + !total_bandwidth / 10) 10000)
-      !total_bandwidth) !complete_bandwidth)
-  (!total_bandwidth - !last_sec);
-  complete_bandwidth := !complete_bandwidth - !remaining_bandwidth;
-(*  lprintf "Remaining %d[%d]\n" !remaining_bandwidth !complete_bandwidth;  *)
-  sent_bytes.(!counter-1) <- !remaining_bandwidth;
-  if !remaining_bandwidth > 0 then
-    next_uploads ()
+  (* buffer empties with time... *)
+  let new_streaming_time = BasicSocket.current_time () in
+  let deltat = (match !streaming_time with
+    | None -> 0.
+    | Some t -> new_streaming_time -. t) in
+  streaming_left := !streaming_left + 
+    (int_of_float (!CommonGlobals.payload_bandwidth *. deltat));
+  streaming_left := min !streaming_left (streaming_amount ());
+  streaming_time := Some new_streaming_time;
+  next_uploads_aux ()
 
 let reset_upload_timer () = ()
 
@@ -852,7 +828,7 @@ let ready_for_upload c =
 
 let add_pending_slot c =
   if client_has_a_slot c then begin
-      if !verbose_upload then lprintf_nl "[cUp] Avoided inserting an uploader in pending slots!"
+      if !verbose_upload then lprintf_nl () "Avoided inserting an uploader in pending slots!"
     end
   else
   if not (Intmap.mem (client_num c) !pending_slots_map) then
@@ -882,7 +858,8 @@ let rec give_a_slot c =
 and find_pending_slot () =
   try
     let rec iter () =
-      let _, c = Intmap.top !pending_slots_map in
+      let c = Intmap.nth !pending_slots_map 
+	(Random.int (Intmap.length !pending_slots_map)) in
       give_a_slot c
     in
     iter ()
@@ -892,15 +869,18 @@ let add_pending_slot c =
   let csh = client_upload c in
   let cdir = shared_dir csh in
   let cprio = ref (shared_prio csh) in
+  let cfriend = ref (if !!friends_upload_slot then 1 else 0) in
   (* if cdir <> "" then
     lprintf "Testing cdir %s\n" cdir; *)
   Intmap.iter (fun _ c ->
     let sh = client_upload c in
-    if shared_dir sh = cdir then decr cprio
+    if shared_dir sh = cdir then decr cprio;
+    if client_has_a_friend_slot c then decr cfriend;
   ) !CommonClient.uploaders;
   (* if cdir <> "" then
-    lprintf "Testing cprio %d\n" !cprio; *)
-  if !cprio > 0 then begin
+    lprintf "Testing cprio %d cfriend %d\n" !cprio !cfriend; *)
+  if !cprio > 0 || 
+     (is_friend c && !cfriend > 0) then begin
     remove_pending_slot c;
     if client_is_connected c then begin
       set_client_has_a_slot c true;
@@ -931,7 +911,7 @@ let dynamic_refill_upload_slots () =
   let open_slots n =
     let i = ref n in
     if !verbose_upload then
-      lprintf_nl "[cUp] try to allocate %d more slots" n;
+      lprintf_nl () "try to allocate %d more slots" n;
     while !i > 0 do
       find_pending_slot ();
       decr i
@@ -939,10 +919,14 @@ let dynamic_refill_upload_slots () =
 
   let slot_bw = 3072 in
   let min_upload_slots = 3 in
-(*  let estimated_capacity = !!max_hard_upload_rate * 1024 in *)
   let estimated_capacity = detected_uplink_capacity () in
+  let estimated_capacity = if !!max_hard_upload_rate = 0 then
+    estimated_capacity
+  else 
+    (* max_hard_upload_rate lowered manually,... *)
+    mini estimated_capacity (!!max_hard_upload_rate * 1024) in
   if !verbose_upload then
-    lprintf_nl "[cUp] usage: %d(%d) capacity: %d"
+    lprintf_nl () "usage: %d(%d) capacity: %d"
       (short_delay_upload_usage ())
       (upload_usage ())
       estimated_capacity;
@@ -952,13 +936,13 @@ let dynamic_refill_upload_slots () =
 (* enough free bw for another slot *)
     if short_delay_upload_usage () + slot_bw < estimated_capacity then begin
       if !verbose_upload then
-	lprintf_nl "[cUp] uplink not fully used";
+	lprintf_nl () "uplink not fully used";
       incr not_saturated_count
     end else reset_state ();
 
     if len < min_upload_slots then begin
       if !verbose_upload then
-	lprintf_nl "[cUp] too few upload slots";
+	lprintf_nl () "too few upload slots";
       open_slots (min_upload_slots - len);
       reset_state ()
     end else if !not_saturated_count >= 2 then begin
@@ -998,9 +982,7 @@ let refill_upload_slots () =
     end
 
 let consume_bandwidth len =
-  remaining_bandwidth := !remaining_bandwidth - len
-
-let remaining_bandwidth () = !remaining_bandwidth
+  streaming_left := !streaming_left - len
 
 (**********************************************************************
 
@@ -1040,10 +1022,10 @@ let queue_download_request f len =
 let upload_download_timer () =
   (try download_engine ()
     with e ->
-        lprintf_nl "[cUp] Exception %s in download_engine" (Printexc2.to_string e)
+        lprintf_nl () "Exception %s in download_engine" (Printexc2.to_string e)
   );
   (try next_uploads ()
-  with e -> lprintf_nl "[cUp] exc %s in upload" (Printexc2.to_string e))
+  with e -> lprintf_nl () "exc %s in upload" (Printexc2.to_string e))
 
 let words_of_filename =
   let extension_list = [
@@ -1066,7 +1048,7 @@ let words_of_filename =
   let get_name_keywords file_name =
     match remove_short (String2.stem file_name) [] with
       [] | [_] ->
-        lprintf_nl "[cUp] Not enough keywords to recover %s" file_name;
+        lprintf_nl () "Not enough keywords to recover %s" file_name;
         [file_name]
     | l -> l
   in
@@ -1081,10 +1063,8 @@ let _ =
       let total_uploaded = ref Int64.zero in
       S.shared_iter (fun s ->
           let i = S.as_shared_impl s in
-          total_uploaded :=
-          Int64.add !total_uploaded i.S.impl_shared_uploaded;
-          total_shared :=
-          Int64.add !total_shared i.S.impl_shared_size
+          total_uploaded := !total_uploaded ++ i.S.impl_shared_uploaded;
+          total_shared := !total_shared ++ i.S.impl_shared_size
       );
       buf_int64 buf !total_shared;
       buf_int64 buf !total_uploaded;
