@@ -142,6 +142,7 @@ type file_info = {
     mutable file_last_seen : int;
     mutable file_priority : int;
     mutable file_uids : Uid.t list;
+    mutable file_sub_files : (string * int64) list;
   }
   
 type user_info = {
@@ -171,6 +172,13 @@ type server_info = {
     mutable server_users : int list option;
     mutable server_banner : string;
     mutable server_preferred : bool;
+    mutable server_version : string;
+    mutable server_max_users : int64;
+    mutable server_soft_limit : int64;
+    mutable server_hard_limit : int64;
+    mutable server_lowid_users : int64;
+    mutable server_ping : int;
+
   } 
 
 type room_info = {
@@ -212,6 +220,7 @@ type client_info = {
     mutable client_downloaded : int64;
     mutable client_uploaded : int64;
     mutable client_upload : string option;
+    mutable client_sui_verified : bool option;
 (*  mutable client_sock_addr : string;  *)
   }
 
@@ -237,6 +246,7 @@ type shared_info = {
     mutable shared_uploaded : int64;
     mutable shared_requests : int;
     mutable shared_uids : Uid.t list; (* net file UID *)
+    mutable shared_sub_files : (string * int64) list;
   }
   
   
@@ -322,5 +332,6 @@ let file_info_test =
     file_last_seen = BasicSocket.last_time ();
     file_priority = 0;
    file_uids = [];
+    file_sub_files = [];
  } 
   
