@@ -26,6 +26,7 @@ open CommonSearch
 open CommonTypes
 open CommonGlobals
 open CommonGraphics
+open CommonShared
 open GuiTypes
 open CommonComplexOptions
 open CommonFile
@@ -40,6 +41,11 @@ let lprintf_nl () =
   lprintf "%s[dCon] "
     (log_time ()); lprintf_nl2
 
+(* prints a new logline with date, module and does not start newline *)
+let lprintf_n () =
+  lprintf "%s[dCon] "
+    (log_time ()); lprintf
+
 let rec dollar_escape o with_frames s =
   String2.convert false (fun b escaped c ->
       if escaped then
@@ -73,6 +79,11 @@ let rec dollar_escape o with_frames s =
               Buffer.add_string b Terminal.ANSI.ansi_GREEN;
             false
 
+        | 'c' ->
+            if o.conn_output = ANSI then
+              Buffer.add_string b Terminal.ANSI.ansi_CYAN;
+            false
+
         | 'n' ->
             if o.conn_output = ANSI then
               Buffer.add_string b Terminal.ANSI.ansi_NORMAL;
@@ -93,8 +104,10 @@ let rec dollar_escape o with_frames s =
 
 let eval auth cmd o =
   let buf = o.conn_buf in
+  let cmd = Url.decode cmd in
   let cmd =
-    if String2.check_prefix cmd "ed2k://" then "dllink " ^ cmd
+    if String2.check_prefix cmd "ed2k://" ||
+       String2.check_prefix cmd "http://" then "dllink " ^ cmd
     else if String2.check_prefix cmd "fha://" then "ovlink " ^ cmd
     else cmd in
   let l = String2.tokens cmd in
@@ -333,11 +346,11 @@ let check_calendar () =
   let time = last_time () in
   let tm = Unix.localtime (date_of_int time) in
   List.iter (fun (days, hours, command) ->
-      if (List.mem tm.Unix.tm_wday days || days = [])  &&
+      if (List.mem tm.Unix.tm_wday days || days = []) &&
         (List.mem tm.Unix.tm_hour hours || hours = []) then begin
-          eval (ref true) command calendar_options;
           lprintf_nl () "Calendar execute: %s" command;
-          lprintf_nl () "%s" (Buffer.contents calendar_options.conn_buf);
+          eval (ref true) command calendar_options;
+          lprintf_nl () "Calendar result: %s" (Buffer.contents calendar_options.conn_buf);
           Buffer.reset calendar_options.conn_buf;
         end
   ) !!calendar
@@ -390,8 +403,8 @@ let user_reader o telnet sock nread  =
         if cmd <> "" then begin
             before_telnet_output o sock;
             let buf = o.conn_buf in
-            Buffer.clear buf;
-            if o.conn_output = ANSI then Printf.bprintf buf "> $b%s$n\n" cmd;
+            Buffer.reset buf;
+            if o.conn_output = ANSI then Printf.bprintf buf "> $c%s$n\n" cmd;
             eval telnet.telnet_auth cmd o;
             Buffer.add_char buf '\n';
             if o.conn_output = ANSI then Buffer.add_string buf "$n";
@@ -438,10 +451,8 @@ type telnet_conn = {
 
 let iac_will_naws = "\255\253\031"
 
-let user_reader o telnet sock nread  =
+let user_reader o telnet sock nread =
   let b = TcpBufferedSocket.buf sock in
-  let end_pos = b.pos + b.len in
-  let new_pos = end_pos - nread in
   let rec iter () =
     if b.len > 0 then
       let c = b.buf.[b.pos] in
@@ -513,7 +524,7 @@ let user_reader o telnet sock nread  =
               before_telnet_output o sock;
               let buf = o.conn_buf in
               Buffer.reset buf; 
-              if o.conn_output = ANSI then Printf.bprintf buf "> $b%s$n\n" cmd;
+              if o.conn_output = ANSI then Printf.bprintf buf "> $c%s$n\n" cmd;
               eval telnet.telnet_auth cmd o;
               Buffer.add_char buf '\n';
               if o.conn_output = ANSI then Buffer.add_string buf "$n";
@@ -560,7 +571,6 @@ let telnet_handler t event =
   match event with
     TcpServerSocket.CONNECTION (s, Unix.ADDR_INET (from_ip, from_port)) ->
       let from_ip = Ip.of_inet_addr from_ip in
-      if Ip.matches from_ip !!allowed_ips then
         let token = create_token unlimited_connection_manager in
         let sock = TcpBufferedSocket.create_simple token
           "telnet connection"
@@ -580,6 +590,7 @@ let telnet_handler t event =
             conn_width = 80;
             conn_height = 0;
           } in
+      if Ip.matches from_ip !!allowed_ips then begin
         TcpBufferedSocket.prevent_close sock;
         TcpBufferedSocket.set_max_output_buffer sock !!interface_buffer;
         TcpBufferedSocket.set_reader sock (user_reader o telnet);
@@ -589,16 +600,24 @@ let telnet_handler t event =
         TcpBufferedSocket.write_string sock iac_will_naws;
 
         before_telnet_output o sock;
-        TcpBufferedSocket.write_string sock (text_of_html !!motd_html);
+        TcpBufferedSocket.write_string sock
+	   (Printf.sprintf "Welcome to MLDonkey %s" Autoconf.current_version);
 
         TcpBufferedSocket.write_string sock (dollar_escape o false
-            "\n$bWelcome on mldonkey command-line$n\n\nUse $r?$n for help\n\n");
+            "\n$cWelcome on mldonkey command-line$n\n\nUse $r?$n for help\n\n");
 
         after_telnet_output o sock
+	end
       else begin
-        lprintf_nl () "Telnet connection from %s rejected (see allowed_ips setting)"
-          (Ip.to_string from_ip);
-        Unix.close s
+        before_telnet_output o sock;
+	let reject_message =
+	  Printf.sprintf "Telnet connection from %s rejected (see allowed_ips setting)\n"
+	    (Ip.to_string from_ip)
+	in
+	TcpBufferedSocket.write_string sock (dollar_escape o false reject_message);
+	shutdown sock Closed_connect_failed;
+	lprintf_n () "%s" reject_message;
+	Unix.close s
       end
 
   | _ -> ()
@@ -704,7 +723,18 @@ type file_ext =
 | JPEG
 | JAVASCRIPT
 | MPEG
+| AVI
+| WMV
+| ASF
+| MOV
+| OGM
+| RM
+| MKV
 | PNG
+| GIF
+| MP3
+| WMA
+| OGG
 | TEXTS
 | UNKN
 | WML
@@ -718,12 +748,32 @@ let extension_to_file_ext extension =
   | "htm"
   | "html" -> HTMLS
   | "ico" -> ICON
+  | "jpe"
   | "jpeg"
   | "jpg" -> JPEG
   | "js" -> JAVASCRIPT
+  | "mpe"
   | "mpeg"
   | "mpg" -> MPEG
+  | "avi" -> AVI
+  | "wmv" -> WMV
+  | "asf" -> ASF
+  | "mov"
+  | "movie"
+  | "qt" -> MOV
+  | "ogm" -> OGM
+  | "ra"
+  | "ram"
+  | "rm"
+  | "rmvb"
+  | "rv9"
+  | "rt" -> RM
+  | "mkv" -> MKV
   | "png" -> PNG
+  | "gif" -> GIF
+  | "mp3" -> MP3
+  | "wma" -> WMA
+  | "ogg" -> OGG
   | "txt" -> TEXTS
   | "wml" -> WML
   | _ -> UNKN
@@ -738,7 +788,18 @@ let ext_to_file_type ext =
   | JAVASCRIPT -> TXT
   | JPEG -> BIN
   | MPEG -> BIN
+  | AVI -> BIN
+  | WMV -> BIN
+  | ASF -> BIN
+  | MOV -> BIN
+  | OGM -> BIN
+  | RM -> BIN
+  | MKV -> BIN
   | PNG -> BIN
+  | GIF -> BIN
+  | MP3 -> BIN
+  | WMA -> BIN
+  | OGG -> BIN
   | TEXTS -> TXT
   | WML -> TXT
 
@@ -752,7 +813,18 @@ let ext_to_mime_type ext =
   | JAVASCRIPT -> "text/javascript"
   | JPEG -> "image/jpg"
   | MPEG -> "video/mpeg"
+  | AVI -> "video/x-msvideo"
+  | WMV -> "video/x-ms-wmv"
+  | ASF -> "video/x-ms-asf"
+  | MOV -> "video/quicktime"
+  | OGM -> "application/ogg" (* is that correct ? *)
+  | RM -> "audio/x-pn-realaudio"
+  | MKV -> "video/x-matroska" (* is that correct ? *)
   | PNG -> "image/png"
+  | GIF -> "image/gif"
+  | MP3 -> "audio/mpeg"
+  | WMA -> "audio/x-ms-wma"
+  | OGG -> "application/ogg" (* is that correct ? *)
   | TEXTS -> "text/plain"
   | WML -> "text/vnd.wap.wml"
 
@@ -771,7 +843,7 @@ let read_theme_page page =
         let file = open_in theme_page in
         let size = (Unix.stat theme_page).Unix.st_size in
         let s = String.make size ' ' in
-        let ok = really_input file s 0 size in
+        let _ = really_input file s 0 size in
         close_in file; s
 
 let add_simple_commands buf =
@@ -787,7 +859,7 @@ let http_add_gen_header r =
   add_reply_header r "Connection" "close"
 
 let add_gzip_headers r =
-  if Autoconf.has_zlib && !!html_use_gzip then begin
+  if !!html_use_gzip then begin
     add_reply_header r "Content-Encoding" "gzip";
     add_reply_header r "Vary" "Accept-Encoding";
   end
@@ -814,8 +886,16 @@ let http_add_bin_header r ext clen =
   http_file_type := ext_to_file_type ext;
   http_add_gen_header r;
   add_reply_header r "Content-Type" (ext_to_mime_type ext);
-  http_add_bin_info_header r clen;
-  add_gzip_headers r
+  http_add_bin_info_header r clen
+
+let http_add_bin_stream_header r ext =
+  http_file_type := BIN;
+  http_add_gen_header r;
+  let mime_type = ext_to_mime_type ext in
+  let mime_type = if mime_type <> "" then mime_type
+  else "application/binary" in
+  add_reply_header r "Content-Type" mime_type;
+  add_reply_header r "Accept-Ranges" "bytes"
 
 let http_send_bin r buf filename =
   let file_to_send = File.to_string filename in
@@ -886,6 +966,55 @@ let clear_page buf =
   Buffer.reset buf;
   http_file_type := UNK
 
+let send_preview r file fd size filename exten =
+  let (begin_pos, end_pos) =
+    try
+      let (begin_pos, end_pos) = request_range r in
+      let end_pos = match end_pos with
+          None -> size
+        | Some end_pos -> end_pos in
+      let range_size = end_pos -- begin_pos in
+      add_reply_header r "Content-Length"
+        (Int64.to_string range_size);
+      add_reply_header r "Content-Range"
+        (Printf.sprintf "bytes %Ld-%Ld/%Ld"
+          begin_pos (end_pos -- one)
+        size);
+      r.reply_head <- "206 Partial Content";
+      begin_pos, end_pos
+    with _ ->
+        add_reply_header r "Content-Length"
+          (Int64.to_string size);
+        zero, size
+  in
+  let len = String.length exten in
+  let exten = if len = 0 then exten
+      else String.lowercase (String.sub exten 1 (len - 1)) in
+  http_add_bin_stream_header r (extension_to_file_ext exten);
+
+  let s = String.create 200000 in
+  set_max_output_buffer r.sock (String.length s);
+  set_rtimeout r.sock 10000.;
+  let rec stream_file file pos sock =
+    let max = (max_refill sock) - 1 in
+    if max > 0 && !pos < end_pos then
+      let max64 = min (end_pos -- !pos) (Int64.of_int max) in
+      let max = Int64.to_int max64 in
+      Unix32.read fd !pos s 0 max;
+      pos := !pos ++ max64;
+      set_lifetime sock 60.;
+(*                      lprintf "HTTPSEND: refill %d %Ld\n" max !pos;*)
+(*                    lprintf "HTTPSEND: [%s]\n" (String.escaped
+                        (String.sub s 0 max)); *)
+      write sock s 0 max;
+      if output_buffered sock = 0 then begin
+(*                          lprintf "Recursing STREAM\n"; *)
+          stream_file file pos sock
+        end
+  in
+  r.reply_stream <- Some (stream_file file (ref begin_pos))
+
+
 let http_handler o t r =
   CommonInteractive.display_vd := false;
   CommonInteractive.display_bw_stats := false;
@@ -950,8 +1079,16 @@ let http_handler o t r =
               Printf.bprintf buf "<p align=\"left\"><small>";
               let mfiles = List2.tail_map file_info !!files in
               List.iter (fun file ->
-                  Printf.bprintf buf  "<a href=\"wap.wml?%s=%d\">%s</a> <a href=\"wap.wml?VDC=%d\">C</a> [%-5d] %5.1f %s %s/%s <br />" (if downloading file then "VDP" else "VDR" ) (file.file_num) (if downloading file then "P" else "R" ) (file.file_num) (file.file_num) (file.file_download_rate /. 1024.)(short_name file) (print_human_readable file (Int64.sub file.file_size file.file_downloaded)) (print_human_readable file file.file_size);
-
+                  Printf.bprintf buf  "<a href=\"wap.wml?%s=%d\">%s</a> <a href=\"wap.wml?VDC=%d\">C</a> [%-5d] %5.1f %s %s/%s <br />" 
+                    (if downloading file then "VDP" else "VDR" ) 
+                    (file.file_num) 
+                    (if downloading file then "P" else "R" ) 
+                    (file.file_num) 
+                    (file.file_num) 
+                    (file.file_download_rate /. 1024.)
+                    (short_name file) 
+                    (print_human_readable file (file.file_size -- file.file_downloaded)) 
+                    (print_human_readable file file.file_size);
               ) mfiles;
               Printf.bprintf buf "<br />Downloaded %d/%d files " (List.length !!done_files) (List.length !!files);
               Printf.bprintf buf "</small></p>";
@@ -1008,7 +1145,12 @@ let http_handler o t r =
 
         | "oneframe.html" ->
             html_open_page buf t r true;
-            Buffer.add_string buf !!motd_html
+            Buffer.add_string buf !!motd_html;
+	    Buffer.add_string buf "<p><pre>";
+	    ignore (CommonComplexOptions.buildinfo false buf);
+	    Buffer.add_string buf "\n";
+	    ignore (CommonComplexOptions.runinfo false buf);
+	    Buffer.add_string buf "</pre>\n"
 
         | "bw_updown.png" ->
             (match http_error_no_gd "png" with
@@ -1230,7 +1372,8 @@ let http_handler o t r =
             ) r.get_url.Url.args;
             let b = Buffer.create 10000 in
 
-            DriverInteractive.display_file_list b o;
+            let list = (List2.tail_map file_info !!files) in
+            DriverInteractive.display_file_list b o list;
             html_open_page buf t r true;
             Buffer.add_string buf (html_escaped (Buffer.contents b))
 
@@ -1306,52 +1449,36 @@ let http_handler o t r =
                   let file = file_find file_num in
                   let fd = file_fd file in
                   let size = file_size file in
+		  let filename = file_best_name file in
+		  let exten = Filename2.last_extension filename in
+		    send_preview r file fd size filename exten
 
-                  let (begin_pos, end_pos) =
-                    try
-                      let (begin_pos, end_pos) = request_range r in
-                      let end_pos = match end_pos with
-                          None -> size
-                        | Some end_pos -> end_pos in
-                      let range_size = end_pos -- begin_pos in
-                      add_reply_header r "Content-Length"
-                        (Int64.to_string range_size);
-                      add_reply_header r "Content-Range"
-                        (Printf.sprintf "bytes %Ld-%Ld/%Ld"
-                          begin_pos (end_pos -- one)
-                        size);
-                      r.reply_head <- "206 Partial Content";
-                      begin_pos, end_pos
-                    with _ ->
-                        add_reply_header r "Content-Length"
-                          (Int64.to_string size);
-                        zero, size
-                  in
+              | args ->
+                  List.iter (fun (s,v) ->
+                      lprintf_nl () "[%s]=[%s]" (String.escaped s) (String.escaped v))
+                  args;
+                  raise Not_found
+            end
 
-                  add_reply_header r "Content-type" "application/binary";
-                  add_reply_header r "Accept-Ranges" "bytes";
-
-                  let s = String.create 200000 in
-                  set_max_output_buffer r.sock (String.length s);
-                  set_rtimeout r.sock 10000.;
-                  let rec stream_file file pos sock =
-                    let max = (max_refill sock) - 1 in
-                    if max > 0 && !pos < end_pos then
-                      let max64 = min (end_pos -- !pos) (Int64.of_int max) in
-                      let max = Int64.to_int max64 in
-                      Unix32.read fd !pos s 0 max;
-                      pos := !pos ++ max64;
-                      set_lifetime sock 60.;
-(*                      lprintf "HTTPSEND: refill %d %Ld\n" max !pos;*)
-(*                    lprintf "HTTPSEND: [%s]\n" (String.escaped
-                        (String.sub s 0 max)); *)
-                      write sock s 0 max;
-                      if output_buffered sock = 0 then begin
-(*                          lprintf "Recursing STREAM\n"; *)
-                          stream_file file pos sock
-                        end
-                  in
-                  r.reply_stream <- Some (stream_file file (ref begin_pos))
+        | "preview_upload" ->
+            begin
+              clear_page buf;
+              match r.get_url.Url.args with
+                ["q", file_num] ->
+                  let file_num = int_of_string file_num in
+		  let file = shared_find file_num in
+		  let impl = as_shared_impl file in
+		  let info = shared_info file in
+		  let filename = impl.impl_shared_fullname in
+		  let exten = Filename2.last_extension impl.impl_shared_codedname in
+		  if not (Sys.file_exists filename) then
+		    begin
+		      lprintf_nl () "file %s not found" filename;
+		      raise Not_found
+		    end;
+		  let fd = Unix32.create_ro filename in
+		  let size = info.shared_size in
+		    send_preview r file fd size filename exten
 
               | args ->
                   List.iter (fun (s,v) ->
@@ -1417,7 +1544,10 @@ let http_handler o t r =
     | BIN -> Buffer.contents buf
     | UNK -> "Unknown type for content :" ^ (Buffer.contents buf)
   in
-  r.reply_content <- if Autoconf.has_zlib && !!html_use_gzip then Autoconf.zlib__gzip_string s else s
+  r.reply_content <- 
+    if !http_file_type <> BIN && !!html_use_gzip then 
+      Autoconf.zlib__gzip_string s 
+    else s
 
 let http_options = {
     conn_buf = Buffer.create 10000;
@@ -1438,6 +1568,6 @@ let create_http_handler () =
       default = http_handler http_options;
     } in
   option_hook allowed_ips (fun _ -> config.addrs <- !!allowed_ips);
-  let sock = find_port "http server" !!http_bind_addr http_port
-      (Http_server.handler config) in
+  ignore(find_port "http server" !!http_bind_addr http_port
+      (Http_server.handler config));
   config.port <- !!http_port
