@@ -19,6 +19,7 @@
 
 open Md4
 
+open Printf2
 open CommonClient
 open CommonNetwork
 open CommonResult
@@ -378,11 +379,9 @@ function cancelAll(x){for(i=0;i\\<document.selectForm.elements.length;i++){var j
   |]
     (List.map (fun file ->
         [|
-          (Printf.sprintf "[\\<a href=\\\"submit\\?q\\=vd+%d\\\"\\>%-5d\\</a\\> \\<a target=_blank href=http://donkeyfakes.gambri.net/fakecheck/update/fakecheck.php\\?size\\=%s\\&md4=%s\\>%s\\</a\\>]"
+          (Printf.sprintf "[\\<a href=\\\"submit\\?q\\=vd+%d\\\"\\>%-5d\\</a\\> %s]"
               file.file_num
               file.file_num
-              (Int64.to_string file.file_size)
-            (Md4.to_string file.file_md4)
             (net_name file)
           );
 
@@ -457,13 +456,13 @@ function cancelAll(x){for(i=0;i\\<document.selectForm.elements.length;i++){var j
 	let qnum = ref 0 in
 
 	List.iter (fun file ->
-	 tsize := Int64.add !tsize file.file_size;
-     tdl := Int64.add !tdl file.file_downloaded;
+	 tsize := !tsize ++ file.file_size;
+     tdl := !tdl ++ file.file_downloaded;
 	 trate := !trate +. file.file_download_rate;
 
 	 if file.file_state = FileQueued then begin
-		qsize := Int64.add !qsize file.file_size;
-		qdl := Int64.add !qdl file.file_downloaded;
+		qsize := !qsize ++ file.file_size;
+		qdl := !qdl ++ file.file_downloaded;
 		incr qnum;
 	 end;
 
@@ -534,7 +533,7 @@ if !!html_mods_use_js_tooltips then Printf.bprintf buf
 \\<td title=\\\"Cancel\\\" class=\\\"dlheader brs\\\"\\>C\\</td\\>"
 (if !qnum > 0 then begin
 	Printf.sprintf "title=\\\"Active(%d): %s/%s | Queued(%d): %s/%s\\\""
-	(List.length guifiles - !qnum) (size_of_int64 (Int64.sub !tdl !qdl)) (size_of_int64 (Int64.sub !tsize !qsize))
+	(List.length guifiles - !qnum) (size_of_int64 (!tdl -- !qdl)) (size_of_int64 (!tsize -- !qsize))
 	!qnum (size_of_int64 !qdl) (size_of_int64 !qsize);
 end
 else "")
@@ -656,13 +655,13 @@ let ctd fn td = Printf.sprintf "\\<td onClick=\\\"location.href='submit?q=vd+%d'
 
 
           (if !!html_mods_vd_age then
-			ctd file.file_num (let age = (BasicSocket.last_time ()) - file.file_age in time_to_string age)
+			ctd file.file_num (let age = (BasicSocket.last_time ()) - file.file_age in Date.time_to_string age "long")
 		   else "");
 
           (if !!html_mods_vd_last then
 			ctd file.file_num (if file.file_last_seen > 0
              then let last = (BasicSocket.last_time ()) - file.file_last_seen in
-                time_to_string last
+                Date.time_to_string last "long"
              else "-"
           	)
 			else ""
@@ -678,7 +677,7 @@ let ctd fn td = Printf.sprintf "\\<td onClick=\\\"location.href='submit?q=vd+%d'
           );
 
           (ctd file.file_num (if (file.file_download_rate < 10.24 || stalled file) then "-"
-				else time_to_string (calc_file_eta file)) );
+				else Date.time_to_string (calc_file_eta file) "long"));
 
           (if !!html_mods_vd_prio then
 	      (Printf.sprintf "\\<td class=\\\"dl ar\\\"\\>\\<div id=\\\"divSelectPriority%d\\\"\\>\\<select id=\\\"selectPriority%d\\\" name=\\\"selectPriority%d\\\"
@@ -816,7 +815,7 @@ let simple_print_file_list finished buf files format =
                     "-", "$n"
                   else
                     Printf.sprintf "%4.1f" (
-                      file.file_download_rate /. 1024.), "$b"
+                      file.file_download_rate /. 1024.), "$c"
             in
             [|
               (Printf.sprintf "%0s[%0s]%0s"
@@ -894,12 +893,8 @@ let simple_print_file_list finished buf files format =
           |]
       ) files)
 
-let display_file_list buf o =
-  display_vd := true;
-  if not (use_html_mods o) then
-(*    Printf.bprintf buf "Downloaded %d/%d files\n" (List.length !!done_files)
-    (List.length !!files); *)
-    Printf.bprintf buf "\nDown: %.1f KB/s ( %d + %d ) | Up: %.1f KB/s ( %d + %d ) | Shared: %d/%s"
+let print_bw_stats buf =
+  Printf.bprintf buf "Down: %.1f KB/s ( %d + %d ) | Up: %.1f KB/s ( %d + %d ) | Shared: %d/%s | Uploaded: %s"
        (( (float_of_int !udp_download_rate) +. (float_of_int !control_download_rate)) /. 1024.0)
          !udp_download_rate
          !control_download_rate
@@ -907,18 +902,25 @@ let display_file_list buf o =
          !udp_upload_rate
          !control_upload_rate
          !nshared_files
-         (size_of_int64 !upload_counter);
+  (size_of_int64 !nshared_bytes)
+  (size_of_int64 !upload_counter)
 
+let display_active_file_list buf o list =
+  display_vd := true;
+
+  if not (use_html_mods o) then begin
+(*    Printf.bprintf buf "Downloaded %d/%d files\n" (List.length !!done_files)
+    (List.length !!files); *)
+    print_bw_stats buf;
     Printf.bprintf buf "\n";
+  end;
 
   if o.conn_output <> HTML && !!improved_telnet then
   begin
-    let list = List2.tail_map file_info !!files in
     let list = Sort.list (fun f1 f2 -> percent f1 >= percent f2) list in
     simple_print_file_list false buf list o
   end
   else
-  let list = List2.tail_map file_info !!files in
   let list =
     try
       let sorter =
@@ -948,8 +950,8 @@ let display_file_list buf o =
   in
   simple_print_file_list false buf list o
 
-let display_file_list buf o =
-  display_file_list buf o;
+let display_file_list buf o l =
+  display_active_file_list buf o l;
   if not (use_html_mods o) then
     Printf.bprintf buf "%0sDownloaded %d files\n" (if !!term_ansi then "$n" else "") (List.length !!done_files);
   if !!done_files <> [] then begin
@@ -993,7 +995,7 @@ let old_print_search buf o results =
       ( "1", "1", "srh ar", "Size", "Size" ) ;
       ( "1", "1", "srh ar", "Availability", "A" ) ;
       ( "1", "1", "srh ar", "Complete Sources", "C" ) ;
-      ( "3", "0", "srh", "Hash (click for lookup)", "Hash check" ) ;
+      ( "2", "0", "srh", "Hash (click for lookup)", "Hash check" ) ;
       ( "1", "1", "srh ar", "Length", "Len" ) ;
       ( "1", "1", "srh ar", "Codec", "Code" ) ;
       ( "1", "1", "srh ar", "Bitrate", "Rate" ) ;
@@ -1078,7 +1080,7 @@ let old_print_search buf o results =
                     Printf.bprintf buf "%s\n" (shorten name !!max_name_len);
                     List.iter (fun s ->
                         if use_html_mods o then Printf.bprintf buf "\\<BR\\>";
-                        Printf.bprintf buf "       %s\n" s
+                        Printf.bprintf buf "       %s\n" (shorten s !!max_name_len)
                     ) names;
               end;
               if r.result_done then
@@ -1131,7 +1133,6 @@ let old_print_search buf o results =
                 Printf.bprintf buf "\\<td class=\\\"sr ar\\\"\\>%s\\</td\\>
 			\\<td class=\\\"sr ar\\\"\\>%s\\</td\\>
 			\\<td class=\\\"sr ar\\\"\\>%s\\</td\\>
-			\\<td class=\\\"sr\\\"\\>\\<a href=\\\"http://donkeyfakes.gambri.net/fakecheck.php?hash=%s\\\"\\>DF\\</a\\>\\</td\\>
 			\\<td class=\\\"sr\\\"\\>\\<a href=\\\"http://bitzi.com/lookup/ed2k:%s\\\"\\>BI\\</a\\>\\</td\\>
 			\\<td class=\\\"sr\\\"\\>\\<a href=\\\"http://www.filedonkey.com/url/%s\\\"\\>FD\\</a\\>\\</td\\>
 			\\<td class=\\\"sr ar\\\"\\>%s\\</td\\>
@@ -1142,7 +1143,6 @@ let old_print_search buf o results =
                   !csource
 		  real_hash
 		  real_hash
-		  real_hash
 		  !clength
 		  (if !ccodec = "" then
 		     begin
