@@ -384,11 +384,6 @@ let gui_closed gui sock  msg =
 let gui_initialize gui = 
 
   gui.gui_initialized <- true;
-  let connecting = ref true in
-  networks_iter_all (fun n ->
-      gui_send gui (Network_info (network_info n)));
-  gui_send gui (Console (DriverControlers.text_of_html !!motd_html));
-  
   networks_iter_all (fun n ->
       gui_send gui (Network_info (network_info n)));
   gui_send gui (Console (DriverControlers.text_of_html !!motd_html));
@@ -506,7 +501,7 @@ lprintf "Sending for %s\n" prefix;
           List.iter (fun file ->
               
               List.iter (fun s ->
-                  let section = section_name s in
+                  let _ = section_name s in
                   List.iter (fun o ->
                       gui_send gui (
                         P.Add_plugin_option (r.network_name, o)
@@ -578,6 +573,7 @@ let gui_reader (gui: gui_record) t _ =
           | _ ->
               guis := gui :: !guis;
               gui.gui_auth <- true;
+              gui.gui_conn.conn_user <- find_ui_user user;
               
               if not gui.gui_initialized then 
                 gui_initialize gui;
@@ -665,7 +661,7 @@ let gui_reader (gui: gui_record) t _ =
                     network_extend_search r s e)
           
           | P.KillServer -> 
-              exit_properly 0
+              CommonInteractive.clean_exit 0
           
           | P.Search_query s ->
               
@@ -735,7 +731,7 @@ search.op_search_end_reply_handlers;
                 if not (networks_iter_until_true
                     (fun n ->
                        try
-                         network_parse_url n url
+                         let s,r = network_parse_url n url in r
                        with e ->
                          lprintf "Exception %s for network %s\n"
                            (Printexc2.to_string e) (n.network_name);
@@ -754,7 +750,7 @@ search.op_search_end_reply_handlers;
                     H.req_proxy = !CommonOptions.http_proxy;
                     H.req_request = H.HEAD;
                     H.req_user_agent = 
-                       Printf.sprintf "MLdonkey/%s" Autoconf.current_version;
+                       Printf.sprintf "MLDonkey/%s" Autoconf.current_version;
                 } in
                 H.whead r 
                   (fun headers ->
