@@ -50,7 +50,18 @@ open Gettext (* open last  as most modules redefine _s and _b *)
 let _s x = _s "DriverMain" x
 let _b x = _b "DriverMain" x
 
+(* prints a new logline with date, module and starts newline *)
+let lprintf_nl () =
+  lprintf "%s"
+    (log_time ()); lprintf_nl2
+
+(* prints a new logline with date, module and does not start newline *)
+let lprintf_n () =
+  lprintf "%s"
+    (log_time ()); lprintf
+
 let keep_console_output = ref false
+let pid = ref ""
 
 let do_daily () =
   incr CommonWeb.days
@@ -72,7 +83,7 @@ let minute_timer () =
 
 let hourly_timer timer =
   incr CommonWeb.hours;
-  CommonWeb.load_web_infos ();
+  CommonWeb.load_web_infos false;
   if !CommonWeb.hours mod !!compaction_delay = 0 then Gc.compact ();
   if !!backup_options_delay <> 0
      && !CommonWeb.hours mod !!backup_options_delay = 0 then
@@ -86,11 +97,11 @@ let second_timer timer =
   (try
       update_link_stats ()
     with e ->
-        lprintf_nl (_b "Exception %s") (Printexc2.to_string e));
+        lprintf_nl () (_b "Exception %s") (Printexc2.to_string e));
 	  (try
      CommonUploads.refill_upload_slots ()
    with e ->
-        lprintf_nl (_b "Exception %s") (Printexc2.to_string e));
+        lprintf_nl () (_b "Exception %s") (Printexc2.to_string e));
   CommonUploads.reset_upload_timer ();
   CommonUploads.shared_files_timer ();
   ()
@@ -101,7 +112,7 @@ let start_interfaces () =
   if !!http_port <> 0 then begin try
         ignore (DriverControlers.create_http_handler ());
       with e ->
-          lprintf_nl (_b "Exception %s while starting HTTP interface")
+          lprintf_nl () (_b "Exception %s while starting HTTP interface")
             (Printexc2.to_string e);
     end;
 
@@ -130,7 +141,7 @@ let start_interfaces () =
 
 let _ =
   CommonWeb.add_web_kind "motd.html" (fun _ filename ->
-      lprintf_nl (_b "motd.html changed");
+      lprintf_nl () (_b "motd.html changed");
     motd_html =:= File.to_string filename
   );
   CommonWeb.add_web_kind "motd.conf" (fun _ filename ->
@@ -148,20 +159,23 @@ let _ =
 	| "del_item" ->
             CommonInteractive.del_item_from_fully_qualified_options name value
 	| _ ->
-	    lprintf_nl (_b "UNUSED LINE: %s") line
+	    lprintf_nl () (_b "UNUSED LINE: %s") line
 
       done;
     with
     | End_of_file ->
 	close_in ic
-    | e -> lprintf_nl (_b "Error while reading motd.conf(%s): %s") filename
+    | e -> lprintf_nl () (_b "Error while reading motd.conf(%s): %s") filename
 	(Printexc2.to_string e);
 	close_in ic
   );
   CommonWeb.add_web_kind "guarding.p2p" (fun _ filename ->
-      Ip_set.bl := Ip_set.load filename
+      Ip_set.bl := Ip_set.load filename;
 (*      Ip_set.bl := Ip_set.load_merge !Ip_set.bl filename *)
-  )
+      CommonServer.check_blocked_servers ()
+  );
+  CommonWeb.add_web_kind "geoip.dat" (fun _ filename ->
+      Geoip.init (Geoip.unpack filename))
 
 
 
@@ -237,10 +251,7 @@ while (my $uri = shift @ARGV) {
     "admin" ""
   in
   File.from_string (Filename.concat file_basedir "mldonkey_submit") file;
-  try
-  Unix.chmod  (Filename.concat file_basedir "mldonkey_submit") 0o755
-  with
-  e -> ()
+  Unix2.chmod (Filename.concat file_basedir "mldonkey_submit") 0o755
 
 let load_config () =
 
@@ -253,48 +264,20 @@ let load_config () =
   let exists_users_ini =
     Sys.file_exists (options_file_name users_ini)
   in
-  if not exists_downloads_ini then
+  if not exists_users_ini && exists_downloads_ini then
     begin
-      let oc = open_out (options_file_name downloads_ini) in
-      close_out oc;
-      if not exists_users_ini then
-        begin
-          let oc = open_out (options_file_name users_ini) in
-          close_out oc;
-        end;
-    end
-  else
-    if not exists_users_ini then
-      begin
-        lprintf_nl "No config file (users.ini) found. Importing users from downloads.ini.";
-        ( try Unix2.copy "downloads.ini" "users.ini" with _ -> () );
-      end;
+      lprintf_nl () "No config file (users.ini) found. Importing users from downloads.ini.";
+      ( try Unix2.copy "downloads.ini" "users.ini" with _ -> () );
+    end;
 
-(*
-  let exists_expert_ini = Sys.file_exists
-      (options_file_name downloads_expert_ini) in
-  if not exists_expert_ini then begin
-      if exists_downloads_ini then begin
-          lprintf "Using old config file\n";
-          (try Unix2.copy "downloads.ini" "downloads_expert.ini" with _ -> ());
-          (try Unix2.copy "downloads.ini" "donkey.ini" with _ -> ());
-          (try Unix2.copy "downloads.ini" "donkey_expert.ini" with _ -> ());
+  let ini_files_exist = Sys.file_exists (options_file_name downloads_ini) in
 
-          end else begin
-
-          lprintf "No config file found. Generating one.\n";
-          let oc = open_out (options_file_name downloads_expert_ini) in
-          close_out oc;
-        end
-    end; *)
   (try
       Options.load downloads_ini;
       Options.load users_ini;
-(*      Options.load downloads_expert_ini;       *)
     with e ->
-        lprintf_nl "Exception %s during options load" (Printexc2.to_string e);
-        exit 2;
-        ());
+        lprintf_nl () "Exception %s during options load" (Printexc2.to_string e);
+        exit 70);
 
   (* Here, we try to update options when a new version of mldonkey is
      used. For example, we can add new web_infos... *)
@@ -302,8 +285,8 @@ let load_config () =
 
   CommonMessages.load_message_file ();
   if !!html_mods then begin
-      if !!html_mods_style > 0 && !!html_mods_style < (Array.length !html_mods_styles) then
-        commands_frame_height =:= (snd !html_mods_styles.(!!html_mods_style));
+      if !!html_mods_style > 0 && !!html_mods_style < Array.length CommonMessages.styles then
+        commands_frame_height =:= CommonMessages.styles.(!!html_mods_style).CommonMessages.frame_height;
       CommonMessages.colour_changer ();
     end;
   networks_iter_all (fun r ->
@@ -341,14 +324,14 @@ let load_config () =
 
   Arg.parse ([
       "-v", Arg.Unit (fun _ ->
-          lprintf_nl "%s" (CommonGlobals.version ());
+          lprintf_nl () "%s" (CommonGlobals.version ());
           exit 0), _s " : print version number and exit";
       "-exit", Arg.Unit (fun _ -> exit 0), ": exit immediatly";
       "-format", Arg.String (fun file ->
-          let format = CommonMultimedia.get_info file in
-          ()), _s  " <filename> : check file format";
+          ignore (CommonMultimedia.get_info file)),
+          _s  " <filename> : check file format";
       "-test_ip", Arg.String (fun ip ->
-          lprintf_nl "%s = %s" ip (Ip.to_string (Ip.of_string ip));
+          lprintf_nl () "%s = %s" ip (Ip.to_string (Ip.of_string ip));
           exit 0), _s "<ip> : undocumented";
       "-check_impl", Arg.Unit (fun _ ->
           CommonNetwork.check_network_implementations ();
@@ -371,10 +354,13 @@ let load_config () =
        _s ": keep output to stderr after startup";
       "-daemon", Arg.Unit (fun _ ->
           (* Removed due to savannah bug #11514 . *)
-          lprintf_nl "\n\nOption -daemon was removed.\nUse 'mlnet > /dev/null 2>&1 &' instead. Exiting...";
-          exit 0), _s " : this argument was removed, core will exit";
+          lprintf_nl () "\n\nOption -daemon was removed.\nUse 'mlnet > /dev/null 2>&1 &' instead. Exiting...";
+          exit 64), _s " : this argument was removed, core will exit";
       "-find_port", Arg.Set find_other_port,
       _s " : find another port when one is already used";
+      "-pid", Arg.String (fun s -> pid := s;
+      ),
+       _s ": directory for pid file";
     ] @
       !more_args
       @
@@ -383,32 +369,9 @@ let load_config () =
 (*      Files.dump_file file; exit 0 *)
   ) "";
 
+  if not ini_files_exist && not !keep_console_output then log_file =:= "mlnet.log";
 
-(**** CREATE DIRS   ****)
-
-  (*
-  Unix2.safe_mkdir !!incoming_directory;
-
-  File.from_string (Filename.concat !!incoming_directory "Readme.txt")
-  "This directory contains the files downloaded, after commit.
-The 'incoming/files/' folder contains simple files.
-The 'incoming/directories/' folder contains whole directories (probably
-   downloaded with Bittorrent).
-
-If you want to share files, you should:
-  - Put simple files or directories where all files should be shared separately
-   in the 'incoming/files/' folder, or better, in the shared/ folder.
-   is for downloaded files. Files to be shared should be
-   put in the 'shared/' directory instead.
-  - Put directories that should be shared as one file in the
-   'incoming/directories/' folder. Currently, such directories can only be
-   shared on the Bittorrent network, by providing the corresponding
-   .torrent in the 'torrents/seeded/' folder.
-";
-
-  Unix2.safe_mkdir (Filename.concat !!incoming_directory "files");
-  Unix2.safe_mkdir (Filename.concat !!incoming_directory "directories");
-*)
+  (**** CREATE DIRS   ****)
 
   List.iter (fun s ->
       Unix2.safe_mkdir s.shdir_dirname;
@@ -426,9 +389,9 @@ let _ =
   let t = Unix.localtime (Unix.time ()) in
   if (t.Unix.tm_year<=104) then
     begin
-      lprintf_nl (_b "\n\n\nYour system has a system date earlier than 2004, please correct it.");
-      lprintf_nl (_b "MLdonkey can not work with such a system date, exiting...");
-      CommonGlobals.exit_properly 0
+      lprintf_nl () (_b "\n\n\nYour system has a system date earlier than 2004, please correct it.");
+      lprintf_nl () (_b "MLdonkey can not work with such a system date, exiting...");
+      CommonGlobals.exit_properly 71
     end;
 
   (
@@ -474,13 +437,13 @@ or getting a binary compiled with glibc %s.\n\n")
   end;
 
 (*  lprintf "(3) networks_iter load_complex_options\n"; *)
-  lprintf_nl (_b "Check http://www.mldonkey.net/ for updates");
+  lprintf_nl () (_b "Check http://www.mldonkey.net/ for updates");
   networks_iter (fun r -> network_load_complex_options r);
   lprintf (_b "%senabling networks: ") (log_time ());
   networks_iter (fun r ->
 (*      lprintf "(4) networks_iter enabling\n"; *)
-      network_enable r;
       lprintf (_b "%s ") r.network_name;
+      network_enable r;
 (* are there drawbacks to start recover_temp unconditionally here ? *)
       if !!recover_temp_on_startup then
         network_recover_temp r;
@@ -495,11 +458,10 @@ or getting a binary compiled with glibc %s.\n\n")
 	  end);
   if not !found then lprintf (_b "none");
   lprint_newline ();
+  networks_iter_all (fun n -> network_update_options n);
   CommonOptions.start_running_plugins := true;
   CommonInteractive.force_download_quotas ();
 
-  TcpBufferedSocket.set_max_opened_connections
-    (fun _ -> !!max_opened_connections);
   TcpBufferedSocket.set_max_connections_per_second
     (fun _ -> !!max_connections_per_second);
 
@@ -521,29 +483,19 @@ or getting a binary compiled with glibc %s.\n\n")
   Options.prune_file downloads_ini;
   Options.prune_file users_ini;
 (*  Options.prune_file downloads_expert_ini; *)
-  add_timer 10. (fun _ -> try CommonWeb.load_web_infos () with _ -> ());
-  lprintf_nl (_b "To command: telnet %s %d")
+  add_timer 5. (fun _ -> try CommonWeb.load_web_infos true with _ -> ());
+  lprintf_nl ()  (_b "To command: telnet %s %d")
 	(if !!telnet_bind_addr = Ip.any then "127.0.0.1"
 		else Ip.to_string !!telnet_bind_addr)  !!telnet_port;
-  lprintf_nl (_b "Or with browser: http://%s:%d")
+  lprintf_nl ()  (_b "Or with browser: http://%s:%d")
 	(if !!http_bind_addr = Ip.any then "127.0.0.1"
 		else Ip.to_string !!http_bind_addr)  !!http_port;
-  lprintf_nl (_b "For a GUI check out http://sancho-gui.sourceforge.net");
-  lprintf_nl (_b "Connect to IP %s, port %d")
+  lprintf_nl ()  (_b "For a GUI check out http://sancho-gui.sourceforge.net");
+  lprintf_nl ()  (_b "Connect to IP %s, port %d")
 	(if !!gui_bind_addr = Ip.any then "127.0.0.1"
 		else Ip.to_string !!gui_bind_addr)  !!gui_port;
-  lprintf_nl (_b "If you connect from a remote machine adjust allowed_ips");
-  if Autoconf.system = "windows" then lprintf (_b "%s") win_message;
-
-  if Autoconf.system <> "windows" then
-    (* Doesn't work on windows with mingw, because getpid always returns 948 *)
-    (
-      let oc = open_out "mlnet.pid" in
-      output_string oc ( Printf.sprintf "%s\n" ( string_of_int ( Unix.getpid () ) ) );
-      close_out oc;
-      CommonGlobals.do_at_exit (fun _ -> try Sys.remove "mlnet.pid" with _ -> ());
-      if !verbose then lprintf_nl (_b "Starting with pid %s") (string_of_int(Unix.getpid ()))
-    );
+  lprintf_nl ()  (_b "If you connect from a remote machine adjust allowed_ips");
+  if Autoconf.system = "cygwin" && not !keep_console_output then lprintf (_b "%s") win_message;
 
   add_init_hook (fun _ ->
       if not !gui_included && ( !!start_gui || !!ask_for_gui ) then
@@ -557,7 +509,7 @@ or getting a binary compiled with glibc %s.\n\n")
             Sys.file_exists asker then
             ignore (Sys.command (Printf.sprintf "%s %s&" asker !!mldonkey_gui));
         with Not_found ->
-            lprintf_nl (_b "Not running under X, not trying to start the GUI")
+            lprintf_nl () (_b "Not running under X, not trying to start the GUI")
             );
   );
 
@@ -566,61 +518,55 @@ or getting a binary compiled with glibc %s.\n\n")
         let new_pw = Unix.getpwnam !!run_as_user in
         MlUnix.setuid new_pw.Unix.pw_uid;
         let pw = Unix.getpwuid (Unix.getuid()) in
-        lprintf_nl (_b "mldonkey is now running as user %s") pw.Unix.pw_name;
+        lprintf_nl () (_b "mldonkey is now running as user %s") pw.Unix.pw_name;
       with e ->
-          lprintf_nl (_b "Exception %s trying to set user_uid [%s]")
+          lprintf_nl () (_b "Exception %s trying to set user_uid [%s]")
           (Printexc2.to_string e) !!run_as_user;
-          exit 2
+          exit 67
     end;
 
   if !!run_as_useruid <> 0 then begin
       try
         MlUnix.setuid !!run_as_useruid;
-        lprintf_nl (_b "mldonkey is now running as uid %d")  !!run_as_useruid;
+        lprintf_nl () (_b "mldonkey is now running as uid %d")  !!run_as_useruid;
       with e ->
-          lprintf_nl (_b "Exception %s trying to set user_uid [%d]")
+          lprintf_nl () (_b "Exception %s trying to set user_uid [%d]")
           (Printexc2.to_string e) !!run_as_useruid;
-          exit 2
+          exit 67
     end;
 
   if !!create_mlsubmit then save_mlsubmit_reg ();
   DriverInteractive.initialization_completed := true;
   DriverInteractive.save_config ();
 
-  if Autoconf.system <> "windows" then
+  if not Autoconf.windows then
     MlUnix.set_signal  Sys.sigchld
-      (Sys.Signal_handle (fun _ -> lprintf_nl "Received SIGCHLD, doing nothing"));
+      (Sys.Signal_handle (fun _ -> if !verbose then lprintf_nl () "Received SIGCHLD, doing nothing"));
 
-  if Autoconf.system <> "windows" then
+  if not Autoconf.windows then
     MlUnix.set_signal  Sys.sighup
       (Sys.Signal_handle (fun _ ->
-	 lprintf_nl "Received SIGHUP, closing all files/sockets";
+	 lprintf_nl () "Received SIGHUP, closing all files/sockets";
          BasicSocket.close_all ();
 	 Unix32.close_all ()
          ));
 
-  if Autoconf.system <> "windows" then
+  if not Autoconf.windows then
     MlUnix.set_signal  Sys.sigpipe
-      (Sys.Signal_handle (fun _ -> if !verbose then lprintf_nl "Received SIGPIPE, doing nothing"));
+      (Sys.Signal_handle (fun _ -> if !verbose then lprintf_nl () "Received SIGPIPE, doing nothing"));
 
   MlUnix.set_signal  Sys.sigint
-    (Sys.Signal_handle (fun _ -> lprintf_nl "Received SIGINT, stopping MLDonkey...";
-        CommonGlobals.exit_properly 0));
+    (Sys.Signal_handle (fun _ -> lprintf_nl () "Received SIGINT, stopping MLDonkey...";
+        CommonInteractive.clean_exit 0));
 
   MlUnix.set_signal  Sys.sigterm
-    (Sys.Signal_handle (fun _ -> lprintf_nl "Received SIGTERM, stopping MLDonkey...";
-        CommonGlobals.exit_properly 0));
+    (Sys.Signal_handle (fun _ -> lprintf_nl () "Received SIGTERM, stopping MLDonkey...";
+        CommonInteractive.clean_exit 0));
 
-  if !verbose then lprintf_nl (_b "Activated system signal handling");
-
-  Unix32.max_cache_size := MlUnix.max_filedescs
+  if !verbose then lprintf_nl () (_b "Activated system signal handling")
 
 let _ =
   let security_space_filename = "config_files_space.tmp" in
-
-  lprintf_nl (_b "Core started");
-  core_included := true;
-
   begin
 (* Create a 'config_files_security_space' megabytes file to protect some space
 for config files at the end. *)
@@ -640,18 +586,49 @@ for config files at the end. *)
       in
       Unix32.close security_space_fd;
     with _ ->
-        lprintf_nl (_b "Cannot create Security space file:");
-        lprintf_nl (_b " not enough space on device or bad permissions");
-        lprintf_nl (_b "Exiting...");
-        exit 2;
+        lprintf_nl () (_b "Cannot create Security space file:");
+        lprintf_nl () (_b " not enough space on device or bad permissions");
+        lprintf_nl () (_b "Exiting...");
+        exit 73;
   end;
+  Unix32.external_start (CommonGlobals.version());
+
+  (
+    let pid_filename =
+      Printf.sprintf "%s.pid" (Filename.basename Sys.argv.(0))
+    in
+    let pid_file, s =
+        Filename.concat !pid pid_filename,
+	Printf.sprintf "%s\n" (string_of_int(Unix.getpid()))
+    in
+    let oc = open_out pid_file in
+    output_string oc s;
+    close_out oc;
+    CommonGlobals.do_at_exit (fun _ -> try Sys.remove pid_file with _ -> ());
+    if !verbose then
+      lprintf_nl () (_b "Starting with pid %s") (string_of_int(Unix.getpid ()))
+  );
+
+(* When a core is spawned from a gui, the only way to know the startup has
+   succeeded is the string token "Core started". *)
+  if not !keep_console_output then
+    begin
+      try
+        Printf.fprintf Pervasives.stderr "%sCore started\n" (log_time ());
+        Pervasives.flush Pervasives.stderr;
+      with _ -> ()
+    end;
+
+  lprintf_nl () (_b "Core started");
+  core_included := true;
+
   CommonGlobals.do_at_exit (fun _ ->
       (* If we have an error with too many file-descriptors,
          just close all of them *)
       (try
          BasicSocket.close_all ();
        with e ->
-           lprintf_nl "Exception %s in do_at_exit while closing sockets."
+           lprintf_nl () "Exception %s in do_at_exit while closing sockets."
              (Printexc2.to_string e);
       );
       CommonGraphics.remove_files ();
@@ -661,29 +638,16 @@ for config files at the end. *)
       DriverInteractive.save_config ();
       CommonComplexOptions.save_sources ();
       CommonComplexOptions.backup_options ();
-      lprintf_nl (_b "Core stopped")
+      Geoip.close ();
+      Unix32.external_exit ();
+      lprintf_nl () (_b "Core stopped")
     );
 
-  if not !keep_console_output then begin
-      lprintf_nl (_b "Disabling output to console, to enable: stdout true");
-
-      if !!log_file <> "" then begin
-          (*
-          try
-            Printf.printf "+XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX+\n";
-            let oc = open_out !!log_file in
-            lprintf "Logging in %s\n" !!log_file;
-
-            (* Don't close stdout !!!
-            (match !lprintf_output with
-               None -> () | Some oc -> close_out oc);
-*)
-            log_to_file oc;
-            lprintf "Start log in %s\n" !!log_file;
-          with e ->
-              lprintf "Exception %s while opening log file: %s\n"
-(Printexc2.to_string e) !!log_file *)
-          ()
-        end else
-              close_log ()
-    end
+  if not !keep_console_output then
+    if !!log_file = "" then 
+      begin
+        lprintf_nl () (_b "Option log_file is empty, disable logging completely...");
+        lprintf_nl () (_b "Disabling output to console, to enable: stdout true");
+	log_to_file stdout;
+        close_log ()
+      end
