@@ -299,7 +299,7 @@ class box columns friend_tab =
             Known_location _ -> M.fT_tx_direct
           | _ -> "")
       | Col_client_rating -> string_of_int f.gclient_rating
-      | Col_client_connect_time -> Gui_graph.time_to_string (f.gclient_connect_time)
+      | Col_client_connect_time -> Date.time_to_string (f.gclient_connect_time) "long"
       | Col_client_software -> f.gclient_software
       | Col_client_release -> f.gclient_release
       | Col_client_emulemod -> f.gclient_emulemod
@@ -435,6 +435,7 @@ class box_friends box_files friend_tab =
         client_downloaded = c.gclient_downloaded;
         client_uploaded = c.gclient_uploaded;
         client_upload = c.gclient_upload;
+	client_sui_verified = None;
 (*        client_sock_addr = c.gclient_sock_addr;*)
       }
     
@@ -641,6 +642,7 @@ class box_list friend_tab =
         client_downloaded = c.gclient_downloaded;
         client_uploaded = c.gclient_uploaded;
         client_upload = c.gclient_upload;
+	client_sui_verified = None;
 (*        client_sock_addr = string_of_kind c.gclient_kind; *)
       }
 
