@@ -85,7 +85,7 @@ let size_of_int64 size =
 
 let int64_of_size size =
   let len = String.length size in
-  let u = String.sub size (len - 2) len in
+  let u = String.sub size (len - 2) 2 in
   let s = String.sub size 0 (len - 3) in
   try
     match u with
@@ -363,7 +363,7 @@ let create_search query_entry max_hits net search_type =
   in
   s
 
-let rec rec_description_of_query q = 
+let rec rec_description_of_query q =
   match q with
   | Q_HIDDEN l
   | Q_AND l
@@ -371,7 +371,7 @@ let rec rec_description_of_query q =
 
   | Q_ANDNOT (q1, q2) -> rec_description_of_query q1
   | Q_MODULE (_,q) -> rec_description_of_query q
-	
+
   | Q_KEYWORDS (_,s) -> [s]
   | Q_MINSIZE _ 
   | Q_MAXSIZE _ -> []
@@ -382,7 +382,7 @@ let rec rec_description_of_query q =
   | Q_MP3_ALBUM (_,s) -> [s]
 
   | Q_COMBO _ -> []
-      
+
   | Q_MP3_BITRATE _ -> []
 
 (* Return few words for a given query *)
@@ -502,7 +502,7 @@ let is_connected state =
 (*************************************************************************)
 
 let uploader_state_to_string state has_upload =
-  if has_upload
+  if has_upload = source_has_upload
     then begin
       match state with
           Connected_downloading _  -> !M.dT_tx_updown
@@ -824,18 +824,7 @@ let string_of_availability availability chunks =
 let main_availability_of net availabilities  =
   try
     List.assoc net availabilities
-  with _ ->
-    begin
-      match availabilities with
-          (_, availability)::_ ->
-             begin
-               let len = String.length availability in
-               String.make len (char_of_int 0)
-             end
-
-        | _ -> ""
-
-    end
+  with _ -> ""
 
 let availability_bar availability chunks b =
   if !!O.gtk_look_graphical_availability
@@ -1034,7 +1023,7 @@ let location_kind_to_string kind =
 let hash_of_client kind =
   match kind with
       Known_location _ -> "-"
-    | Indirect_location (_, md4) -> Printf.sprintf "%s" (Md4.to_string md4)
+    | Indirect_location (_, md4, _, _) -> Printf.sprintf "%s" (Md4.to_string md4)
 
 (*************************************************************************)
 (*                                                                       *)
@@ -1057,6 +1046,10 @@ let string_of_tag_name name =
     | Field_Filename -> "filename"
     | Field_Size -> "size"
     | Field_Uid -> "uid"
+    | Field_Medialength -> "length"
+    | Field_Mediacodec -> "codec"
+    | Field_Lastseencomplete -> "lastseencompl"
+    | Field_Filerating -> "rating"
     | Field_UNKNOWN s -> U.simple_utf8_of s
   
 let tags_to_string tags =
@@ -1264,32 +1257,86 @@ let list_files tree =
 let client_to_source c =
   let s =
     {
-     source_num = c.client_num;
-     source_network = c.client_network;
+     source_num             = c.client_num;
+     source_network         = c.client_network;
 
-     source_kind = c.client_kind;
-     source_state = c.client_state;
-     source_type = c.client_type;
-     source_tags = c.client_tags;
-     source_name = U.utf8_of c.client_name;
-     source_files = None;
-     source_rating = c.client_rating;
-     source_chat_port = c.client_chat_port;
-     source_connect_time = BasicSocket.last_time () - c.client_connect_time;
-     source_last_seen = BasicSocket.current_time ();
-     source_software = concat_strings c.client_software (concat_strings c.client_emulemod c.client_release);
-     source_downloaded = c.client_downloaded;
-     source_uploaded = c.client_uploaded;
-     source_upload_rate = 0.;
-     source_download_rate = 0.;
-     source_upload = c.client_upload;
-     source_has_upload = false;
-     source_availability = [];
+     source_kind            = c.client_kind;
+     source_state           = c.client_state;
+     source_type            = c.client_type;
+     source_tags            = c.client_tags;
+     source_name            = U.utf8_of c.client_name;
+     source_files           = None;
+     source_rating          = c.client_rating;
+     source_chat_port       = c.client_chat_port;
+     source_connect_time    = BasicSocket.last_time () - c.client_connect_time;
+     source_last_seen       = BasicSocket.current_time ();
+     source_software        = concat_strings c.client_software (concat_strings c.client_emulemod c.client_release);
+     source_downloaded      = c.client_downloaded;
+     source_uploaded        = c.client_uploaded;
+     source_upload_rate     = 0.;
+     source_download_rate   = 0.;
+     source_upload          = c.client_upload;
+     source_has_upload      = source_only;
+     source_availability    = [];
      source_files_requested = [];
     }
   in
   s
 
+(*************************************************************************)
+(*                                                                       *)
+(*                        uid_to_common_uid                              *)
+(*                                                                       *)
+(*************************************************************************)
+
+let uid_to_common_uid uid =
+  match uid with
+    Bitprint (sha1,ttr) -> Sha1 sha1
+  | BTUrl url -> Sha1 url
+  | _ -> uid
+
+(*************************************************************************)
+(*                                                                       *)
+(*                        ustring_of_uid                                 *)
+(*                                                                       *)
+(*************************************************************************)
+
+let ustring_of_uid uid =
+  let sep = ":" in
+  match uid with
+    Bitprint (sha1,ttr) ->
+      "urn" ^ sep ^ "sha1" ^ sep ^ (Sha1.to_string sha1)
+  | Sha1 sha1 ->
+      "urn" ^ sep ^ "sha1" ^ sep ^ (Sha1.to_string sha1)
+  | Ed2k ed2k ->
+      "urn" ^ sep ^ "ed2k" ^ sep ^ (Md4.to_string ed2k)
+  | Md5 md5 ->
+      "urn" ^ sep ^ "md5" ^ sep ^ (Md5.to_string md5)
+  | TigerTree ttr -> 
+      "urn" ^ sep ^ "ttr" ^ sep ^ (TigerTree.to_string ttr)
+  | Md5Ext md5 ->
+      "urn" ^ sep ^ "sig2dat" ^ sep ^ (Md5Ext.to_base32 md5)
+  | BTUrl url ->
+      "urn" ^ sep ^ "sha1" ^ sep ^ (Sha1.to_string url)
+  | FileTP file ->
+      "urn" ^ sep ^ "filetp" ^ sep ^ (Md4.to_string file)
+  | NoUid -> ""
+
+(*************************************************************************)
+(*                                                                       *)
+(*                        normalize_uids                                 *)
+(*                                                                       *)
+(*************************************************************************)
+
+let normalize_uids uid_list =
+  let l = ref [] in
+  List.iter (fun t ->
+    let uid_type = uid_to_common_uid (Uid.to_uid t) in
+    let _t = Uid.create uid_type in
+    if not (List.mem _t !l) then l := _t :: !l
+  ) uid_list;
+  !l
+
 (*************************************************************************)
 (*                                                                       *)
 (*                        uid_list_to_string                             *)
@@ -1299,4 +1346,66 @@ let client_to_source c =
 let uid_list_to_string l =
   match l with
       [] -> ""
-    | uid :: _ -> U.simple_utf8_of (Uid.to_string uid)
+    | uid :: _ -> U.simple_utf8_of (ustring_of_uid (Uid.to_uid uid))
+
+(*************************************************************************)
+(*                                                                       *)
+(*                        to_uid_type                                    *)
+(*                                                                       *)
+(*************************************************************************)
+
+let to_uid_type l =
+  match l with
+      [] -> NoUid
+    | uid :: _ -> Uid.to_uid uid
+
+(*************************************************************************)
+(*                                                                       *)
+(*                        shared_info_to_shared_file                     *)
+(*                                                                       *)
+(*************************************************************************)
+
+let shared_info_to_shared_file si =
+  {
+    g_shared_num       = si.shared_num;
+    g_shared_network   = si.shared_network;
+    g_shared_filename  = si.shared_filename;
+    g_shared_size      = si.shared_size;
+    g_shared_uploaded  = si.shared_uploaded;
+    g_shared_requests  = si.shared_requests;
+    g_shared_uids      = normalize_uids si.shared_uids;
+    g_shared_last_seen = BasicSocket.current_time ();
+  }
+
+(*************************************************************************)
+(*                                                                       *)
+(*                        file_info_to_g_file_info                       *)
+(*                                                                       *)
+(*************************************************************************)
+
+let file_info_to_g_file_info f =
+  {
+    g_file_num             = f.file_num;
+    g_file_network         = f.file_network;
+
+    g_file_comment         = f.file_comment;
+    g_file_name            = f.file_name;
+    g_file_names           = f.file_names;
+    g_file_size            = f.file_size;
+    g_file_downloaded      = f.file_downloaded;
+    g_file_active_sources  = f.file_active_sources;
+    g_file_all_sources     = f.file_all_sources;
+    g_file_state           = f.file_state;
+    g_file_chunks          = f.file_chunks;
+    g_file_availability    = f.file_availability;
+    g_file_sources         = f.file_sources;
+    g_file_download_rate   = f.file_download_rate;
+    g_file_format          = f.file_format;
+    g_file_chunks_age      = f.file_chunks_age;
+    g_file_age             = (BasicSocket.last_time () - f.file_age);
+    g_file_last_seen       = (BasicSocket.last_time () - f.file_last_seen);
+    g_file_priority        = f.file_priority;
+    g_file_uids            = normalize_uids f.file_uids;
+
+    g_file_stats           = [];
+  }
