@@ -66,12 +66,12 @@ open TcpMessages
 
 (* prints a new logline with date, module and starts newline *)
 let lprintf_nl () =
-  lprintf "%s[BT]: "
+  lprintf "%s[BT] "
     (log_time ()); lprintf_nl2
 
 (* prints a new logline with date, module and does not start newline *)
 let lprintf_n () =
-  lprintf "%s[BT]: "
+  lprintf "%s[BT] "
     (log_time ()); lprintf
 
 let http_ok = "HTTP 200 OK"
@@ -138,6 +138,17 @@ let connect_trackers file event f =
       ("ip", Ip.to_string !!set_client_ip) :: args else args
   in
 
+  let enabled_trackers =
+    let enabled_trackers = List.filter (fun t -> t.tracker_enabled) file.file_trackers in
+    if enabled_trackers <> [] then enabled_trackers
+    else begin
+      (* if there is no tracker left, do something ? *)
+      if !verbose_msg_servers then
+	lprintf_nl () "No trackers left, reenabling all of them...";
+      List.iter (fun t -> t.tracker_enabled <- true) file.file_trackers;
+      file.file_trackers
+    end in
+
   List.iter (fun t ->
 
       (* if we have too few sources we may ask the tracker before the interval *)
@@ -156,8 +167,9 @@ let connect_trackers file event f =
           if file.file_tracker_connected && t.tracker_last_clients_num = 0 &&
             t.tracker_last_conn < 1 then begin
               if !verbose_msg_servers then
-                lprintf_nl () "Request error from tracker: removing %s" t.tracker_url;
-              remove_tracker t.tracker_url file
+                lprintf_nl () "Request error from tracker: disabling %s" t.tracker_url;
+	      t.tracker_enabled <- false;
+              (* remove_tracker t.tracker_url file *)
             end
           (* Send request to tracker *)
           else begin
@@ -179,8 +191,9 @@ let connect_trackers file event f =
                   H.basic_request with
                   H.req_url = Url.of_string ~args: args url;
                   H.req_proxy = !CommonOptions.http_proxy;
-                  H.req_user_agent =
-                    Printf.sprintf "MLdonkey/%s" Autoconf.current_version;
+                  H.req_user_agent = get_user_agent ();
+                  (* #4541 [egs]  supports redirect *)
+                  H.req_max_retry = !!max_tracker_redirect;
                 } in
 
               if !verbose_msg_servers then
@@ -198,11 +211,7 @@ let connect_trackers file event f =
         if !verbose_msg_servers then
           lprintf_nl () "Request NOT sent to tracker %s - next request in %ds for file: %s"
             t.tracker_url (t.tracker_interval - (last_time () - t.tracker_last_conn)) file.file_name
-  ) file.file_trackers;
-  (* if there is no tracker left, do something ? *)
-  if List.length file.file_trackers = 0 then
-    if !verbose_msg_servers then
-      lprintf_nl () "No trackers left ..."
+  ) enabled_trackers
 
 (** In this function we decide which peers will be
   uploaders. We send a choke message to current uploaders
@@ -260,10 +269,9 @@ let disconnect_client c reason =
         try
 (*          List.iter (fun r -> Int64Swarmer.free_range r) c.client_ranges; *)
           set_client_disconnected c reason;
-          let file = c.client_file in
           (try if c.client_good then count_seen c with _ -> ());
           (* this is not useful already done in the match
-          (try close sock reason with _ -> ());	  *)
+          (try close sock reason with _ -> ());   *)
 (*---------not needed ?? VvvvvV---------------
           c.client_ranges <- [];
           c.client_block <- None;
@@ -288,7 +296,7 @@ let disconnect_client c reason =
                      (Useful for availability)
                   *)
                   Int64Swarmer.unregister_uploader up
-(*	      c.client_registered_bitfield <- false;
+(*        c.client_registered_bitfield <- false;
           for i = 0 to String.length c.client_bitmap - 1 do
             c.client_bitmap.[0] <- '0';
           done*)
@@ -303,7 +311,7 @@ let disconnect_client c reason =
               remove_client c;
               recompute_uploaders ();
             end
-          else	
+          else  
             remove_client c;
         with _ -> ()
   end
@@ -338,6 +346,7 @@ let download_finished file =
         (*CommonComplexOptions.file_completed*)
         file_completed (as_file file);
         (* Remove the swarmer for this file as it is not useful anymore... *)
+        Int64Swarmer.remove_swarmer file.file_swarmer;
         file.file_swarmer <- None;
         (* At this point, the file state is FileDownloaded. We should not remove
            the file, because we continue to upload. *)
@@ -353,6 +362,16 @@ let check_finished swarmer file =
       download_finished file
 
 let bits = [| 128; 64; 32;16;8;4;2;1 |]
+
+(* Check/set bits in strings (bittorrent format) *)
+
+let is_bit_set s n =
+  (Char.code s.[n lsr 3]) land bits.(n land 7) <> 0
+
+let set_bit s n =
+  let i = n lsr 3 in
+  s.[i] <- Char.unsafe_chr (Char.code s.[i] lor bits.(n land 7))
+
 (* Official client seems to use max_range_request 5 and max_range_len 2^14 *)
 (* How much requests in the 'pipeline' *)
 let max_range_requests = 5
@@ -384,21 +403,16 @@ let send_bitfield c =
         Int64Swarmer.verified_bitmap swarmer
   in
 
-  if !verbose then lprintf_nl () "Sending verified bitmap: [%s]" bitmap;
+  if !verbose_download then lprintf_nl () "Sending verified bitmap: [%s]" bitmap;
 
+(* In the future, only accept bitmap.[n] > '2' when verification works *)
   send_client c (BitField
       (
       let nchunks = String.length bitmap in
       let len = (nchunks+7)/8 in
       let s = String.make len '\000' in
       for i = 0 to nchunks - 1 do
-        let n = i lsr 3 in
-        let j = i land 7 in
-(* In the future, only accept bitmap.[n] > '2' when verification works *)
-        if bitmap.[i] >= '2' then begin
-            s.[n] <- char_of_int (int_of_char s.[n]
-                lor bits.(j))
-          end
+        if bitmap.[i] >= '2' then set_bit s i;
       done;
       s
     ))
@@ -483,11 +497,11 @@ let rec client_parse_header counter cc init_sent gconn sock
       | ConnectionWaiting token ->
           cancel_token token;
           if !verbose_msg_clients then
-            lprintf_nl () "[BT]: Waiting for connection to client !!!";
+            lprintf_nl () "Waiting for connection to client !!!";
           c.client_sock <- Connection sock
       | Connection s when s != sock ->
           if !verbose_msg_clients then
-            lprintf_nl () "[BT]: CLIENT %d: IMMEDIATE RECONNECTION" (client_num c);
+            lprintf_nl () "CLIENT %d: IMMEDIATE RECONNECTION" (client_num c);
           disconnect_client c (Closed_for_error "Reconnected");
           c.client_sock <- Connection sock;
       | Connection _  -> ()
@@ -501,9 +515,9 @@ let rec client_parse_header counter cc init_sent gconn sock
       end;
     connection_ok c.client_connection_control;
     if !verbose_msg_clients then
-      lprintf_nl () "[BT]: file and client found";
+      lprintf_nl () "file and client found";
 (*    if not c.client_incoming then *)
-    send_bitfield c;
+     send_bitfield c; 
     c.client_blocks_sent <- file.file_blocks_downloaded;
 (*
       TODO !!! : send interested if and only if we are interested
@@ -536,9 +550,12 @@ let rec client_parse_header counter cc init_sent gconn sock
 *)
 and update_client_bitmap c =
   let file = c.client_file in
+  
   let swarmer = match file.file_swarmer with
       None -> assert false
-    | Some swarmer -> swarmer in
+    | Some swarmer -> swarmer 
+  in
+  
   let up =
     match c.client_uploader with
       None ->
@@ -553,18 +570,18 @@ and update_client_bitmap c =
   let bitmap = match c.client_bitmap with
       None ->
         let len = Int64Swarmer.partition_size swarmer in
-        let bitmap = String.make (len*8) '0' in
+        let bitmap = Bitv.create len false in
         c.client_bitmap <- Some bitmap;
         bitmap
     | Some bitmap -> bitmap
   in
 
-  if c.client_new_chunks <> [] then
+  if c.client_new_chunks <> [] then begin
     let chunks = c.client_new_chunks in
     c.client_new_chunks <- [];
-    let file = c.client_file in
-    List.iter (fun n -> bitmap.[n] <- '1') chunks;
-    Int64Swarmer.update_uploader up (AvailableCharBitmap bitmap)
+    List.iter (fun n -> Bitv.set bitmap n true) chunks;
+    Int64Swarmer.update_uploader up (AvailableBitv bitmap);
+  end
 
 
 (** In this function we decide which piece we must request from client.
@@ -575,8 +592,10 @@ and get_from_client sock (c: client) =
   let file = c.client_file in
   (* Check if there's not enough requests in the 'pipeline'
      and if a request can be send (not choked and file is downloading) *)
-  if List.length c.client_ranges_sent < max_range_requests &&
-    file_state file = FileDownloading && (c.client_choked == false) then
+  if List.length c.client_ranges_sent < max_range_requests 
+      && file_state file = FileDownloading 
+      && (c.client_choked == false) 
+    then
   (* num is the number of the piece, x and y are the position
      of the subpiece in the piece(!), r is a (CommonSwarmer) range *)
 
@@ -586,92 +605,128 @@ and get_from_client sock (c: client) =
     let swarmer = Int64Swarmer.uploader_swarmer up in
 
   try
+  
     let num, x,y, r =
+
       if !verbose_msg_clients then begin
-          lprintf_nl () "CLIENT %d: Finding new range to send" (client_num c);
-        end;
+        lprintf_nl () "CLIENT %d: Finding new range to send" (client_num c);
+      end;
 
       if !verbose_swarming then begin
-          lprintf_n () "Current download:\n  Current chunks: ";
-          (try
+        lprintf_n () "Current download:\n  Current chunks: ";
+
+        try
           List.iter (fun (x,y) -> lprintf "%Ld-%Ld " x y) c.client_chunks
-	  with _ -> lprintf "No Chunks");
-	  lprint_newline ();
-          lprintf_n () "Current ranges: ";
-          List.iter (fun (p1,p2, r) ->
-              let (x,y) = Int64Swarmer.range_range r
-              in
-              lprintf "%Ld-%Ld[%Ld-%Ld] " p1 p2 x y) c.client_ranges_sent;
-          (match c.client_range_waiting with
-              None -> ()
-            | Some (x,y,r) -> lprintf "Waiting %Ld-%Ld" x y);
-	  lprint_newline ();
-          lprintf_n () "Current block: ";
-          (match c.client_block with
-              None -> lprintf "none"
-            | Some b -> Int64Swarmer.print_block b);
-	  lprint_newline ();
-          lprintf_nl () "Finding Range:";
-        end;
+        with _ -> lprintf "No Chunks";
+          
+        lprint_newline ();
+          
+        lprintf_n () "Current ranges: ";
+          
+        List.iter (fun (p1,p2, r) ->
+          let (x,y) = Int64Swarmer.range_range r in
+          lprintf "%Ld-%Ld[%Ld-%Ld] " p1 p2 x y
+        ) c.client_ranges_sent;
+
+        match c.client_range_waiting with
+        | None -> ()
+        | Some (x,y,r) -> lprintf "Waiting %Ld-%Ld" x y;
+
+        lprint_newline ();
+          
+        lprintf_n () "Current block: ";
+          
+        match c.client_block with
+        | None -> lprintf "none"
+        | Some b -> Int64Swarmer.print_block b;
+
+        lprint_newline ();
+      
+        lprintf_nl () "Finding Range:";
+      end;
+
       try
+
         (*We must find a block to request first, and then
           some range inside this block
         *)
+
         let rec iter () =
+
           match c.client_block with
-            None ->
-              if !verbose_swarming then
-                lprintf_nl () "No block";
+
+            None -> 
+
+              if !verbose_swarming then lprintf_nl () "No block";
               update_client_bitmap c;
               (try Int64Swarmer.verify_one_chunk swarmer with _ -> ());
               (*Find a free block in the swarmer*)
               let b = Int64Swarmer.find_block up in
-              if !verbose_swarming then begin
-                  lprintf_n () "Block Found: "; Int64Swarmer.print_block b;
-		  lprint_newline ()
-                end;
+              if !verbose_swarming then begin 
+                lprintf_n () "Block Found: "; Int64Swarmer.print_block b;
+                lprint_newline ()
+              end;
               c.client_block <- Some b;
+
              (*We put the found block in client_block to
                request range in this block. (Useful for
                not searching each time a new block)
               *)
+
               iter ()
+
           | Some b ->
+
               if !verbose_swarming then begin
-                  lprintf_n () "Current Block: "; Int64Swarmer.print_block b;
-		  lprint_newline ()
-                end;
+                lprintf_n () "Current Block: "; Int64Swarmer.print_block b;
+                lprint_newline ()
+              end;
+
               try
                 (*Given a block find a range inside*)
                 let (x,y,r) =
                   match c.client_range_waiting with
                     Some (x,y,r) ->
-                      c.client_range_waiting <- None;
-                      (x,y,r)
-                  | None -> Int64Swarmer.find_range up in
+                        c.client_range_waiting <- None;
+                        (x,y,r)
+                  | None -> 
+                        Int64Swarmer.find_range up 
+                in
 
                 let (x,y,r) =
+                  
                   if y -- x > max_range_len then begin
                       c.client_range_waiting <- Some (x ++ max_range_len, y, r);
                       (x, x ++ max_range_len, r)
-                    end else (x,y,r) in
-                c.client_ranges_sent <- c.client_ranges_sent @ [x,y, r];
+                  end else 
+                      (x,y,r) 
+                in
+                
+                  c.client_ranges_sent <- c.client_ranges_sent @ [x,y, r];
 (*                Int64Swarmer.alloc_range r; *)
-                let num = Int64Swarmer.block_num swarmer b in
-                if !verbose_swarming then
-                  lprintf_nl () "Asking %d For Range %Ld-%Ld" num x y;
+                
+                  let num = Int64Swarmer.block_num swarmer b in
+
+                  if !verbose_swarming then 
+                    lprintf_nl () "Asking %d For Range %Ld-%Ld" num x y;
+                      
+                  num, x -- file.file_piece_size ** Int64.of_int num, y -- x, r
 
-                num, x -- file.file_piece_size ** Int64.of_int num, y -- x, r
               with Not_found ->
+  
                   (*If we don't find a range to request inside the block,
                     iter to choose another block*)
                   if !verbose_swarming then
                     lprintf_nl () "Could not find range in current block";
 (*                  c.client_blocks <- List2.removeq b c.client_blocks; *)
+
                   c.client_block <- None;
+
                   iter ()
         in
+
         iter ()
+
       with Not_found ->
           (*If we don't find a block to request we can check if the
             file is finished (if there's missing pieces we can't decide
@@ -684,16 +739,16 @@ and get_from_client sock (c: client) =
           check_finished swarmer file;
           raise Not_found
     in
+
     send_client c (Request (num,x,y));
+
     if !verbose_msg_clients then
       lprintf_nl () "CLIENT %d: Asking %s For Range %Ld-%Ld"
-        (client_num c)
-      (Sha1.to_string c.client_uid)
-      x y
+        (client_num c) (Sha1.to_string c.client_uid) x y
+
   with Not_found ->
-        if not (Int64Swarmer.check_finished swarmer) && !verbose_hidden_errors then
-          lprintf_nl ()
-            "BTClient.get_from_client ERROR: can't find a block to download and file is not yet finished for file : %s..." file.file_name
+        if not (Int64Swarmer.check_finished swarmer) && !verbose_download then
+          lprintf_nl () "BTClient.get_from_client ERROR: can't find a block to download and file is not yet finished for file : %s..." file.file_name
 
 
 (** In this function we match a message sent by a client
@@ -749,7 +804,7 @@ and client_to_client c sock msg =
                 | (p1,p2,r) :: _ ->
                     let (x,y) = Int64Swarmer.range_range r in
                     lprintf_nl () "Current range from %s : %Ld [%d] (asked %Ld-%Ld[%Ld-%Ld])"
-                      c.client_software position len
+                      (brand_to_string c.client_brand) position len
                       p1 p2 x y
               );
 
@@ -763,14 +818,9 @@ and client_to_client c sock msg =
               Int64Swarmer.downloaded swarmer in
 
             (*Update rate and amount of data received from client*)
-            c.client_downloaded <- c.client_downloaded ++
-              (new_downloaded -- old_downloaded);
+            count_download c (new_downloaded -- old_downloaded);
             (* use len here with max_dr quickfix *)
             Rate.update c.client_downloaded_rate  (float_of_int len);
-            (* update the stats *)
-            let len64 = Int64.of_int len in
-            count_download c file len64;
-            network_must_update network;
             if !verbose_msg_clients then
               (match c.client_ranges_sent with
                   [] -> lprintf_nl () "EMPTY Ranges !!!"
@@ -818,8 +868,9 @@ and client_to_client c sock msg =
       (* Disconnect if that is ourselves. *)
       if not (c.client_uid = !!client_uid) then
         begin
-          c.client_brand <- (parse_brand p);
-          c.client_software <- (parse_software p);
+          let brand, release = parse_software p in
+          c.client_brand <- brand;
+          c.client_release <- release;
 (* TODO : enable it
       c.client_release <- (parse_release p c.client_brand);
  *)
@@ -856,50 +907,40 @@ and client_to_client c sock msg =
             None -> ()
           | Some swarmer ->
               c.client_new_chunks <- [];
-              let len = String.length p in
-              let bitmap = String.make (len*8) '0' in
-              for i = 0 to len - 1 do
-                for j = 0 to 7 do
-                  if (int_of_char p.[i]) land bits.(j) <> 0 then
-                    begin
-                      c.client_new_chunks <- i*8+j :: c.client_new_chunks;
-                      bitmap.[i*8+j] <- '1';
-                    end
-                  else
-                    bitmap.[i*8+j] <- '0';
-                done;
-              done;
-
-              update_client_bitmap c;
-
-              let verified = Int64Swarmer.verified_bitmap swarmer in
+              
               let npieces = Int64Swarmer.partition_size swarmer in
-              (* parse the pieces to see if the client is interesting *)
-              for i = 0 to npieces -1  do
-                (* lprintf "bitmap: %c, verified: %c" bitmap.[i] verified.[i]; *)
-                if bitmap.[i] = '1' && verified.[i] < '2' then
-                  c.client_interesting <- true;
-              done;
+              let nbits = String.length p * 8 in
 
-              if !verbose_msg_clients then
-                lprintf_nl () "BitField translated";
-              if !verbose_msg_clients then
-                lprintf_nl () "Old BitField Unregistered";
-              (match c.client_uploader with
-                  None -> assert false
-                | Some up ->
-                    Int64Swarmer.update_uploader up
-                      (AvailableCharBitmap bitmap));
-              c.client_registered_bitfield <- true;
-              c.client_bitmap <- Some bitmap;
-              (* send interested if we are *)
-              if c.client_interesting then
-                send_interested c;
-              if !verbose_msg_clients then
-                lprintf_nl () "New BitField Registered";
-(*        for i = 1 to max_range_requests - List.length c.client_ranges do
-          (try get_from_client sock c with _ -> ())
-        done*)
+              if nbits < npieces then begin
+                lprintf_nl () "Error: expected bitfield of atleast %d but got %d" npieces nbits;
+                disconnect_client c (Closed_for_error "Wrong bitfield length")
+              end else begin
+
+                let verified = Int64Swarmer.verified_bitmap swarmer in
+
+                for i = 0 to npieces - 1 do
+                  if is_bit_set p i then begin
+                    c.client_new_chunks <- i :: c.client_new_chunks;
+                    if verified.[i] < '2' then
+                      c.client_interesting <- true;
+                  end 
+                done;
+
+                update_client_bitmap c;
+                c.client_registered_bitfield <- true;
+
+                if c.client_interesting then
+                  send_interested c;
+
+                if !verbose_msg_clients then
+                  lprintf_nl () "New BitField Registered";
+
+                (*  for i = 1 to max_range_requests - List.length c.client_ranges do
+                      (try get_from_client sock c with _ -> ())
+                    done
+                *)
+
+            end;
         end;
         (* Note: a bitfield must only be sent after the handshake and before everything else: NOT here *)
 
@@ -941,8 +982,8 @@ and client_to_client c sock msg =
                       end
                   end
           | None, Some _ -> lprintf_nl () "no bitmap but client_uploader";
-	  | Some _ , None ->lprintf_nl () "bitmap but no client_uploader";
-	  | None, None -> lprintf_nl () "no bitmap no client_uploader";
+    | Some _ , None ->lprintf_nl () "bitmap but no client_uploader";
+    | None, None -> lprintf_nl () "no bitmap no client_uploader";
         end
 *)
         end
@@ -961,7 +1002,7 @@ and client_to_client c sock msg =
                    didn't send a client bitmap after the handshake. *)
                 let (ip,port) = c.client_host in
                   if !verbose_msg_clients then lprintf_nl () "%s:%d with software %s : Choke send, but no client bitmap"
-                    (Ip.to_string ip) port (c.client_software)
+                    (Ip.to_string ip) port (brand_to_string c.client_brand)
             | Some up ->
                 Int64Swarmer.clear_uploader_ranges up
           end;
@@ -1297,8 +1338,9 @@ let get_sources_from_tracker file =
             match (key, value) with
             | String "failure reason", String failure ->
                 (* On failure, remove the faulty tracker from file.file_trackers list *)
-		remove_tracker t.tracker_url file;
-                lprintf_nl () "Failure from Tracker %s in file: %s Reason: %s\nBT: Tracker %s removed for failure"
+		t.tracker_enabled <- false;
+		(* remove_tracker t.tracker_url file; *)
+                lprintf_nl () "Failure from Tracker %s in file: %s Reason: %s\nBT: Tracker %s disabled for failure"
                   t.tracker_url file.file_name failure t.tracker_url
             | String "warning message", String warning ->
                 lprintf_nl () "Warning from Tracker %s in file: %s Reason: %s" t.tracker_url file.file_name warning
@@ -1372,7 +1414,7 @@ let get_sources_from_tracker file =
                                     (Ip.to_string !peer_ip) !port br.blocking_description;
                                 false)
                         then
-                          let c = new_client file !peer_id (!peer_ip,!port)
+                          let _ = new_client file !peer_id (!peer_ip,!port)
                           in
                           if !verbose_sources > 1 then lprintf_nl () "Received %s:%d" (Ip.to_string !peer_ip)
                           !port;
@@ -1449,20 +1491,19 @@ let rec iter_upload sock c =
           c.client_upload_requests <- tail;
           iter_upload sock c
         end else
-      if c.client_allowed_to_write >= len then begin
+      if c.client_allowed_to_write >= 0L then begin
+        try
           c.client_upload_requests <- tail;
 
           let file = c.client_file in
           let offset = pos ++ file.file_piece_size *.. num in
           c.client_allowed_to_write <- c.client_allowed_to_write -- len;
-          c.client_uploaded <- c.client_uploaded ++ len;
+          count_upload c len;
           let len = Int64.to_int len in
-          CommonUploads.consume_bandwidth len;
 (*          lprintf "Unix32.read: offset %Ld len %d\n" offset len; *)
           Unix32.read (file_fd file) offset upload_buffer 0 len;
           (* update upload rate from len bytes *)
-          (* will be reverted to len instead of len / 2 when rate bug will be fixed *)
-          Rate.update c.client_upload_rate  (float_of_int (len / 2));
+          Rate.update c.client_upload_rate  (float_of_int len);
           file.file_uploaded <- file.file_uploaded ++ (Int64.of_int len);
           let _ =
             (* update stats *)
@@ -1477,11 +1518,9 @@ let rec iter_upload sock c =
           in
 (*          lprintf "sending piece\n"; *)
           send_client c (Piece (num, pos, upload_buffer, 0, len));
-          (* update stats *)
-          let uploaded = Int64.of_int len in
-          count_upload c file uploaded;
-          network_must_update network;
           iter_upload sock c
+	with e -> if !verbose then lprintf_nl ()
+		    "Exception %s in iter_upload" (Printexc2.to_string e)
         end else
         begin
 (*          lprintf "client is waiting for another piece\n"; *)
@@ -1502,9 +1541,14 @@ let client_can_upload c allowed =
       match c.client_upload_requests with
         [] -> ()
       | _ :: tail ->
-          CommonUploads.consume_bandwidth allowed;
-          c.client_allowed_to_write <-
-            c.client_allowed_to_write ++ (Int64.of_int allowed);
+    let new_allowed_to_write =
+      c.client_allowed_to_write ++ (Int64.of_int allowed) in
+      if allowed > 0 && CommonUploads.can_write_len sock
+            (Int64.to_int new_allowed_to_write)
+      then begin
+        CommonUploads.consume_bandwidth allowed;
+        c.client_allowed_to_write <- new_allowed_to_write;
+      end;
           iter_upload sock c
   )
 
