@@ -182,11 +182,9 @@ let value_to_file file_size file_state assocs =
             torrent_nodes = file_nodes;
 *)
             torrent_announce =
-            (
-              try
-                (List.hd file_trackers)
-              with _ -> ""
-            );
+            (match file_trackers with
+	      | h::q -> h
+	      | [] -> "");
             torrent_announce_list = file_trackers;
           } in
         let torrent_diskname = Filename.concat downloads_directory
@@ -229,7 +227,6 @@ let value_to_file file_size file_state assocs =
 
 let file_to_value file =
   try
-    let sources = Hashtbl2.to_list file.file_clients in
     let assocs =
       [
         "file_temp", string_to_value (Unix32.filename (file_fd file));
@@ -271,10 +268,6 @@ send us more clients.
       lprintf_nl () "exception %s in file_to_value"
         (Printexc2.to_string e); raise e
 
-let old_files =
-  define_option bittorrent_section ["old_files"]
-    "" (list_option (tuple2_option (string_option, int64_option))) []
-
 
 let save_config () =
   Options.save_with_help bittorrent_ini
@@ -283,7 +276,6 @@ let save_config () =
 let config_files_loaded = ref false
 
 let load _ =
-  lprintf_nl () "Loading stats";
   (try
       Options.load bt_stats_ini;
     with Sys_error _ -> ());
@@ -298,20 +290,20 @@ let new_stats_array () =
   )
 
 
-let gstats_by_brand = define_option bt_stats_section ["stats"] ""
+let gstats_array = define_option bt_stats_section ["stats"] ""
     (array_option StatsOption.t) (new_stats_array ())
 
 
 let _ =
-  option_hook gstats_by_brand (fun _ ->
-      let old_stats = !!gstats_by_brand in
+  option_hook gstats_array (fun _ ->
+      let old_stats = !!gstats_array in
       let old_len = Array.length old_stats in
       if old_len <> brand_count then
         let t = new_stats_array () in
         for i = 0 to old_len - 1 do
           t.(i) <- old_stats.(i)
         done;
-        gstats_by_brand =:= t
+        gstats_array =:= t
   )
 
 let diff_time = ref 0
@@ -336,4 +328,5 @@ let _ =
   (* Shut up message "Network.save/load_complex_options not implemented by BitTorrent" *)
   network.op_network_load_complex_options <- load;
   network.op_network_save_complex_options <- save;
+  network.op_network_update_options <- (fun _ -> ());
   network.op_network_save_sources <- (fun _ -> ())
