@@ -88,6 +88,8 @@ let op_file_debug file =
   Buffer.contents buf
 
 let op_file_commit file new_name =
+  Int64Swarmer.remove_swarmer file.file_swarmer;
+  file.file_swarmer <- None;
   if file_state file <> FileShared then
     begin
       if not (List.mem (file.file_name, file_size file) !!old_files) then
@@ -95,7 +97,7 @@ let op_file_commit file new_name =
       set_file_state file FileShared;
 
       if Unix32.destroyed (file_fd file) then
-        if !verbose_files then lprintf_nl () "op_file_commit: FD is destroyed... repairing";
+        if !verbose then lprintf_nl () "op_file_commit: FD is destroyed... repairing";
 
 (* During the commit operation, for security, the file_fd is destroyed. So
   we create it again to be able to share this file again. *)
@@ -115,7 +117,7 @@ let op_file_commit file new_name =
               file.file_torrent_diskname new_torrent_diskname));
       file.file_torrent_diskname <- new_torrent_diskname;
 
-    end	
+    end 
 
 let op_file_print_html file buf =
 
@@ -126,13 +128,22 @@ let op_file_print_html file buf =
     ("Filename", "sr br", "Filename");
     ("", "sr", file.file_name) ];
 
+  Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+  html_mods_td buf [
+    ("Search for other possible Torrent Files", "sr br", "Torrent Srch");
+    ("", "sr", Printf.sprintf "\\<a target=\\\"_blank\\\" href=\\\"http://isohunt.com/%s\\\"\\>IsoHunt\\</a\\>"
+         (file.file_name)
+      )
+ ];
+
   Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
   html_mods_td buf [
     ("Tracker(s)", "sr br", "Tracker(s)");
     ("", "sr",
       (let tracker_string = ref "" in
        List.iter (fun tracker ->
-        tracker_string := !tracker_string ^ (shorten tracker.tracker_url !!max_name_len) ^ " "
+         tracker_string := (if tracker.tracker_enabled then "" else "*") ^ 
+	   !tracker_string ^ (shorten tracker.tracker_url !!max_name_len) ^ " "
       ) file.file_trackers;
       Printf.sprintf "%s" !tracker_string)) ];
 
@@ -176,66 +187,82 @@ let op_file_print_html file buf =
         "" -> "-"
       | _ -> file.file_encoding) ];
 
-  match file.file_trackers with
-      [] -> ()
-    | t :: _ ->
-      Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
-      html_mods_td buf [
-        ("Last Connect", "sr br", "Last Connect");
-        ("", "sr", string_of_date t.tracker_last_conn) ];
+  let rec print_first_tracker l =
+    match l with
+      | [] -> ()
+      | t :: q ->
+	  if not t.tracker_enabled then print_first_tracker q
+	  else begin
+	    Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+	    html_mods_td buf [
+              ("Last Connect", "sr br", "Last Connect");
+              ("", "sr", string_of_date t.tracker_last_conn) ];
 
-      Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
-      html_mods_td buf [
-        ("Connect Interval", "sr br", "Con Interval");
-        ("", "sr", Printf.sprintf "%d" t.tracker_interval) ];
+	    Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+	    html_mods_td buf [
+              ("Connect Interval", "sr br", "Con Interval");
+              ("", "sr", Printf.sprintf "%d" t.tracker_interval) ];
 
-      Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
-      html_mods_td buf [
-        ("Connect Min Interval", "sr br", "Con Min Interval");
-        ("", "sr", Printf.sprintf "%d" t.tracker_min_interval) ];
-      (* show only interesting answers*)
-      if t.tracker_torrent_downloaded > 0 then begin
-          Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
-          html_mods_td buf [
-            ("Downloaded", "sr br", "Downloaded");
-            ("", "sr", Printf.sprintf "%d" t.tracker_torrent_downloaded) ]
-        end;
-      if t.tracker_torrent_complete > 0 then begin
-          Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
-          html_mods_td buf [
-            ("Complete (seeds)", "sr br", "Complete");
-            ("", "sr", Printf.sprintf "%d" t.tracker_torrent_complete) ]
-        end;
-      if t.tracker_torrent_incomplete > 0 then begin
-          Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
-          html_mods_td buf [
-            ("Incomplete (peers)", "sr br", "Incomplete");
-            ("", "sr", Printf.sprintf "%d" t.tracker_torrent_incomplete) ]
-        end;
-      if t.tracker_torrent_total_clients_count > 0 then begin
-          Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
-          html_mods_td buf [
-            ("Total client count", "sr br", "All clients");
-            ("", "sr", Printf.sprintf "%d" t.tracker_torrent_total_clients_count) ]
-        end;
-      if t.tracker_torrent_last_dl_req > 0 then begin
-          Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
-          html_mods_td buf [
-            ("Latest torrent request", "sr br", "Latest request");
-            ("", "sr", Printf.sprintf "%ds" t.tracker_torrent_last_dl_req) ]
-        end;
-      if String.length t.tracker_id > 0 then begin
-          Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
-          html_mods_td buf [
-           ("Tracker id", "sr br", "Tracker id");
-           ("", "sr", Printf.sprintf "%s" t.tracker_id) ]
-        end;
-      if String.length t.tracker_key > 0 then begin
-          Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
-          html_mods_td buf [
-           ("Tracker key", "sr br", "Tracker key");
-           ("", "sr", Printf.sprintf "%s" t.tracker_key) ]
-        end
+	    Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+	    html_mods_td buf [
+              ("Connect Min Interval", "sr br", "Con Min Interval");
+              ("", "sr", Printf.sprintf "%d" t.tracker_min_interval) ];
+	    (* show only interesting answers*)
+	    if t.tracker_torrent_downloaded > 0 then begin
+              Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+              html_mods_td buf [
+		("Downloaded", "sr br", "Downloaded");
+		("", "sr", Printf.sprintf "%d" t.tracker_torrent_downloaded) ]
+            end;
+	    if t.tracker_torrent_complete > 0 then begin
+              Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+              html_mods_td buf [
+		("Complete (seeds)", "sr br", "Complete");
+		("", "sr", Printf.sprintf "%d" t.tracker_torrent_complete) ]
+            end;
+	    if t.tracker_torrent_incomplete > 0 then begin
+              Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+              html_mods_td buf [
+		("Incomplete (peers)", "sr br", "Incomplete");
+		("", "sr", Printf.sprintf "%d" t.tracker_torrent_incomplete) ]
+            end;
+	    if t.tracker_torrent_total_clients_count > 0 then begin
+              Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+              html_mods_td buf [
+		("Total client count", "sr br", "All clients");
+		("", "sr", Printf.sprintf "%d" t.tracker_torrent_total_clients_count) ]
+            end;
+	    if t.tracker_torrent_last_dl_req > 0 then begin
+              Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+              html_mods_td buf [
+		("Latest torrent request", "sr br", "Latest request");
+		("", "sr", Printf.sprintf "%ds" t.tracker_torrent_last_dl_req) ]
+            end;
+	    if String.length t.tracker_id > 0 then begin
+              Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+              html_mods_td buf [
+		("Tracker id", "sr br", "Tracker id");
+		("", "sr", Printf.sprintf "%s" t.tracker_id) ]
+            end;
+	    if String.length t.tracker_key > 0 then begin
+              Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+              html_mods_td buf [
+		("Tracker key", "sr br", "Tracker key");
+		("", "sr", Printf.sprintf "%s" t.tracker_key) ]
+	    end 
+	  end in
+  print_first_tracker file.file_trackers;
+
+  let cntr = ref 0 in
+  List.iter (fun (filename, size) ->
+    Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+    let fs = Printf.sprintf "File %d" !cntr in
+    html_mods_td buf [
+      (fs, "sr br", fs);
+      ("", "sr", (Printf.sprintf "%s (%Ld bytes)" filename size)) 
+    ];
+    incr cntr;
+  ) file.file_files
 
 let op_file_print_sources_html file buf =
 
@@ -245,29 +272,29 @@ let op_file_print_sources_html file buf =
             None -> "" | Some swarmer ->
               Int64Swarmer.verified_bitmap swarmer) in
 
-      html_mods_table_header buf "sourcesTable" "sources al" [
+      let header_list = [
         ( "1", "srh br ac", "Client number", "Num" ) ;
         ( "0", "srh br", "Client UID", "UID" ) ;
         ( "0", "srh br", "Client software", "Soft" ) ;
         ( "0", "srh", "IP address", "IP address" ) ;
         ( "0", "srh br ar", "Port", "Port" ) ;
+        ] @ (if !Geoip.active then [( "0", "srh br ar", "Country Code/Name", "CC" )] else []) @ [
         ( "1", "srh ar", "Total UL bytes to this client for all files", "UL" ) ;
         ( "1", "srh ar br", "Total DL bytes from this client for all files", "DL" ) ;
-        ( "1", "srh ar", "Interested [T]rue, [F]alse", "I" ) ;
-        ( "1", "srh ar", "Choked [T]rue, [F]alse", "C" ) ;
+        ( "0", "srh ar", "Interested [T]rue, [F]alse", "I" ) ;
+        ( "0", "srh ar", "Choked [T]rue, [F]alse", "C" ) ;
         ( "1", "srh br ar", "Allowed to write", "A" ) ;
+        ( "0", "srh ar", "Interesting [T]rue, [F]alse", "I" );
+        ( "0", "srh ar", "Already sent interested [T]rue, [F]alse", "A" );
+        ( "0", "srh br ar", "Already sent not interested [T]rue, [F]alse", "N" );
 
-        ( "1", "srh ar", "Interesting [T]rue, [F]alse", "I" );
-        ( "1", "srh ar", "Already sent interested [T]rue, [F]alse", "A" );
-        ( "1", "srh br ar", "Already sent not interested [T]rue, [F]alse", "N" );
+        ( "0", "srh ar", "Good [T]rue, [F]alse", "G" );
+        ( "0", "srh ar", "Incoming [T]rue, [F]alse", "I" );
+        ( "0", "srh br ar", "Registered bitfield [T]rue, [F]alse", "B" );
 
-        ( "1", "srh ar", "Good [T]rue, [F]alse", "G" );
-        ( "1", "srh ar", "Incoming [T]rue, [F]alse", "I" );
-        ( "1", "srh br ar", "Registered bitfield [T]rue, [F]alse", "B" );
-
-        ( "1", "srh ar", "Optimist Time", "O" );
-        ( "1", "srh ar", "Last optimist", "L.Opt" );
-        ( "1", "srh br ar", "Num try", "N" );
+        ( "0", "srh ar", "Optimist Time", "O" );
+        ( "0", "srh ar", "Last optimist", "L.Opt" );
+        ( "0", "srh br ar", "Num try", "N" );
 
 (*
         ( "0", "srh", "Bitmap (absent|partial|present|verified)", (colored_chunks
@@ -276,17 +303,22 @@ let op_file_print_sources_html file buf =
 *)
         ( "1", "srh ar", "Number of full chunks", (Printf.sprintf "%d"
               (String.length (String2.replace
-                  (String2.replace chunks '0' "") '1' "")) )) ]      ;
+                  (String2.replace chunks '0' "") '1' "")) )) 
+      ] in
+
+      html_mods_table_header buf "sourcesTable" "sources al" header_list;
 
       Hashtbl.iter (fun _ c ->
           Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr());
 
-          html_mods_td buf [
+          let cc,cn = Geoip.get_country (fst c.client_host) in
+          let td_list = [
             ("", "sr br ar", Printf.sprintf "%d" (client_num c));
             ("", "sr br", (Sha1.to_string c.client_uid));
-            ("", "sr br", c.client_software);
+            ("", "sr br", Printf.sprintf "%s %s" (brand_to_string c.client_brand) c.client_release);
             ("", "sr", (Ip.to_string (fst c.client_host)));
             ("", "sr br ar", Printf.sprintf "%d" (snd c.client_host));
+            ] @ (if !Geoip.active then [( cn, "sr br", cc)] else []) @ [
             ("", "sr ar", (size_of_int64 c.client_uploaded));
             ("", "sr ar br", (size_of_int64 c.client_downloaded));
             ("", "sr", (if c.client_interested then "T" else "F"));
@@ -312,9 +344,11 @@ let op_file_print_sources_html file buf =
                 (match c.client_bitmap with
                     None -> ()
                   | Some bitmap ->
-                      String.iter (fun s -> if s = '1' then incr fc) bitmap );
-                (Printf.sprintf "%d" !fc) ) ) ];
+                      Bitv.iter (fun s -> if s then incr fc) bitmap);
+                (Printf.sprintf "%d" !fc) ) ) 
+          ] in
 
+          html_mods_td buf td_list;
           Printf.bprintf buf "\\</tr\\>";
 
       ) file.file_clients;
@@ -331,12 +365,18 @@ let op_file_check file =
   | Some swarmer ->
       Int64Swarmer.verify_all_chunks swarmer true
 
+let remove_all_clients file =
+  Hashtbl.clear file.file_clients; 
+  file.file_clients_num <- 0
+
 let op_file_cancel file =
+  Int64Swarmer.remove_swarmer file.file_swarmer;
   file.file_swarmer <- None;
   BTClients.file_stop file;
   remove_file file;
   BTClients.disconnect_clients file;
-  Sys.remove file.file_torrent_diskname
+  remove_all_clients file;
+  if Sys.file_exists file.file_torrent_diskname then Sys.remove file.file_torrent_diskname
 
 let op_ft_cancel ft =
   Hashtbl.remove ft_by_num ft.ft_id
@@ -351,22 +391,12 @@ let op_file_info file =
   let last_seen = match file.file_swarmer with
       None -> [| last_time () |]
     | Some swarmer -> Int64Swarmer.compute_last_seen swarmer in
-  {
-    P.file_fields = P.Fields_file_info.all;
 
-    P.file_comment = file_comment (as_file file);
+    { (impl_file_info file.file_file) with
+
     P.file_name = file.file_name;
-    P.file_num = (file_num file);
     P.file_network = network.network_num;
     P.file_names = [file.file_name, P.noips()];
-    P.file_md4 = Md4.null;
-    P.file_size = file_size file;
-    P.file_downloaded = file_downloaded file;
-    P.file_all_sources = 0;
-    P.file_active_sources = 0;
-    P.file_state = file_state file;
-    P.file_sources = None;
-    P.file_download_rate = file_download_rate file.file_file;
     P.file_chunks = (match file.file_swarmer with
         None -> "" | Some swarmer ->
           Int64Swarmer.verified_bitmap swarmer);
@@ -374,12 +404,10 @@ let op_file_info file =
     [network.network_num,(match file.file_swarmer with
           None -> "" | Some swarmer ->
             Int64Swarmer.availability swarmer)];
-    P.file_format = FormatNotComputed 0;
+
     P.file_chunks_age = last_seen;
-    P.file_age = file_age file;
-    P.file_last_seen = BasicSocket.last_time ();
-    P.file_priority = file_priority (as_file file);
     P.file_uids = [Uid.create (BTUrl file.file_id)];
+    P.file_sub_files = file.file_files;
   }
 
 let op_ft_info ft =
@@ -410,6 +438,7 @@ let op_ft_info ft =
     P.file_last_seen = BasicSocket.last_time ();
     P.file_priority = 0;
     P.file_uids = [];
+    P.file_sub_files = [];
   }
 
 
@@ -425,8 +454,8 @@ let load_torrent_string s =
     in
   File.from_string torrent_diskname s;
 
-  if !verbose_torrent then
-    lprintf_nl () "Starting torrent download with diskname:%s"
+  if !verbose then
+    lprintf_nl () "Starting torrent download with diskname: %s"
         torrent_diskname;
   let file = new_download file_id torrent torrent_diskname in
   BTClients.get_sources_from_tracker file;
@@ -435,8 +464,8 @@ let load_torrent_string s =
   file
 
 let load_torrent_file filename =
-  if !verbose_torrent then
-    lprintf_nl () "BTInteractive.load_torrent_file %s" filename;
+  if !verbose then
+    lprintf_nl () "load_torrent_file %s" filename;
   let s = File.to_string filename in
   (* Delete the torrent if it is in the downloads dir. because it gets saved
      again under the torrent name and we don't want to clutter up this dir. .*)
@@ -557,7 +586,8 @@ let share_files _ =
           if !verbose_share then lprintf_nl () "Removing torrent share for %s" file.file_torrent_diskname;
           BTClients.file_stop file;
           remove_file file;
-          BTClients.disconnect_clients file
+          BTClients.disconnect_clients file;
+          remove_all_clients file;
         end
   ) shared_files_copy
 
@@ -568,14 +598,14 @@ let retry_all_ft () =
   ) ft_by_num
 
 let load_torrent_from_web r ft =
-  if !verbose_torrent then
+  if !verbose then
       lprintf_nl () "Loading torrent from web";
   let module H = Http_client in
 
-  if !verbose_torrent then
+  if !verbose then
       lprintf_nl () "calling...";
   H.wget r (fun filename ->
-      if !verbose_torrent then
+      if !verbose then
           lprintf_nl () "done...";
       if ft_state ft = FileDownloading then begin
           load_torrent_file filename;
@@ -584,7 +614,7 @@ let load_torrent_from_web r ft =
 
 let valid_torrent_extension url =
   let ext = String.lowercase (Filename2.last_extension url) in
-  if !verbose_torrent then lprintf_nl () "Last extension: %s" ext;
+  if !verbose then lprintf_nl () "Last extension: %s" ext;
   if ext = ".torrent" || ext = ".tor" then true else false
 
 let get_regexp_string text r =
@@ -597,7 +627,7 @@ let op_network_parse_url url =
   let location_regexp = "Location: \\(.*\\)" in
   try
     let real_url = get_regexp_string url (Str.regexp location_regexp) in
-    if !verbose_torrent then lprintf_nl () "Trying to load %s really %s" url real_url;
+    if !verbose then lprintf_nl () "Loading %s, really %s" url real_url;
     if (valid_torrent_extension real_url)
        || (String2.contains url "Content-Type: application/x-bittorrent")
       then (
@@ -607,7 +637,7 @@ let op_network_parse_url url =
             H.basic_request with
             H.req_url = u;
             H.req_proxy = !CommonOptions.http_proxy;
-            H.req_user_agent = Printf.sprintf "MLdonkey/%s" Autoconf.current_version;
+            H.req_user_agent = get_user_agent ();
             H.req_referer = (
               let referers = !!BTOptions.referers in
               let (rule_search,rule_value) =
@@ -616,7 +646,7 @@ let op_network_parse_url url =
                     ) referers )
                 with Not_found -> ("",real_url) in
               Some (Url.of_string rule_value) );
-            H.req_headers = try
+            H.req_headers = (try
               let cookies = List.assoc u.Url.server !!BTOptions.cookies in
               [ ( "Cookie", List.fold_left (fun res (key, value) ->
                       if res = "" then
@@ -625,64 +655,58 @@ let op_network_parse_url url =
                         res ^ "; " ^ key ^ "=" ^ value
                   ) "" cookies
                 ) ]
-            with Not_found -> [];
+            with Not_found -> []);
+      H.req_max_retry = 10;
           } in
 
         let file_diskname = Filename.basename u.Url.short_file in
         let ft = new_ft file_diskname in
         ft.ft_retry <- load_torrent_from_web r ;
         load_torrent_from_web r ft;
-        if !verbose_torrent then lprintf_nl () "wget started";
-        true
+        if !verbose then lprintf_nl () "wget started";
+        "started download", true
       )
     else
-      false
+      "", false
   with
     | Not_found ->
       if (valid_torrent_extension url) then
         try
-          if !verbose_torrent then lprintf_nl () "Not_found and trying to load %s" url;
+          if !verbose then lprintf_nl () "Not_found and trying to load %s" url;
           load_torrent_file url;
-          true
+          "", true
         with e ->
           lprintf_nl () "Exception %s while 2nd loading" (Printexc2.to_string e);
-          false
+	  let s = Printf.sprintf "Can not load load torrent file: %s"
+	    (Printexc2.to_string e) in
+          s, false
       else
         begin
-          if !verbose_torrent then lprintf_nl () "Not_found and url has non valid torrent extension: %s" url;
-          false
+          if !verbose then lprintf_nl () "Not_found and url has non valid torrent extension: %s" url;
+          "Not_found and url has non valid torrent extension", false
         end
     | e ->
        lprintf_nl () "Exception %s while loading" (Printexc2.to_string e);
-       false
+       let s = Printf.sprintf "Can not load load torrent file: %s"
+	 (Printexc2.to_string e) in
+       s, false
 
 let op_client_info c =
   let module P = GuiTypes in
   let (ip,port) = c.client_host in
-  let id = c.client_uid in
-  {
+  { (impl_client_info c.client_client) with
+
     P.client_network = network.network_num;
     P.client_kind = Known_location (ip,port);
     P.client_state = client_state (as_client c);
     P.client_type = client_type c;
-    P.client_tags = [];
-    P.client_name =
-    (Printf.sprintf "%s:%d" (Ip.to_string ip) port);
-    P.client_files = None;
-    P.client_num = (client_num c);
-    P.client_rating = 0;
-    P.client_chat_port = 0 ;
-    P.client_connect_time = BasicSocket.last_time ();
-    P.client_software = c.client_software;
-(* TODO: switch it
+    P.client_name = (Printf.sprintf "%s:%d" (Ip.to_string ip) port);
+    P.client_software = (brand_to_string c.client_brand);
     P.client_release = c.client_release;
- *)
-    P.client_release = "";
-    P.client_emulemod = "";
     P.client_downloaded = c.client_downloaded;
     P.client_uploaded = c.client_uploaded;
     P.client_upload = Some (c.client_file.file_name);
-(*          P.client_sock_addr = (Ip.to_string ip); *)
+
   }
 
 let op_client_connect c =
@@ -702,23 +726,24 @@ let op_client_dprint c o file =
   let info = file_info file in
   let buf = o.conn_buf in
   let cc = as_client c in
-  let cinfo = client_info cc in
   client_print cc o;
   Printf.bprintf buf (_b "\n%18sDown  : %-10s                  Uploaded: %-10s  Ratio: %s%1.1f (%s)\n") ""
     (Int64.to_string c.client_downloaded)
   (Int64.to_string c.client_uploaded)
   (if c.client_downloaded > c.client_uploaded then "-" else "+")
-  (if c.client_uploaded > Int64.zero then (Int64.to_float (Int64.div c.client_downloaded c.client_uploaded)) else (1.))
+  (if c.client_uploaded > Int64.zero then 
+     Int64.to_float (c.client_downloaded // c.client_uploaded) 
+   else 1.)
   ("BT");
   (Printf.bprintf buf (_b "%18sFile  : %s\n") "" info.GuiTypes.file_name)
 
 let op_client_dprint_html c o file str =
   let info = file_info file in
   let buf = o.conn_buf in
-  let cc = as_client c in
-  let cinfo = client_info cc in
+  let ac = as_client c in
+  let cinfo = client_info ac in
   Printf.bprintf buf " \\<tr onMouseOver=\\\"mOvr(this);\\\"
-	onMouseOut=\\\"mOut(this);\\\" class=\\\"%s\\\"\\>" str;
+  onMouseOut=\\\"mOut(this);\\\" class=\\\"%s\\\"\\>" str;
 
   let show_emulemods_column = ref false in
   if Autoconf.donkey = "yes" then begin
@@ -726,16 +751,15 @@ let op_client_dprint_html c o file str =
         show_emulemods_column := true
     end;
 
+    let cc,cn = Geoip.get_country (fst c.client_host) in
+
     html_mods_td buf ([
         ("", "srb ar", Printf.sprintf "%d" (client_num c));
-        ((string_of_connection_state (client_state cc)), "sr",
-          (short_string_of_connection_state (client_state cc)));
+        ((string_of_connection_state (client_state ac)), "sr",
+          (short_string_of_connection_state (client_state ac)));
         ((Sha1.to_string c.client_uid), "sr", cinfo.GuiTypes.client_name);
-        ("", "sr", "bT"); (* cinfo.GuiTypes.client_software *)
-(* TODO : switch it
-        ("", "sr", cinfo.GuiTypes.client_release); (* cinfo.GuiTypes.client_release *)
- *)
-        ("", "sr", ""); (* cinfo.GuiTypes.client_release *)
+        ("", "sr", (brand_to_string c.client_brand)); (* cinfo.GuiTypes.client_software *)
+        ("", "sr", c.client_release); 
       ] @
         (if !show_emulemods_column then [("", "sr", "")] else [])
       @ [
@@ -743,7 +767,9 @@ let op_client_dprint_html c o file str =
         ("", "sr ar", Printf.sprintf "%d"
             (((last_time ()) - cinfo.GuiTypes.client_connect_time) / 60));
         ("", "sr", "D");
+        ("", "sr", "N");
         ("", "sr", (Ip.to_string (fst c.client_host)));
+        ] @ (if !Geoip.active then [(cn, "sr", cc)] else []) @ [
         ("", "sr ar", (size_of_int64 c.client_uploaded));
         ("", "sr ar", (size_of_int64 c.client_downloaded));
         ("", "sr", info.GuiTypes.file_name); ]);
@@ -762,7 +788,7 @@ let get_default_tracker () =
 
 let compute_torrent filename announce comment = 
   let announce = if announce = "" then get_default_tracker () else announce in
-  if !verbose_torrent then lprintf_nl () "compute_torrent: [%s] [%s] [%s]"
+  if !verbose then lprintf_nl () "compute_torrent: [%s] [%s] [%s]"
    filename announce comment;
   let basename = Filename.basename filename in
   let torrent = Filename.concat seeded_directory
@@ -884,6 +910,52 @@ let commands =
        _s ""
     ), _s " :\t\t\t\tstops all bittorrent downloads, use this if you want to make sure that the stop signal actualy gets to the tracker\n\t\t\t\twhen shuting mlnet down, but you have to wait till the stops get to the tracker and not wait too long,\n\t\t\t\tso mldonkey reconnects to the tracker :)";
 
+    "tracker", "Network/Bittorrent", Arg_multiple (fun args o ->
+        try
+          let num = ref "" in
+          let urls = ref [] in
+          (match args with
+          | nums :: [] -> raise Not_found
+          | nums :: rest -> num := nums; urls := rest
+          | _ -> raise Not_found);
+
+          let num = int_of_string !num in
+          Hashtbl.iter (fun _ file ->
+              if file_num file = num then begin
+                  if !verbose then
+                    lprintf_nl () "adding trackers for file %i" num;
+                  set_trackers file !urls;
+                  raise Exit
+                end
+          ) files_by_uid;
+         let buf = o.conn_buf in
+          if o.conn_output = HTML then
+            html_mods_table_one_row buf "serversTable" "servers" [
+              ("", "srh", "file not found"); ]
+          else
+            Printf.bprintf buf "file not found";
+          _s ""
+        with
+        | Exit -> 
+            let buf = o.conn_buf in
+            if o.conn_output = HTML then
+              html_mods_table_one_row buf "serversTable" "servers" [
+                ("", "srh", "tracker added"); ]
+            else
+              Printf.bprintf buf "tracker added";
+            _s ""
+        | _ ->
+            if !verbose then
+              lprintf_nl () "Not enough or wrong parameters.";
+            let buf = o.conn_buf in
+            if o.conn_output = HTML then
+              html_mods_table_one_row buf "serversTable" "servers" [
+                ("", "srh", "Not enough or wrong parameters."); ]
+            else
+              Printf.bprintf buf "Not enough or wrong parameters.";
+            _s ""        
+    ), " <num> <url> <url>... :\t\t\tadd urls as trackers for num.";
+
 (* TODO : add some code from make_torrent
     "print_torrent", Arg_one (fun filename o ->
 
@@ -900,9 +972,8 @@ let op_gui_message s =
   match get_int16 s 0 with
     0 ->
       let text = String.sub s 2 (String.length s - 2) in
-      if !verbose_torrent then lprintf_nl () "received torrent from gui...";
-      let _ = load_torrent_string text in
-      ()
+      if !verbose then lprintf_nl () "received torrent from gui...";
+      ignore (load_torrent_string text)
   | 1 -> (* 34+ *)
       let n = get_int s 2 in
       let a, pos = get_string s 6 in
@@ -928,20 +999,21 @@ let _ =
   file_ops.op_file_check <- op_file_check;
   file_ops.op_file_cancel <- op_file_cancel;
   file_ops.op_file_info <- op_file_info;
-  (* We don't do anything here as the name to save as is defined by the
-     torrent. We could change the name if we changed the torrent, too. *)
   file_ops.op_file_save_as <- (fun file name -> ());
 
   network.op_network_gui_message <- op_gui_message;
   network.op_network_connected <- op_network_connected;
   network.op_network_parse_url <- op_network_parse_url;
-  (* Shut up "Network.share not implemented by BitTorrent" *)
   network.op_network_share <- (fun fullname codedname size -> ());
-  (* Same with Network.forget_search and Network.search... *)
   network.op_network_forget_search <- (fun s -> ());
+  network.op_network_connect_servers <- (fun s -> ());
   network.op_network_search <- (fun ss buf -> ());
-  (* and Network.recover_temp *)
+  network.op_network_download <- (fun r -> dummy_file);
   network.op_network_recover_temp <- (fun s -> ());
+  network.op_network_clean_exit <- (fun s ->
+    List.iter (fun file -> BTClients.file_stop file) !current_files;
+    List.for_all (fun file -> not file.file_tracker_connected) !current_files;
+  );
 
   client_ops.op_client_info <- op_client_info;
   client_ops.op_client_connect <- op_client_connect;
@@ -964,5 +1036,6 @@ let _ =
             T.shared_network = network.network_num;
             T.shared_filename = file_best_name (as_file file);
             T.shared_uids = [Uid.create (Sha1 file.file_id)];
+            T.shared_sub_files = file.file_files;
             }
   )
