@@ -31,6 +31,7 @@ open CommonFile
 open CommonTypes
 open CommonOptions
 open CommonGlobals
+open CommonResult
   
 open DonkeyTypes
 open DonkeyOptions
@@ -108,9 +109,6 @@ let value_to_md4 v =
 
 let value_to_client is_friend assocs = 
   let get_value name conv = conv (List.assoc name assocs) in
-  let get_value_nil name conv = 
-    try conv (List.assoc name assocs) with _ -> []
-  in
   let (ip,port) = 
     try
       get_value "client_addr" (fun v ->
@@ -132,7 +130,7 @@ let value_to_client is_friend assocs =
         Indirect_location(name, md4) *)
   in
   
-  let last_conn = 
+  let _ = 
     try
       let last_conn =
         (min (get_value "client_age" value_to_int) 
@@ -203,9 +201,6 @@ let client_option =
 
 let value_to_server assocs = 
   let get_value name conv = conv (List.assoc name assocs) in
-  let get_value_nil name conv = 
-    try conv (List.assoc name assocs) with _ -> []
-  in
   let ip, port = get_value "server_addr" (fun v ->
         match v with
           List [ip;port] | SmallList [ip;port] ->
@@ -386,7 +381,6 @@ let value_to_file file_size file_state assocs =
     else Array2.min file.file_chunks_age);
 *)
   
-  file_md4s_to_register := file :: !file_md4s_to_register;
   as_file file
   
 (*  
@@ -435,11 +429,6 @@ module SharedFileOption = struct
     let value_to_shinfo v =
       match v with
         Options.Module assocs ->
-          let get_value name conv = conv (List.assoc name assocs) in
-          let get_value_nil name conv = 
-            try conv (List.assoc name assocs) with _ -> []
-          in
-          
           let sh_md4s = try
               value_to_array (fun v ->
                   Md4.of_string (value_to_string v)) (List.assoc "md4s" assocs)
@@ -480,10 +469,6 @@ module StatsOption = struct
     let value_to_stat v =
       match v with
         Options.Module assocs ->
-          let get_value name conv = conv (List.assoc name assocs) in
-          let get_value_nil name conv = 
-            try conv (List.assoc name assocs) with _ -> []
-          in
           { 
             brand_seen = value_to_int (List.assoc "seen" assocs);
             brand_banned = value_to_int (List.assoc "banned" assocs);
@@ -512,27 +497,23 @@ module ModStatsOption = struct
     let value_to_mod_stat v =
       match v with
         Options.Module assocs ->
-          let get_value name conv = conv (List.assoc name assocs) in
-          let get_value_nil name conv = 
-            try conv (List.assoc name assocs) with _ -> []
-          in
           { 
-            brand_mod_seen = value_to_int (List.assoc "mseen" assocs);
-            brand_mod_banned = value_to_int (List.assoc "mbanned" assocs);
-            brand_mod_filerequest = value_to_int (List.assoc "mfilereqs" assocs);
-            brand_mod_download = value_to_int64 (List.assoc "mdownload" assocs);
-            brand_mod_upload = value_to_int64 (List.assoc "mupload" assocs);
+            brand_seen = value_to_int (List.assoc "mseen" assocs);
+            brand_banned = value_to_int (List.assoc "mbanned" assocs);
+            brand_filerequest = value_to_int (List.assoc "mfilereqs" assocs);
+            brand_download = value_to_int64 (List.assoc "mdownload" assocs);
+            brand_upload = value_to_int64 (List.assoc "mupload" assocs);
           }
           
       | _ -> failwith "Options: not a mod_stat option"
           
     let stat_to_mod_value b =
       Options.Module [
-        "mseen", int_to_value b.brand_mod_seen;
-        "mbanned", int_to_value b.brand_mod_banned;
-        "mfilereqs", int_to_value b.brand_mod_filerequest;
-        "mdownload", int64_to_value b.brand_mod_download;
-        "mupload", int64_to_value b.brand_mod_upload;
+        "mseen", int_to_value b.brand_seen;
+        "mbanned", int_to_value b.brand_banned;
+        "mfilereqs", int_to_value b.brand_filerequest;
+        "mdownload", int64_to_value b.brand_download;
+        "mupload", int64_to_value b.brand_upload;
       ]
     
     
@@ -570,10 +551,6 @@ module UidOption = struct
     
   end
 
-let old_files = 
-  define_option donkey_section ["old_files"] 
-  "The files that were downloaded" (list_option Md4.option) []
-
 let brotherhood = 
   define_option donkey_section ["brotherhood"] 
     "The links between files being downloaded" (list_option 
@@ -605,31 +582,22 @@ let force_add_server ip port =
     DonkeyGlobals.find_server ip port
   with _ ->
       let s = DonkeyGlobals.new_server ip port !!initial_score in
-      DonkeyGlobals.servers_ini_changed := true;
       s        
       
 let check_add_server ip port =
-  if Ip.valid ip && Ip.reachable ip &&
+  if Ip.usable ip &&
     not (is_black_address ip port) && port <> 4662 then
     force_add_server ip port
   else raise Not_found
 
 let safe_add_server ip port =
-  if Ip.valid ip && Ip.reachable ip &&
+  if Ip.usable ip &&
     not (is_black_address ip port) && port <> 4662 then
     try
       ignore (DonkeyGlobals.find_server ip port)
     with _ ->
-        let s = DonkeyGlobals.new_server ip port !!initial_score in
-        DonkeyGlobals.servers_ini_changed := true
+        ignore (DonkeyGlobals.new_server ip port !!initial_score)
         
-let remove_server ip port =
-  try
-    let s = DonkeyGlobals.find_server ip port in
-    DonkeyGlobals.servers_ini_changed := true;
-    DonkeyGlobals.remove_server ip port
-  with _ -> ()
-
 let config_files_loaded = ref false  
       
 let load _ =
@@ -651,37 +619,37 @@ let new_stats_array () =
   
 let new_mod_stats_array () = 
   Array.init brand_mod_count (fun _ ->
-      { dummy_mod_stats with brand_mod_seen = 0 }
+      { dummy_stats with brand_seen = 0 }
   )
 
-let gstats_by_brand = define_option stats_section ["stats"] "" 
+let gstats_array = define_option stats_section ["stats"] "" 
     (array_option StatsOption.t) (new_stats_array ())
 
-let gstats_by_brand_mod = define_option mod_stats_section ["stats"] "" 
+let gstats_mod_array = define_option mod_stats_section ["stats"] "" 
     (array_option ModStatsOption.t) (new_mod_stats_array ())
 
 let _ =
-  option_hook gstats_by_brand (fun _ ->
-      let old_stats = !!gstats_by_brand in
+  option_hook gstats_array (fun _ ->
+      let old_stats = !!gstats_array in
       let old_len = Array.length old_stats in
       if old_len <> brand_count then
         let t = new_stats_array () in
         for i = 0 to old_len - 1 do
           t.(i) <- old_stats.(i)
         done;
-        gstats_by_brand =:= t
+        gstats_array =:= t
   )
   
 let _ =
-  option_hook gstats_by_brand_mod (fun _ ->
-      let old_mod_stats = !!gstats_by_brand_mod in
+  option_hook gstats_mod_array (fun _ ->
+      let old_mod_stats = !!gstats_mod_array in
       let old_mod_len = Array.length old_mod_stats in
       if old_mod_len <> brand_mod_count then
         let t = new_mod_stats_array () in
         for i = 0 to old_mod_len - 1 do
           t.(i) <- old_mod_stats.(i)
         done;
-        gstats_by_brand_mod =:= t
+        gstats_mod_array =:= t
   )
 
 let diff_time = ref 0
@@ -708,15 +676,18 @@ let save_sources _ =
 
 let guptime () = !!guptime - !diff_time
   
-let load_sources () = 
-  (try 
-(*      lprintf "load_sources: loading sources\n"; *)
+let load_sources () =
+  (lprintf "(loading sources ";
+   try 
       let cleaner = DonkeySources.attach_sources_to_file file_sources_section in
       cleaner ();
+      lprintf ".";
       Options.load file_sources_ini;
+      lprintf ".";
       cleaner ();
+      lprintf ".";
       sources_loaded := true;
-(*      lprintf "load_sources: sources loaded\n"; *)
+      lprintf " completed) ";
       List.iter (fun list ->
           let files = ref [] in
           List.iter (fun m ->
@@ -732,6 +703,45 @@ let load_sources () =
       ) !!brotherhood
     with _ -> ())
 
+let check_result r tags =
+  if r.result_names = [] || r.result_size = Int64.zero then begin
+      if !verbose then begin
+          lprintf_n () "BAD RESULT:";
+          List.iter (fun tag ->
+              lprintf "[%s] = [%s]" (string_of_field tag.tag_name)
+                (string_of_tag_value tag.tag_value);
+              lprint_newline ();
+          ) tags;
+        end;
+      false
+    end
+  else true
+
+(* Inserted in complexOptions to be able to access old_files (ugly) *)
+let result_of_file md4 tags =
+  let rec r = {  dummy_result with
+      result_uids = [Uid.create (Ed2k md4)];
+      result_done = (List.mem md4 !!old_files) || (Hashtbl.mem files_by_md4 md4); 
+    } in
+  List.iter (fun tag ->
+      match tag with
+        { tag_name = Field_Filename; tag_value = String s } ->
+          r.result_names <- s :: r.result_names
+      | { tag_name = Field_Size; tag_value = Uint64 v } ->
+          r.result_size <- v;
+      | { tag_name = Field_Format; tag_value = String s } ->
+          r.result_tags <- tag :: r.result_tags;
+          r.result_format <- s
+      | { tag_name = Field_Type; tag_value = String s } ->
+          r.result_tags <- tag :: r.result_tags;
+          r.result_type <- s
+      | _ ->
+          r.result_tags <- tag :: r.result_tags
+  ) tags;
+  if check_result r tags then
+    let rs = update_result_num r in
+    Some rs
+  else None
     
   
 let _ =
