@@ -61,7 +61,7 @@ let network = CommonNetwork.new_network "ED2K" "Donkey"
   ]
 
 let connection_manager = network.network_connection_manager
-let connections_controler = TcpServerSocket.create_connections_contoler
+let connections_controler = TcpServerSocket.create_connections_controler
     "Edonkey" (fun _ _ -> true)
 
 let (shared_ops : file CommonShared.shared_ops) =
@@ -100,7 +100,7 @@ let file_priority file = file.file_file.impl_file_priority
 let file_size file = file.file_file.impl_file_size
 let file_downloaded file = file_downloaded (as_file file)
 let file_age file = file.file_file.impl_file_age
-let file_fd file = file.file_file.impl_file_fd
+let file_fd file = file_fd (as_file file)
 let file_disk_name file = file_disk_name (as_file file)
 let file_best_name file = file_best_name (as_file file)
 
@@ -120,7 +120,7 @@ let tag_client = 200
 let tag_server = 201
 let tag_file   = 202
 
-let page_size = Int64.of_int 4096
+let page_size = 4096L
 
 let donkey_download_counter = ref Int64.zero
 let donkey_upload_counter = ref Int64.zero
@@ -143,7 +143,7 @@ let nservers = ref 0
 let xs_last_search = ref (-1)
 
 let zone_size = Int64.of_int (180 * 1024)
-let block_size = Int64.of_int 9728000
+let block_size = 9728000L
 
 let queue_timeout = ref (60. *. 10.) (* 10 minutes *)
 
@@ -157,7 +157,6 @@ let master_server = ref (None: DonkeyTypes.server option)
 let udp_sock = ref (None: UdpSocket.t option)
 let listen_sock = ref (None : TcpServerSocket.t option)
 let reversed_sock = ref (None : TcpServerSocket.t option)
-let servers_ini_changed = ref true
 let new_shared = ref false
 
 (*************************************************************************)
@@ -166,7 +165,7 @@ let new_shared = ref false
 (*                                                                       *)
 (*************************************************************************)
 
-module H = Weak2.Make(struct
+module H = Weak.Make(struct
       type t = client
       let hash c = Hashtbl.hash c.client_kind
 
@@ -183,6 +182,7 @@ let servers_list = ref ([] : server list)
 let current_files = ref ([] : file list)
 let xs_servers_list = ref ([] : server list)
 let connected_server_list = ref ([]  : server list)
+let connecting_server_list = ref ([] : server list)
 
 let (banned_ips : (Ip.t, int) Hashtbl.t) = Hashtbl.create 113
 let (old_requests : (int * int, request_record) Hashtbl.t) =
@@ -208,9 +208,7 @@ let udp_servers_replies = (Hashtbl.create 127 : (Md4.t, server) Hashtbl.t)
 
 let file_groups = (Hashtbl.create 1023 : (Md4.t, file_group) Hashtbl.t)
 
-let file_md4s_to_register = ref ([] : file list)
-
-module UdpClientWHashtbl = Weak2.Make(struct
+module UdpClientWHashtbl = Weak.Make(struct
       type t = udp_client
       let hash c = Hashtbl.hash (c.udp_client_ip, c.udp_client_port)
 
@@ -249,6 +247,15 @@ let hashtbl_remove table key v =
       Hashtbl.remove table key
   with _ -> ()
 
+let add_connecting_server c =
+  connecting_server_list := c :: !connecting_server_list
+
+let remove_connecting_server c =
+  connecting_server_list := List2.removeq c !connecting_server_list
+
+let connecting_server_ips () =
+  List.rev_map (fun s -> s.server_ip) !connecting_server_list
+
 let add_connected_server c =
   connected_server_list := c :: !connected_server_list
 
@@ -280,9 +287,7 @@ let update_best_name file =
       match impl.impl_file_probable_name with
         None -> ()
       | Some best_name ->
-          let best_name = String2.replace best_name '/' "::" in
-          set_file_best_name file best_name;
-          if !verbose then lprintf_nl () "BEST NAME now IS %s" best_name;
+          set_file_best_name file best_name
     with Not_found -> ()
 
 let new_file file_diskname file_state md4 file_size filenames writable =
@@ -312,7 +317,7 @@ let new_file file_diskname file_state md4 file_size filenames writable =
           && file.file_diskname = file_diskname
         then
           file.file_file.impl_file_fd <-
-            Unix32.create_diskfile file.file_diskname Unix32.rw_flag 0o666;
+            Some (Unix32.create_diskfile file.file_diskname true);
       if Unix32.destroyed (file_fd file) then
           lprintf_nl () "New Edonkey file with %b && %b remaining destroyed fd %s"
             (not writable) (file.file_diskname = file_diskname) file.file_diskname;
@@ -320,7 +325,7 @@ let new_file file_diskname file_state md4 file_size filenames writable =
   with _ ->
       if !verbose_share then
         lprintf_nl () "New file with md4: %s" (Md4.to_string md4);
-      let file_exists = Unix32.file_exists file_diskname in
+      ignore (Unix32.file_exists file_diskname);
 
       let t =
         if
@@ -331,24 +336,35 @@ let new_file file_diskname file_state md4 file_size filenames writable =
 (* Only if the file does not already exists *)
           not (Sys.file_exists file_diskname)
         then
-          Unix32.create_sparsefile file_diskname
+          Unix32.create_sparsefile file_diskname writable
         else
-          Unix32.create_diskfile file_diskname Unix32.rw_flag 0o666
+          Unix32.create_diskfile file_diskname writable
       in
       let file_size =
         if file_size = Int64.zero then
           try
-            Unix32.getsize file_diskname writable
+            Unix32.getsize file_diskname
           with _ ->
               failwith "Zero length file ?"
         else file_size
       in
 
       if file_size <> zero && writable then (* do not truncate if not writable *)
-        Unix32.ftruncate64 t file_size;
+        begin
+	  try
+            Unix32.ftruncate64 t file_size !!create_file_sparse
+	  with e ->
+	    (try
+	      Unix32.remove t
+	     with e ->
+		lprintf_nl () "Unix32.remove %s exception %s"
+		  (file_diskname) (Printexc2.to_string e));
+	    Unix32.destroy t;
+	    failwith (Printf.sprintf "file size %s is too big, exception: %s"
+	      (size_of_int64 file_size) (Printexc2.to_string e))
+	end;
 
-      let nchunks = Int64.to_int (Int64.div
-            (Int64.sub file_size Int64.one) block_size) + 1 in
+      let nchunks = Int64.to_int (Int64.pred file_size // block_size) + 1 in
       let md4s = if file_size <= block_size then
           [md4]
         else [] in
@@ -371,7 +387,7 @@ let new_file file_diskname file_state md4 file_size filenames writable =
           impl_file_ops = file_ops;
           impl_file_age = last_time ();
           impl_file_size = file_size;
-          impl_file_fd = t;
+          impl_file_fd = Some t;
           impl_file_best_name = Filename.basename file_diskname;
           impl_file_last_seen = last_time () - 100 * 24 * 3600;
         }
@@ -442,7 +458,7 @@ let low_id ip =
 let is_black_address ip port =
   !!black_list && not (low_id ip) && (
 (* lprintf "is black ="; *)
-    not (ip_reachable ip) || (Ip.matches ip !!server_black_list) ||
+    not (Ip.reachable ip) || (Ip.matches ip !!server_black_list) ||
     (List.mem port !!port_black_list) ||
     (match Ip_set.match_ip !Ip_set.bl ip with
         None -> false
@@ -466,7 +482,7 @@ let new_server ip port score =
         server_ip = ip;
         server_cid = None (* client_ip None *);
         server_port = port;
-	server_realport = None;
+        server_realport = None;
         server_sock = NoConnection;
         server_nqueries = 0;
         server_search_queries = Fifo.create ();
@@ -476,7 +492,6 @@ let new_server ip port score =
         server_tags = [];
         server_nfiles = Int64.zero;
         server_nusers = Int64.zero;
-        server_max_users = 0;
         server_name = "";
         server_description = "";
         server_banner = "";
@@ -490,6 +505,14 @@ let new_server ip port score =
         server_id_requests = Fifo.create ();
         server_flags = 0;
         server_has_zlib = false;
+        server_version = "";
+        server_lowid_users = 0L;
+        server_soft_limit = 0L;
+        server_hard_limit = 0L;
+        server_max_users = 0L;
+        server_last_ping = 0.;
+        server_ping = 0;
+
       }
       and server_impl =
         {
@@ -518,8 +541,7 @@ let remove_server ip port =
         NoConnection -> ()
       | ConnectionWaiting token -> cancel_token token
       | Connection sock ->
-          TcpBufferedSocket.shutdown sock Closed_by_user);
-    server_remove (as_server s.server_server)
+          TcpBufferedSocket.shutdown sock Closed_by_user)
   with _ -> ()
 
 let dummy_client =
@@ -546,7 +568,7 @@ let dummy_client =
 (*      client_all_chunks = ""; *)
       client_rating = 0;
       client_brand = Brand_unknown;
-      client_mod_brand = Brand_mod_unknown;
+      client_brand_mod = Brand_mod_unknown;
       client_checked = false;
       client_connected = false;
       client_downloaded = Int64.zero;
@@ -565,6 +587,10 @@ let dummy_client =
       client_emule_proto = emule_proto ();
       client_comp = None;
       client_connection_time = 0;
+      client_req_challenge = Int64.zero;
+      client_sent_challenge = Int64.zero;
+      client_public_key = None;
+      client_sui_verified = None;
       } and
     client_impl = {
       dummy_client_impl with
@@ -602,7 +628,7 @@ let create_client key =
 (*      client_all_chunks = ""; *)
       client_rating = 0;
       client_brand = Brand_unknown;
-      client_mod_brand = Brand_mod_unknown;
+      client_brand_mod = Brand_mod_unknown;
       client_checked = false;
       client_connected = false;
       client_downloaded = Int64.zero;
@@ -621,6 +647,10 @@ let create_client key =
       client_emule_proto = emule_proto ();
       client_comp = None;
       client_connection_time = 0;
+      client_req_challenge = Int64.zero;
+      client_sent_challenge = Int64.zero;
+      client_public_key = None;
+      client_sui_verified = None;
       } and
     client_impl = {
       dummy_client_impl with
@@ -655,6 +685,18 @@ let set_client_type c t=
 let friend_add c =
   friend_add (as_client c)
 
+let string_of_client c =
+  Printf.sprintf "client[%d] %s(%s) %s" (client_num c)
+  c.client_name (brand_to_string c.client_brand)
+  (match c.client_kind with
+      Indirect_address (server_ip, server_port, ip, port) ->
+        Printf.sprintf  " I[%s:%d]" (Ip.to_string (ip_of_id ip)) port;
+    | Direct_address (ip,port) ->
+        Printf.sprintf  " D[%s:%d]" (Ip.to_string ip) port;
+    | Invalid_address _ -> ""
+  )
+
+
 let set_client_name c name md4 =
   if name <> c.client_name || c.client_md4 <> md4 then begin
       c.client_name <- name;
@@ -671,12 +713,11 @@ let find_client_by_name name =
   with ClientFound c -> c
 
 let local_mem_stats level buf =
-  Gc.compact ();
   let client_counter = ref 0 in
   let unconnected_unknown_clients = ref 0 in
   let uninteresting_clients = ref 0 in
   let aliased_clients = ref 0 in
-  let connected_clients = ref 0 in
+  let myconnected_clients = ref 0 in
   let closed_connections = ref 0 in
   let unlocated_client = ref 0 in
   let bad_numbered_clients = ref 0 in
@@ -709,7 +750,7 @@ end;
           (try
               Hashtbl.find connected_clients_by_num num
             with _ ->
-                incr connected_clients;
+                incr myconnected_clients;
                 waiting_msgs := !waiting_msgs + nmsgs;
                 buffers := !buffers + buf_len;
 (*
@@ -739,13 +780,55 @@ end;
   Printf.bprintf buf "   Read Buffers: %d\n" !buffers;
   Printf.bprintf buf "   Write Messages: %d\n" !waiting_msgs;
   Printf.bprintf buf "   Uninteresting clients: %d\n" !uninteresting_clients;
-  Printf.bprintf buf "   Connected clients: %d\n" !connected_clients;
+  Printf.bprintf buf "   Connected clients: %d\n" !myconnected_clients;
   Printf.bprintf buf "   Aliased clients: %d\n" !aliased_clients;
   Printf.bprintf buf "   Closed clients: %d\n" !closed_connections;
   Printf.bprintf buf "   Unlocated clients: %d\n" !unlocated_client;
   Printf.bprintf buf "   Bad numbered clients: %d\n" !bad_numbered_clients;
   Printf.bprintf buf "   Dead clients: %d\n" !dead_clients;
   Printf.bprintf buf "   Disconnected aliases: %d\n" !disconnected_alias;
+
+  Printf.bprintf buf "Number of old files: %d\n" (List.length !!old_files);
+  Printf.bprintf buf "Current files: %d\n" (List.length !current_files);
+
+  let counter = ref 0 in
+  UdpClientWHashtbl.iter (fun _ -> incr counter) udp_clients;
+  Printf.bprintf buf "  udp_clients: %d\n" !counter;
+
+  Printf.bprintf buf "  client_to_client_tags: %d\n" (List.length !client_to_client_tags);
+  Printf.bprintf buf "  client_to_server_tags: %d\n" (List.length !client_to_server_tags);
+  Printf.bprintf buf "  overnet_connectreply_tags: %d\n" (List.length !overnet_connectreply_tags);
+  Printf.bprintf buf "  overnet_connect_tags: %d\n" (List.length !overnet_connect_tags);
+  Printf.bprintf buf "  clients_root: %d\n" (List.length !clients_root);
+  Printf.bprintf buf "  servers_list: %d\n" (List.length !servers_list);
+  Printf.bprintf buf "  xs_servers_list: %d\n" (List.length !xs_servers_list);
+  Printf.bprintf buf "  connected_server_list: %d\n" (List.length !connected_server_list);
+  Printf.bprintf buf "  connecting_server_list: %d\n" (List.length !connecting_server_list);
+  Printf.bprintf buf "  udp_servers_list: %d\n" (List.length !udp_servers_list);
+  Printf.bprintf buf "  interesting_clients: %d\n" (List.length !interesting_clients);
+  Printf.bprintf buf "  shared_files: %d\n" (List.length !shared_files);
+  Printf.bprintf buf "  new_shared_files: %d\n" (List.length !new_shared_files);
+  Printf.bprintf buf "  servers_by_key: %d\n" (Hashtbl.length servers_by_key);
+  Printf.bprintf buf "  banned_ips: %d\n" (Hashtbl.length banned_ips);
+  Printf.bprintf buf "  old_requests: %d\n" (Hashtbl.length old_requests);
+  Printf.bprintf buf "  connected_clients: %d\n" (Hashtbl.length connected_clients);
+  Printf.bprintf buf "  files_by_md4: %d\n" (Hashtbl.length files_by_md4);
+  Printf.bprintf buf "  shared_files_info: %d\n" (Hashtbl.length shared_files_info);
+  Printf.bprintf buf "  file_groups: %d\n" (Hashtbl.length file_groups);
+  Printf.bprintf buf "  udp_servers_replies: %d\n" (Hashtbl.length udp_servers_replies);
+  Printf.bprintf buf "  join_queue_by_md4: %d\n" (Hashtbl.length join_queue_by_md4);
+  Printf.bprintf buf "  join_queue_by_id: %d\n" (Hashtbl.length join_queue_by_id);
+
+(*  let list = H.to_list clients_by_kind in *)
+  if level > 0 then begin
+    H.iter (fun c ->
+      Printf.bprintf buf "[%d ok: %s rating: %d]\n"
+        (client_num c)
+        (string_of_date (c.client_source.DonkeySources.source_age))
+(* TODO: add connection state *)
+        c.client_rating;
+    ) clients_by_kind;
+  end;
   ()
 
 let remove_client c =
@@ -809,188 +892,7 @@ let left_bytes = "MLDK"
 let overnet_server_ip = ref Ip.null
 let overnet_server_port = ref 0
 
-let brand_to_string b =
-  match b with
-    Brand_unknown -> "unknown"
-  | Brand_edonkey -> "eDonkey"
-  | Brand_cdonkey -> "cDonkey"
-  | Brand_mldonkey1 -> "old mldonkey"
-  | Brand_mldonkey2 -> "new mldonkey"
-  | Brand_mldonkey3 -> "trusted mld"
-  | Brand_overnet -> "Overnet"
-  | Brand_newemule -> "eMule"
-  | Brand_lmule -> "xMule"
-  | Brand_shareaza -> "shareaza"
-  | Brand_server -> "server"
-  | Brand_amule -> "aMule"
-  | Brand_lphant -> "lPhant"
-  | Brand_emuleplus -> "ePlus"
-  | Brand_hydranode -> "Hydra"
 
-let brand_mod_to_string b =
-  match b with
-    Brand_mod_unknown -> "unknown"
-  | Brand_mod_extasy -> "Extasy"
-  | Brand_mod_hunter -> "Hunter"
-  | Brand_mod_sivka -> "Sivka"
-  | Brand_mod_ice -> "IcE"
-  | Brand_mod_plus -> "Plus"
-  | Brand_mod_lsd -> "LSD"
-  | Brand_mod_maella -> "Maella"
-  | Brand_mod_pille -> "Pille"
-  | Brand_mod_morphkad -> "MorphKad"
-  | Brand_mod_efmod -> "eF-MOD"
-  | Brand_mod_xtreme -> "Xtreme"
-  | Brand_mod_bionic -> "Bionic"
-  | Brand_mod_pawcio -> "Pawcio"
-  | Brand_mod_zzul -> "ZZUL"
-  | Brand_mod_blackhand -> "Black Hand"
-  | Brand_mod_lovelace -> "lovelace"
-  | Brand_mod_morphnext -> "MorphNext"
-  | Brand_mod_fincan -> "fincan"
-  | Brand_mod_ewombat -> "eWombat"
-  | Brand_mod_morph -> "Morph"
-  | Brand_mod_mortillo -> "MorTillo"
-  | Brand_mod_lh -> "LionHeart"
-  | Brand_mod_emulespana -> "emulEspa\241a"
-  | Brand_mod_blackrat -> "BlackRat"
-  | Brand_mod_enkeydev -> "enkeyDev"
-  | Brand_mod_gnaddelwarz -> "Gnaddelwarz"
-  | Brand_mod_phoenixkad -> "pHoeniX-KAD"
-  | Brand_mod_koizo -> "koizo"
-  | Brand_mod_ed2kfiles -> "ed2kFiles"
-  | Brand_mod_athlazan -> "Athlazan"
-  | Brand_mod_cryptum -> "Cryptum"
-  | Brand_mod_lamerzchoice -> "LamerzChoice"
-  | Brand_mod_notdead -> "NotDead"
-  | Brand_mod_peace -> "peace"
-  | Brand_mod_goldicryptum -> "GoldiCryptum"
-  | Brand_mod_eastshare -> "EastShare"
-  | Brand_mod_mfck -> "[MFCK]"
-  | Brand_mod_echanblard -> "eChanblard"
-  | Brand_mod_sp4rk -> "Sp4rK"
-  | Brand_mod_powermule -> "PowerMule"
-  | Brand_mod_bloodymad -> "bloodymad"
-  | Brand_mod_roman2k -> "Roman2K"
-  | Brand_mod_gammaoh -> "GaMMaOH"
-  | Brand_mod_elfenwombat -> "ElfenWombat"
-  | Brand_mod_o2 -> "O2"
-  | Brand_mod_dm -> "DM"
-  | Brand_mod_sfiom -> "SF-IOM"
-  | Brand_mod_magic_elseve -> "Magic-Elseve"
-  | Brand_mod_schlumpmule -> "SchlumpMule"
-  | Brand_mod_lc -> "LC"
-  | Brand_mod_noamson -> "NoamSon"
-  | Brand_mod_stormit -> "Stormit"
-  | Brand_mod_omax -> "OMaX"
-  | Brand_mod_mison -> "Mison"
-  | Brand_mod_phoenix -> "Phoenix"
-  | Brand_mod_spiders -> "Spiders"
-  | Brand_mod_iberica -> "Ib\233rica"
-  | Brand_mod_mortimer -> "Mortimer"
-  | Brand_mod_stonehenge -> "Stonehenge"
-  | Brand_mod_xlillo -> "Xlillo"
-  | Brand_mod_imperator -> "ImperatoR"
-  | Brand_mod_raziboom -> "Raziboom"
-  | Brand_mod_khaos -> "Khaos"
-  | Brand_mod_hardmule -> "Hardmule"
-  | Brand_mod_sc -> "SC"
-  | Brand_mod_cy4n1d -> "Cy4n1d"
-  | Brand_mod_dmx -> "DMX"
-  | Brand_mod_ketamine -> "Ketamine"
-  | Brand_mod_blackmule -> "Blackmule"
-  | Brand_mod_morphxt -> "MorphXT"
-  | Brand_mod_ngdonkey -> "ngdonkey"
-  | Brand_mod_cyrex -> "Cyrex"
-  | Brand_mod_hawkstar -> "Hawkstar"
-  | Brand_mod_neomule -> "Neo Mule"
-  | Brand_mod_aldo -> "aldo"
-  | Brand_mod_emulede -> "emule.de"
-  | Brand_mod_zx -> "zx"
-  | Brand_mod_ibericaxt -> "ib\233ricaxt"
-  | Brand_mod_candymule -> "candy-mule"
-  | Brand_mod_ackronic -> "ackronic"
-  | Brand_mod_rappis -> "rappis"
-  | Brand_mod_overdose -> "overdose"
-  | Brand_mod_hebmule -> "hebmule"
-  | Brand_mod_senfei -> "senfei"
-  | Brand_mod_spoofmod -> "spoofmod"
-  | Brand_mod_fusspilz -> "fusspilz"
-  | Brand_mod_rocket -> "rocket"
-  | Brand_mod_warezfaw -> "warezfaw"
-  | Brand_mod_emusicmule -> "emusicmule"
-  | Brand_mod_aideadsl -> "aideadsl"
-  | Brand_mod_epo -> "epo"
-  | Brand_mod_kalitsch -> "kalitsch"
-  | Brand_mod_raynz -> "raynz"
-  | Brand_mod_serverclient -> "serverclient"
-  | Brand_mod_bl4ckbird -> "bl4ckbird"
-  | Brand_mod_bl4ckf0x -> "bl4ckf0x"
-  | Brand_mod_rt -> "rt"
-  | Brand_mod_airionix -> "air-ionix"
-  | Brand_mod_ionix -> "ionix"
-  | Brand_mod_tornado -> "tornado"
-  | Brand_mod_antifaker -> "anti-faker"
-  | Brand_mod_netf -> "netf"
-  | Brand_mod_nextemf -> "nextemf"
-  | Brand_mod_proemule -> "proemule"
-  | Brand_mod_szemule -> "szemule"
-  | Brand_mod_darkmule -> "darkmule"
-  | Brand_mod_miragemod -> "miragemod"
-  | Brand_mod_nextevolution -> "nextevolution"
-  | Brand_mod_pootzgrila -> "pootzgrila"
-  | Brand_mod_freeangel -> "freeangel"
-  | Brand_mod_enos -> "enos"
-  | Brand_mod_webys -> "webys"
-
-let string_of_client c =
-  Printf.sprintf "client[%d] %s(%s) %s" (client_num c)
-  c.client_name (brand_to_string c.client_brand)
-  (match c.client_kind with
-      Indirect_address _ | Invalid_address _ -> ""
-    | Direct_address (ip,port) ->
-        Printf.sprintf  " [%s:%d]" (Ip.to_string ip) port;
-  )
-
-
-
-let check_result r tags =
-  if r.result_names = [] || r.result_size = Int64.zero then begin
-      if !verbose then begin
-          lprintf_n () "BAD RESULT:";
-          List.iter (fun tag ->
-              lprintf "[%s] = [%s]" (string_of_field tag.tag_name)
-                (string_of_tag_value tag.tag_value);
-              lprint_newline ();
-          ) tags;
-        end;
-      false
-    end
-  else true
-
-let result_of_file md4 tags =
-  let rec r = {  dummy_result with
-      result_uids = [Uid.create (Ed2k md4)];
-    } in
-  List.iter (fun tag ->
-      match tag with
-        { tag_name = Field_Filename; tag_value = String s } ->
-          r.result_names <- s :: r.result_names
-      | { tag_name = Field_Size; tag_value = Uint64 v } ->
-          r.result_size <- v;
-      | { tag_name = Field_Format; tag_value = String s } ->
-          r.result_tags <- tag :: r.result_tags;
-          r.result_format <- s
-      | { tag_name = Field_Type; tag_value = String s } ->
-          r.result_tags <- tag :: r.result_tags;
-          r.result_type <- s
-      | _ ->
-          r.result_tags <- tag :: r.result_tags
-  ) tags;
-  if check_result r tags then
-    let rs = update_result_num r in
-    Some rs
-  else None
 
 (*************************************************************
 
@@ -1000,21 +902,6 @@ Define a function to be called when the "mem_stats" command
 **************************************************************)
 
 let _ =
-  Heap.add_memstat "DonkeyGlobals" (fun level buf ->
-      Printf.bprintf buf "Current files: %d\n" (List.length !current_files);
-      let list = H.to_list clients_by_kind in
-      Printf.bprintf buf  "Clients_by_kind: %d\n" (List.length list);
-      if level > 0 then
-        List.iter (fun c ->
-            Printf.bprintf buf "[%d ok: %s rating: %d]\n"
-              (client_num c)
-            (string_of_date (c.client_source.DonkeySources.source_age))
-(* TODO: add connection state *)
-            c.client_rating
-            ;
-        ) list;
-  );
-
   Heap.add_memstat "DonkeyGlobals" local_mem_stats
 
 (*************************************************************
@@ -1029,13 +916,14 @@ let _ =
 let client_id c =
   match c.client_kind with
     Direct_address (ip, port) -> (ip, port, zero)
-  | Indirect_address (ip, port, id) -> (ip, port, id)
+  | Indirect_address (server_ip, server_port, id, port) ->
+     (server_ip, server_port, id)
   | Invalid_address _ -> (Ip.null, 0, zero)
 
 let save_join_queue c =
   if c.client_file_queue <> [] then
     let files = List.map (fun (file, chunks, _) ->
-          file, Array.copy chunks
+          file, Bitv.copy chunks
       ) c.client_file_queue in
     begin
       if c.client_debug then
@@ -1066,9 +954,51 @@ let clean_join_queue_tables () =
         Hashtbl.add join_queue_by_id key e
   ) list
 
+let client_public_key = ref ""
+
+let _ =
+  option_hook client_private_key (fun _ ->
+    client_public_key := Unix32.load_key (!!client_private_key);
+
+    (
+    let key_checked = ref false in
+    let key_check_again = ref false in
+    let rec check_client_private_key () =
+      key_checked := true;
+      if not (String.sub !!client_private_key 0 5 = "MIIBC") then
+        if !key_check_again then
+	  begin
+	    lprintf_nl () "can not create proper client_private_key, exiting...";
+	    exit 70
+	  end
+	else
+          begin
+            lprintf_nl () "bad client_private_key detected, creating new key";
+	    set_simple_option donkey_ini "client_private_key" (Unix32.create_key ());
+	    key_check_again := true
+	  end
+    in
+    if not !key_checked then check_client_private_key ();
+    if !key_check_again then
+      begin
+	lprintf_nl () "re-checking private key";
+	check_client_private_key ()
+      end
+    );
+  )
+
 let server_accept_multiple_getsources s =
   (s.server_flags land DonkeyProtoUdp.PingServerReplyUdp.multiple_getsources) <> 0
 
 let server_send_multiple_replies s =
   (s.server_flags land DonkeyProtoUdp.PingServerReplyUdp.multiple_replies) <> 0
 
+let server_send_getsources2 s =
+  (s.server_flags land DonkeyProtoUdp.PingServerReplyUdp.getsources2) <> 0
+
+let full_client_identifier c =
+    Printf.sprintf "%s (%s%s) '%s'"
+      (Ip.to_string c.client_ip)
+      (brand_to_string_short c.client_brand)
+      (if c.client_emule_proto.emule_release = "" then "" else " " ^ c.client_emule_proto.emule_release)
+      (String.escaped c.client_name)
