@@ -70,7 +70,6 @@ let quarter_timer timer =
 
 let fivemin_timer timer =
   DonkeyShare.send_new_shared ();
-  DonkeyChunks.duplicate_chunks ();
   clients_root := []
 
 let second_timer timer =
@@ -82,9 +81,14 @@ let second_timer timer =
             (Printexc2.to_string e)
   )
 
-let halfmin_timer timer =
+let min_timer timer =
   DonkeySources.clean_sources (); (* Moved here from fivemin_timer. *)
-  DonkeyServers.update_master_servers ()
+  DonkeyServers.update_master_servers ();
+ (try
+      DonkeyServers.query_locations_timer ();
+    with _ -> ());
+  (List.iter (fun file -> DonkeyShare.must_share_file file) !new_shared_files;
+  new_shared_files := [])
 (*  DonkeyIndexer.add_to_local_index_timer () *)
 
 
@@ -98,7 +102,7 @@ let disable enabler () =
       is_enabled := false;
       enabler := false;
       if !!enable_donkey then enable_donkey =:= false;
-      Hashtbl2.safe_iter (fun s -> disconnect_server s Closed_by_user)
+      Hashtbl2.safe_iter (fun s -> DonkeyServers.disconnect_server s Closed_by_user)
       servers_by_key;
       H.iter (fun c -> DonkeyClient.disconnect_client c Closed_by_user) 
       clients_by_kind;
@@ -123,27 +127,34 @@ let disable enabler () =
 let reset_tags () =
   let module D = DonkeyProtoClient in
   let m = D.mldonkey_emule_proto in
+
+  let secident = if !!enable_sui then 3 else 0 in
+    m.emule_secident <- secident;
+    m.emule_features <- secident;
+
   let emule_miscoptions1 = D.emule_miscoptions1 m in
   client_to_client_tags :=
   [
     string_tag (Field_UNKNOWN "name") (local_login ());
+    int_tag (Field_UNKNOWN "port") !!donkey_port;
     int_tag (Field_UNKNOWN "version") protocol_version;
     int_tag (Field_UNKNOWN "emule_udpports") (!!donkey_port+4);
     int_tag (Field_UNKNOWN "emule_version") m.emule_version;
     int64_tag (Field_UNKNOWN "emule_miscoptions1") emule_miscoptions1;
-    int_tag (Field_UNKNOWN "port") !!donkey_port;
   ];
+
+  let extended = ref 0x04 in (* support of auxport *)
+    extended := !extended lor 0x01; (* support of compression *)
+
   client_to_server_tags :=
   [
     string_tag (Field_UNKNOWN "name") (local_login ());
     int_tag (Field_UNKNOWN "version") protocol_version;
     int_tag (Field_UNKNOWN "port") !!donkey_port;
+    int_tag (Field_UNKNOWN "extended") !extended;
+    int_tag (Field_UNKNOWN "emule_version") m.emule_version;
   ];
-  let extended = ref 0x04 in (* support of auxport *)
-  if Autoconf.has_zlib then
-    extended := !extended lor 0x01; (* support of compression *)
-    client_to_server_tags := (int_tag
-      (Field_UNKNOWN "extended") !extended)::!client_to_server_tags;
+
   emule_info.DonkeyProtoClient.EmuleClientInfo.tags <- [
     int_tag (Field_UNKNOWN "compression") 
       (if !!emule_compression then m.emule_compression else 0);
@@ -151,7 +162,7 @@ let reset_tags () =
     int_tag (Field_UNKNOWN "udpport") (!!donkey_port+4);
     int_tag (Field_UNKNOWN "sourceexchange") m.emule_sourceexchange;
     int_tag (Field_UNKNOWN "comments") m.emule_comments;
-    int_tag (Field_UNKNOWN "compatibleclient") 10; 
+    int_tag (Field_UNKNOWN "compatibleclient") !DonkeyProtoClient.compatibleclient; 
     int_tag (Field_UNKNOWN "extendedrequest") m.emule_extendedrequest;
     int_tag (Field_UNKNOWN "features") m.emule_features;
     
@@ -172,7 +183,9 @@ let enable () =
     let enabler = ref true in
     is_enabled := true;
     network.op_network_disable <- disable enabler;
-    
+    (try
+      client_public_key := Unix32.load_key (!!client_private_key)
+    with _ -> ());
     if not !!enable_donkey then enable_donkey =:= true;
     
     try
@@ -205,7 +218,7 @@ let enable () =
                 try
                   let file_disk_name = file_disk_name file in
                   if Unix32.file_exists file_disk_name &&
-                    Unix32.getsize file_disk_name false <> Int64.zero then begin
+                    Unix32.getsize file_disk_name <> Int64.zero then begin
                     (* getsize writable=false is ok because file has state FileDownloaded *)
                       lprintf_nl () "FILE DOWNLOADED"; 
                       
@@ -220,7 +233,6 @@ let enable () =
                 (Printexc2.to_string e) (file_disk_name file); 
       ) files_by_md4;
       
-      let list = ref [] in
 (* Normally, we should check that downloaded files are still there.  *)    
       let list = ref [] in
       List.iter (fun file ->
@@ -278,12 +290,11 @@ be useful when users want to share files that they had already previously
         | _ -> failwith "Bad socket address"
       end;
       
-      let port = !!donkey_port in
-      
       reset_tags ();
       
       Options.option_hook global_login reset_tags;
       Options.option_hook login reset_tags;
+      Options.option_hook enable_sui reset_tags;
 
 (**** START TIMERS ****)
       add_session_option_timer enabler check_client_connections_delay 
@@ -306,18 +317,10 @@ be useful when users want to share files that they had already previously
       add_session_timer enabler 1.0 DonkeyServers.udp_walker_timer;
       
       add_session_timer enabler 3600. hourly_timer;
-      add_session_timer enabler 30. halfmin_timer;
+      add_session_timer enabler 60. min_timer;
       add_session_timer enabler 300. fivemin_timer;
       add_session_timer enabler 900. quarter_timer;
       add_session_timer enabler 1. second_timer;
-      add_session_timer enabler 60. (fun _ ->
-          (try
-              DonkeyServers.query_locations_timer ();
-            with _ -> ());
-          List.iter (fun file -> DonkeyShare.must_share_file file) 
-          !new_shared_files;  
-          new_shared_files := [];
-      );
       add_session_option_timer enabler remove_old_servers_delay 
           DonkeyServers.remove_old_servers;
 
@@ -334,6 +337,22 @@ be useful when users want to share files that they had already previously
       lprintf_nl () "Error: Exception %s during startup"
         (Printexc2.to_string e)
 
+
+let rec update_options () =
+  let update v =
+      lprintf_nl () "Updating options to version %i" v;
+      options_version =:= v;
+      update_options ()
+  in
+
+  match !!options_version with
+    0 ->
+      if !!max_sources_per_file = 20000 then
+        max_sources_per_file =:= 5000;
+      update 1
+  | _ -> ()
+
+
 let _ =
 
 (* TODO INDEX  DonkeyIndexer.init (); *)
@@ -346,6 +365,7 @@ let _ =
 );
   *)
   network.op_network_enable <- enable;
+  network.op_network_update_options <- update_options;
 (*  network.network_config_file <- []; *)
   network.op_network_info <- (fun n ->
       { 
