@@ -123,11 +123,12 @@ let rec compute_stats () =
                 lprintf_nl () "WARNING, source was not tested";
               raise Continue
             end
+      (* expected not used anymore
       | File_expected  ->
           if time + 1800 < last_time () then begin
               incr_propositions kind File_not_found;
               raise Continue
-            end
+            end *)
       | File_possible ->
           if !verbose then
             lprintf_nl () "WARNING, source was unknown";
@@ -140,7 +141,6 @@ let rec compute_stats () =
       | File_upload ->
           incr_propositions kind File_found;
           raise Continue
-      | File_unknown -> assert false
     with
     | Not_found ->
 (* For some reason, the request was forgotten. Forget it... *)
@@ -165,7 +165,8 @@ let propose_source file c kind =
         Fifo.put propositions (s, file, kind, last_time ())
     | _ ->
         if !verbose then
-          lprintf_nl () "ERROR, proposed client is indirect";
+          lprintf_nl () "Neighbours: error: proposed client (%s) is indirect for file %s"
+	    (full_client_identifier c) (file_best_name file);
         raise Exit
 
   with Exit ->
@@ -383,18 +384,13 @@ let _ =
       compute_stats ();
       let len = Array.length ngood_propositions in
       buf_int buf len;
-      if !verbose_redirector then lprintf_n () "Stats:";
       for i = 0 to len - 1 do
-        if !verbose_redirector then lprintf_nl2 "   Good[%d] = %Ld\n" i ngood_propositions.(i);
-        if !verbose_redirector then lprintf_nl2 "   Bad[%d] = %Ld\n" i nbad_propositions.(i);
         buf_int64 buf ngood_propositions.(i);
         buf_int64 buf nbad_propositions.(i)
       done;
       let len = Fifo.length propositions in
-      if !verbose_redirector then lprintf_nl2 "   Waiting propositions: %d\n" len;
       buf_int buf len;
       let len = ref 0 in
       Hashtbl.iter (fun _ _ -> incr len) neighbours;
-      if !verbose_redirector then lprintf_nl2 "   Total neighbours: %d\n" !len;
       buf_int buf !len
   )
