@@ -45,7 +45,6 @@ open DonkeyTypes
 open DonkeyOptions
 open DonkeyComplexOptions
 open DonkeyGlobals
-open DonkeyChunks
 open DonkeyReliability
 open DonkeyStats
     
@@ -79,10 +78,10 @@ let sort_file_queue c =
               let v = file_priority f2 - file_priority f1 in
               if v <> 0 then v else
               let s1 = if (file_size f1) > Int64.zero then
-                  Int64.to_int (Int64.div (Int64.mul (file_downloaded f1) (Int64.of_int 100)) (file_size f1))
+                  Int64.to_int ((file_downloaded f1) ** 100L // (file_size f1))
                 else 0 in
               let s2 = if (file_size f2) > Int64.zero then
-                  Int64.to_int (Int64.div (Int64.mul (file_downloaded f2) (Int64.of_int 100)) (file_size f2))
+                  Int64.to_int ((file_downloaded f2) ** 100L // (file_size f2))
                 else 0 in
               s2 - s1
           ) c.client_file_queue;
@@ -135,6 +134,7 @@ let unshare_file file =
 let declare_completed_file file = 
   DonkeyShare.remember_shared_info file (file_disk_name file);
   file_completed (as_file file);
+  Int64Swarmer.remove_swarmer file.file_swarmer;
   file.file_swarmer <- None;
   
   
@@ -220,13 +220,13 @@ let add_client_chunks c file client_chunks =
             if f != file then iter tail
             else begin
                 Int64Swarmer.update_uploader up
-                (AvailableBoolBitmap client_chunks);
-                Array.blit client_chunks 0 chunks 0 (Array.length chunks)
+                (AvailableBitv client_chunks);
+                Bitv.blit client_chunks 0 chunks 0 (Bitv.length chunks)
               end
             
         | [] ->
             let up = Int64Swarmer.register_uploader swarmer (as_client c) 
-              (AvailableBoolBitmap client_chunks) in
+              (AvailableBitv client_chunks) in
             c.client_file_queue <-  c.client_file_queue @
               [file, client_chunks, up]
       in
@@ -326,7 +326,7 @@ let rec get_from_client c =
   | Some (file,up) ->
       
       try
-        let b = Int64Swarmer.current_block up in
+        let _ = Int64Swarmer.current_block up in
         let ranges = Int64Swarmer.current_ranges up in
         let before_find_range = List.length ranges in
 
@@ -365,7 +365,7 @@ let rec get_from_client c =
           try
             let swarmer = Int64Swarmer.uploader_swarmer up in       
             (try Int64Swarmer.verify_one_chunk swarmer with _ -> ());
-            let b = Int64Swarmer.find_block up in
+            let _ = Int64Swarmer.find_block up in
             get_from_client c
             
           with Not_found ->
@@ -415,13 +415,17 @@ let block_received c md4 begin_pos bloc bloc_pos bloc_len =
     
   match c.client_download with
     None -> 
-      if !verbose_hidden_errors then
-        lprintf_nl ()"block_received: block received but no file !, received: %s" (Md4.to_string md4)
+      if !verbose then
+        lprintf_nl ()"block_received for unknown file (md4 %s) for data from %s"
+	  (Md4.to_string md4) (full_client_identifier c)
   | Some (file, up) ->
       
       if file.file_md4 <> md4 then begin
-          if !verbose_hidden_errors then
-            lprintf_nl () "block_received: block for bad file, received: %s, expected: %s" (Md4.to_string md4) (Md4.to_string file.file_md4)
+          if !verbose then
+            lprintf_nl () "block_received for wrong file, received: %s, expected: %s, from %s"
+	      (Md4.to_string md4)
+	      (Md4.to_string file.file_md4)
+	      (full_client_identifier c)
         end else begin
           DonkeySources.set_request_result c.client_source file.file_sources File_upload;
           
@@ -429,8 +433,6 @@ let block_received c md4 begin_pos bloc bloc_pos bloc_len =
           
           set_client_state c (Connected_downloading (file_num file));
           let len64 = Int64.of_int bloc_len in
-          count_download c file len64;
-          network_must_update network;
 
 (* TODO: verify the received data has been requested *)
           
@@ -443,7 +445,7 @@ let block_received c md4 begin_pos bloc bloc_pos bloc_len =
             with
             | e ->
                 let m =
-(*		      Printf.sprintf "File %s begin_pos=%s bloc_begin=%d bloc_len=%d:\nError %s while writing block%s\n" (file_best_name file) (Int64.to_string begin_pos) t.Q.bloc_begin t.Q.bloc_len (Printexc2.to_string e)  *)
+(*          Printf.sprintf "File %s begin_pos=%s bloc_begin=%d bloc_len=%d:\nError %s while writing block%s\n" (file_best_name file) (Int64.to_string begin_pos) t.Q.bloc_begin t.Q.bloc_len (Printexc2.to_string e)  *)
                   (match e with 
                       Unix.Unix_error (Unix.ENOSPC, _, _) -> " (Disk full?)"
                     | _ -> "") in
@@ -462,8 +464,7 @@ let block_received c md4 begin_pos bloc bloc_pos bloc_len =
 (*            List.iter Int64Swarmer.alloc_range c.client_ranges; *)
           let new_downloaded = 
             Int64Swarmer.downloaded swarmer in
-          c.client_downloaded <- c.client_downloaded ++ (
-            new_downloaded -- old_downloaded);
+          count_download c (new_downloaded -- old_downloaded);
 (*
           if not (List.mem c.client_ip bb.block_contributors) then
             bb.block_contributors <- c.client_ip :: 
@@ -489,7 +490,7 @@ let search_found filter search md4 tags =
         { tag_name = Field_Filename; tag_value = String s } -> file_name := s
       | { tag_name = Field_Size; tag_value = Uint64 v } -> file_size := v
       | { tag_name = Field_Availability;
-	  tag_value = (Uint64 v| Fint64 v) } ->
+    tag_value = (Uint64 v| Fint64 v) } ->
           availability := Int64.to_int v;  new_tags := tag :: !new_tags
       | _ -> new_tags := tag :: !new_tags
   ) tags;
