@@ -42,13 +42,20 @@ open DonkeyTypes
 open DonkeyOptions
 open DonkeyComplexOptions
 open DonkeyGlobals
+open DonkeyUdp
 
 
 module Udp = DonkeyProtoUdp
 
+(* prints a new logline with date, module and starts newline *)
 let lprintf_nl () =
   lprintf "%s[EDK] "
-  (log_time ()); lprintf_nl2
+    (log_time ()); lprintf_nl2
+
+(* prints a new logline with date, module and does not start newline *)
+let lprintf_n () =
+  lprintf "%s[EDK] "
+    (log_time ()); lprintf
 
 (* Constants *)
 let udp_requests_new  = 5
@@ -126,10 +133,14 @@ let query_locations_timer () =
               List.iter (
                 fun file ->
                   if !verbose_location then
-                    lprintf_nl () "TCP: Query Location of %s"
-                      (file_best_name file);
+                    lprintf_nl () "TCP: Query Location of %s [%s] [%Ld] [%s]"
+                      (file_best_name file) (Md4.to_string file.file_md4) (file_size file) (Ip.to_string (peer_ip sock));
                   let module M = DonkeyProtoServer in
-                  server_send sock ( M.QueryLocationReq file.file_md4  )
+                  let module E = M.QueryLocation in
+                  server_send sock ( M.QueryLocationReq {
+                     E.md4 = file.file_md4;
+                    E.size = file_size file;
+                  });
               ) (get_query_files server files_queries_per_minute);
           )
         end
@@ -184,12 +195,12 @@ let udp_query_sources () =
     (* query "new servers" ie servers which understand multiple filerequests *)
     List.iter (
       fun s ->
-        let md4s = List.map (
+        let l = List.map (
           fun file ->
-            file.file_md4;
+            (file.file_md4,(file_size file));
         ) (get_query_files s udp_requests_new)
         in
-        udp_server_send s (Udp.QueryLocationUdpReq md4s);
+        udp_server_send_query_location s l;
         s.server_next_udp <- last_time () + udp_requests_wait;
     ) !new_servers;
 
@@ -218,10 +229,6 @@ let disconnect_server s reason =
     | Connection sock ->
         decr nservers;
         TcpBufferedSocket.close sock reason;
-        (*
-          lprintf "%s:%d CLOSED received by server\n"
-          (Ip.to_string s.server_ip) s.server_port;
-        *)
         connection_failed (s.server_connection_control);
         s.server_sock <- NoConnection;
         s.server_score <- s.server_score - 1;
@@ -234,6 +241,7 @@ let disconnect_server s reason =
           | _ -> ());
         s.server_banner <- "";
         s.server_sent_all_queries <- false;
+        remove_connecting_server s;
         remove_connected_server s
 
 
@@ -265,10 +273,11 @@ let client_to_server s t sock =
           disconnect_server s (Closed_for_error "Low ID")
         else begin
           s.server_cid <- Some t.M.SetID.ip;
-	  s.server_realport <- t.M.SetID.port;
+          s.server_realport <- t.M.SetID.port;
           (* disconnect after (connected_server_timeout) seconds of silence *)
           set_rtimeout sock !!connected_server_timeout;
           set_server_state s Connected_initiating;
+          remove_connecting_server s;
           s.server_score <- s.server_score + 5;
           connection_ok (s.server_connection_control);
 
@@ -282,23 +291,42 @@ let client_to_server s t sock =
         end
 
   | M.MessageReq msg ->
+
+      (* This is stupid *)
+      if s.server_version = "" then begin
+        try
+          let prefix = String.sub msg 0 15 in
+          if prefix = "server version " then 
+            s.server_version <- String.sub msg 15 4
+        with _ -> ()
+      end;
+
       if !last_message_sender <> server_num s then begin
-          let server_header = Printf.sprintf "\n+-- From server %s [%s:%d] ------\n"
+          let server_header = Printf.sprintf "\n+-- From server %s [%s:%d] ------"
               s.server_name (Ip.to_string s.server_ip) s.server_port in
-          CommonEvent.add_event (Console_message_event server_header);
+          CommonEvent.add_event (Console_message_event (Printf.sprintf "%s\n" server_header));
+          if !CommonOptions.verbose_msg_servers then
+      lprintf_nl () "%s" server_header;
           last_message_sender := server_num s
         end;
       s.server_banner <- s.server_banner ^ Printf.sprintf "%s\n" msg;
-      let msg = Printf.sprintf "| %s\n" msg in
-      CommonEvent.add_event (Console_message_event msg)
+      let msg = Printf.sprintf "| %s" msg in
+      CommonEvent.add_event (Console_message_event (Printf.sprintf "%s\n" msg));
+      if !CommonOptions.verbose_msg_servers then
+        lprintf_nl () "%s" msg
 
   | M.ServerListReq l ->
-      if !verbose then lprintf_nl () "donkeyServers: Received serverlist";
-      if !!update_server_list_server then
+      if !!update_server_list_server then begin
+        if !verbose_msg_servers then lprintf_nl () "Received serverlist from server";
         let module Q = M.ServerList in
         List.iter (fun s ->
             safe_add_server s.Q.ip s.Q.port
         ) l
+        end
+      else begin
+        if !verbose_msg_servers then lprintf_nl () "Ignored serverlist from server"
+        end
+
 
   | M.ServerInfoReq t ->
       s.server_score <- s.server_score + 1;
@@ -312,14 +340,11 @@ let client_to_server s t sock =
                 s.server_description <- desc
             | _ -> ()
       ) s.server_tags;
-      printf_char 'S';
 
       (* nice and ugly, but it doesn't require any new fields *)
       set_server_state s (Connected
           ( match s.server_cid with
-              Some t -> (match Ip.to_ints t with
-                          | _, _, _, 0 -> (-1)
-                          | _ -> (-2))
+              Some t -> if low_id t then (-1) else (-2) 
               | _ -> (-1)
           )
         );
@@ -348,7 +373,7 @@ let client_to_server s t sock =
          connect. *)
       if !verbose then lprintf_nl () "QueryIDReplyReq: received";
       let module Q = M.QueryIDReply in
-      if Ip.valid t.Q.ip && ip_reachable t.Q.ip then begin
+      if Ip.usable t.Q.ip then begin
         try
           match Fifo.take s.server_id_requests with
               None -> raise Not_found
@@ -449,9 +474,8 @@ and remove clients whose server is deconnected. *)
       ()
 
 let connect_server s =
-  if !!enable_servers && can_open_connection connection_manager
+  if !!enable_servers && not (Ip_set.ip_blocked s.server_ip)
     && (not !!connect_only_preferred_server || s.server_preferred)
-    && not (Ip_set.ip_blocked s.server_ip)
   then
     match s.server_sock with
     | NoConnection ->
@@ -462,7 +486,6 @@ let connect_server s =
               try
                 (* lprintf "CONNECTING ONE SERVER\n"; *)
                 connection_try s.server_connection_control;
-                printf_char 's';
                 let sock = TcpBufferedSocket.connect token "donkey to server"
                     (Ip.to_inet_addr s.server_ip) s.server_port
                     (server_handler s) (* DonkeyProtoCom.server_msg_to_string*)  in
@@ -490,13 +513,9 @@ let connect_server s =
                     C.tags = !client_to_server_tags;
                   }
                 );
+                add_connecting_server s;
                 add_connected_server s;
               with e ->
-                (*
-                  lprintf "%s:%d IMMEDIAT DISCONNECT \n"
-                  (Ip.to_string s.server_ip) s.server_port;
-                  lprintf "DISCONNECTED IMMEDIATLY\n";
-                 *)
                 disconnect_server s (Closed_for_exception e)
           )
           in
@@ -512,7 +531,7 @@ let rec connect_one_server restart =
   (*  lprintf "connect_one_server\n";  *)
   if can_open_connection connection_manager then
     match !servers_list with
-	[] ->
+  [] ->
           if restart then begin
             servers_list := [];
             Hashtbl.iter (fun _ s ->
@@ -521,9 +540,11 @@ let rec connect_one_server restart =
             if !servers_list = [] then begin
               if !print_empty_list then begin
                 print_empty_list := false;
-                lprintf_nl () "Looks like you have no servers in your servers.ini";
-                lprintf_nl () "You should either use the one provided with mldonkey";
-                lprintf_nl () "or import one from the WEB";
+                lprintf_nl () "There are no ED2K-servers in your servers.ini.";
+                lprintf_nl () "Please import servers from a server.met file.";
+                lprintf_nl () "Let MLDonkey use a file configured in web_infos";
+                lprintf_nl () "or enter this link into MLDonkey:";
+    lprintf_nl () "ed2k://|serverlist|http://www.gruk.org/server.met.gz|/"
               end;
 
               raise Not_found;
@@ -576,8 +597,11 @@ let rec check_server_connections () =
  * clean serverlist
  *)
 let remove_old_servers () =
-  if !verbose then lprintf_nl () "REMOVE OLD SERVERS";
-  let t1 = Unix.gettimeofday () in
+  if last_time () - start_time > !!max_server_age * Date.day_in_secs then begin
+  
+  if !verbose then
+    lprintf_nl () "old servers: Start check, remove servers not connected for %d days"
+      !!max_server_age;
 (*
 The new tactic: we sort the servers (the more recently connected first,
 the black-listed ones at the end), and we start checking from the last
@@ -587,57 +611,50 @@ position to the min_left_servers position.
   let to_remove = ref [] in
   let to_keep = ref [] in
   Hashtbl.iter (fun _ s ->
-      if not s.server_preferred &&
-        (is_black_address s.server_ip s.server_port || s.server_port = 4662)
-        then
-          to_remove := s :: !to_remove
-      else
-        to_keep :=
-          (connection_last_conn s.server_connection_control, s) :: !to_keep
+      if connection_was_tried s.server_connection_control then
+        if not s.server_preferred &&
+          (is_black_address s.server_ip s.server_port || s.server_port = 4662)
+          then
+            to_remove := s :: !to_remove
+          else
+            to_keep :=
+              (connection_last_conn s.server_connection_control, s) :: !to_keep
   ) servers_by_key;
-  let t2 = Unix.gettimeofday () in
-  if !verbose then lprintf_nl () "Delay to detect black-listed servers: %2.2f" (t2 -. t1);
 
   if List.length !to_keep > !!min_left_servers then begin
-  let array = Array.of_list !to_keep in
-  Array.sort (fun (ls1,_) (ls2,_) ->
-      if ls1 = ls2 then 0 else if ls1 > ls2 then -1 else 1
-  ) array;
+    let array = Array.of_list !to_keep in
+    Array.sort (fun (ls1,_) (ls2,_) ->
+      compare ls2 ls1
+    ) array;
 
-  if !verbose then
-    for i = 0 to Array.length array - 1 do
+    if !verbose then
+      Array.iteri (fun i (ls, s) ->
+        lprintf_nl () "old servers: server %d last_conn %d" (server_num s) ls) array;
+
+    let min_last_conn =  last_time () - !!max_server_age * Date.day_in_secs in
+
+    for i = Array.length array - 1 downto !!min_left_servers do
       let ls, s = array.(i) in
-      lprintf_nl () "server %d last_conn %d" (server_num s) ls;
-
-    done;
-
-  let min_last_conn =  last_time () - !!max_server_age * Date.day_in_secs in
-
-  for i = Array.length array - 1 downto !!min_left_servers do
-    let ls, s = array.(i) in
         if ls < min_last_conn && s.server_sock = NoConnection
           && not s.server_preferred then begin
-        if !verbose then begin
-            lprintf_nl () "Server too old: %s:%d"
+          if !verbose then
+            lprintf_nl () "old servers: Server too old: %s:%d"
               (Ip.to_string s.server_ip) s.server_port;
 
-          end;
-        to_remove := s :: !to_remove
-      end
-  done;
+          to_remove := s :: !to_remove
+        end
+    done
   end;
-  let t3 = Unix.gettimeofday () in
-  if !verbose then lprintf_nl () "Delay to detect old servers: %2.2f" (t3 -. t2);
 
   List.iter (fun s ->
-    remove_server s.server_ip s.server_port
+    DonkeyGlobals.remove_server s.server_ip s.server_port
   ) !to_remove;
 
-  let t4 = Unix.gettimeofday () in
-  if !verbose then lprintf_nl () "Delay to finally remove servers: %2.2f" (t4 -. t3);
-  if (List.length !to_remove) > 0 || !verbose then
+  if !to_remove <> [] || !verbose then
     lprintf_nl () "Removed %d old edonkey servers." (List.length !to_remove)
 
+  end
+
 
 (* Keep connecting to servers in the background. Don't stay connected to
   them , and don't send your shared files list *)
@@ -701,7 +718,6 @@ Each client issues 1 packet/4hour, so 100000 clients means 25000/hour,
 7 packets/second = 7 * 40 bytes = 280 B/s ...
 *)
 
-let udp_ping = String.create 6
 
 let udp_walker_timer () =
   match !udp_walker_list with
@@ -714,8 +730,19 @@ let udp_walker_timer () =
         end
   | s :: tail ->
       udp_walker_list := tail;
-      UdpSocket.write (get_udp_sock ())
-                  true udp_ping s.server_ip (s.server_port + 4)
+      let c = Int64.of_int (Random.int 65535) in
+      let challenge = 1437204480L ++ c in (* 0x55AA0000 *)
+      s.server_last_ping <- Unix.gettimeofday ();
+      udp_server_send_ping s (DonkeyProtoUdp.PingServerUdpReq challenge);
+
+      let module M = DonkeyProtoUdp in
+      let module E = M.ServerDescUdp in
+      udp_server_send s (M.ServerDescUdpReq {
+        E.ip = Ip.null;
+      })
+
+(*      UdpSocket.write (get_udp_sock ())
+                  true udp_ping s.server_ip (s.server_port + 4) *)
 
 (* sort the servers by preferred first
    then users count with decreasing order
@@ -728,29 +755,34 @@ let compare_servers s2 s1 =
 
 (* check connected servers *)
 let update_master_servers _ =
+  if !verbose_location then
+    lprintf_nl () "master servers: start re-computing";
   let server_list = List.sort compare_servers (connected_servers ()) in
   let masters = ref [] in
+  let tag1 = ref true in
   List.iter (
+    let tag2 = ref true in
     fun s ->
       if s.server_master then
         match s.server_sock with
         | Connection _ ->
-            if !verbose then begin
-                lprintf_nl () "MASTER: OLD MASTER %s" (Ip.to_string s.server_ip);
-              end;
+            if !verbose_location then begin
+        if !tag2 then begin
+                lprintf_n () "master servers (old):";
+    tag1 := false;
+    tag2 := false
+        end;
+        lprintf " %s" (Ip.to_string s.server_ip)
+      end;
             masters := s :: !masters
         | _ -> s.server_master <- false
   ) server_list;
+  if not !tag1 then lprint_newline ();
   let nmasters = ref (List.length !masters) in
 
-  if !verbose then
-    lprintf_nl () "MASTER: nmaster %d" !nmasters;
-
   let make_master s =
     (* normal servers don't have our SHARE, so send list if it becomes a master *)
     do_if_connected s.server_sock (fun sock ->
-        if !verbose then
-          lprintf_nl () "   MASTER: %s" (Ip.to_string s.server_ip);
         s.server_master <- true;
         incr nmasters;
 
@@ -771,8 +803,8 @@ let update_master_servers _ =
        connections *)
     if !nconnected_servers > max_allowed_connected_servers then
       begin
-        if !verbose then
-            lprintf_nl () "MASTER: DISCONNECT %s" (Ip.to_string s.server_ip);
+        if !verbose_location then
+            lprintf_nl () "master servers: disconnect %s" (Ip.to_string s.server_ip);
         nconnected_servers := !nconnected_servers - 3;
         do_if_connected  s.server_sock (fun sock ->
             (* We will disconnect from this server.
@@ -790,8 +822,9 @@ let update_master_servers _ =
             last_time ()
             - connection_last_conn s.server_connection_control
           in
-          if !verbose then
-              lprintf_nl () "MASTER: Checking ip:%s ct:%d" (Ip.to_string s.server_ip) connection_time;
+          if !verbose_location then
+              lprintf_nl () "master servers: Checking ip:%s, users: %Ld, ct:%d"
+          (Ip.to_string s.server_ip) s.server_nusers connection_time;
           if not s.server_master
             && (s.server_preferred
                  || connection_time > !!become_master_delay
@@ -801,9 +834,9 @@ let update_master_servers _ =
               begin
                 if (!nmasters < max_allowed_connected_servers) then
                   begin
-                    if !verbose then
-                        lprintf_nl () "   MASTER: RAISING %s (%Ld)"
-                                  (Ip.to_string s.server_ip) s.server_nusers;
+                    if !verbose_location then
+                        lprintf_nl () "master servers: raising  %s"
+                                  (Ip.to_string s.server_ip);
                     make_master s
                   end
                 else if s.server_sent_all_queries then
@@ -820,11 +853,10 @@ let update_master_servers _ =
                              )
                           then
                             begin
-                              if !verbose then
+                              if !verbose_location then
                                 lprintf_nl ()
-                                  "   MASTER: RAISING %s (%Ld) instead of %s (%Ld)"
-                                  (Ip.to_string s.server_ip) s.server_nusers
-                                  (Ip.to_string ss.server_ip) ss.server_nusers;
+                                  "master servers: raising %s, disconnected %s"
+                                  (Ip.to_string s.server_ip) (Ip.to_string ss.server_ip);
                               ss.server_master <- false;
                               masters := tail;
                               make_master s
@@ -835,21 +867,14 @@ let update_master_servers _ =
               end
         )
     ) server_list;
-  if !verbose then
-      lprintf_nl () "MASTER: clean %d connected %d masters"
+  if !verbose_location then
+      lprintf_nl () "master servers: %d connected %d masters - re-computing completed"
         !nconnected_servers !nmasters
 
 
 open LittleEndian
 
 let _ =
-  udp_ping.[0] <- char_of_int 0xe3;
-  udp_ping.[1] <- char_of_int 0x96;
-  udp_ping.[2] <- char_of_int (Random.int 256);
-  udp_ping.[3] <- char_of_int (Random.int 256);
-  udp_ping.[4] <- char_of_int 0xAA;
-  udp_ping.[5] <- char_of_int 0x55;
-
   CommonWeb.add_redirector_info "DKSV" (fun buf ->
       buf_list (fun buf s ->
           buf_ip buf s.server_ip;
