@@ -17,6 +17,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)
 
+open Int64ops
 open CommonGlobals
 open Printf2
 open Md4
@@ -96,7 +97,6 @@ let new_file_to_share sh codedname old_impl =
         false in
     must_share_file file codedname old_impl;
     file.file_computed_md4s <- md4s;
-    file_md4s_to_register := file :: !file_md4s_to_register;
     let sh_name = Filename.basename sh.sh_name in
     if not (List.mem_assoc sh_name file.file_filenames) then begin
         file.file_filenames <- file.file_filenames @ [sh_name, GuiTypes.noips()];
@@ -144,7 +144,7 @@ let new_file_to_share sh codedname old_impl =
 lprint_newline ());
   *)
     if !verbose_share then
-      lprintf_nl () "Sharing %s" sh.sh_name;
+      lprintf_nl () "new_file_to_share: Sharing %s" sh.sh_name;
   with e ->
       lprintf_nl () "Exception %s while sharing %s" (Printexc2.to_string e)
       sh.sh_name
@@ -152,7 +152,7 @@ lprint_newline ());
 let all_shared () =
   let shared_files = ref [] in
   Hashtbl.iter (fun md4 file ->
-      match  file.file_shared with
+      match file.file_shared with
         None -> ()
       | Some _ ->  shared_files := file :: !shared_files
   ) files_by_md4;
@@ -166,24 +166,24 @@ let all_shared () =
    Should I only do it for master servers, no ?
 *)
 let send_new_shared () =
+  let tag = ref false in
   if !new_shared then
     begin
       new_shared := false;
-      let socks = ref [] in
       if !new_shared_files <> [] then
         begin
           List.iter (fun s ->
             if s.server_master then
               begin
-                if !verbose_share then
-                  lprintf_nl () "donkey send_new_shared: found master server";
+                if !verbose_share || !verbose then
+                  lprintf_nl () "send_new_shared: found master server %s:%d"
+		    (Ip.to_string s.server_ip) s.server_port;
+		tag := true;
                 do_if_connected s.server_sock (fun sock ->
                   server_send_share s.server_has_zlib sock !new_shared_files)
-              end
-                )
-          (connected_servers ());
-          if !verbose_share then
-              lprintf_nl () "donkey send_new_shared: Sent %d new shared files to servers"
+              end) (connected_servers ());
+          if !tag && (!verbose_share || !verbose) then
+              lprintf_nl () "send_new_shared: Sent %d new shared files to servers"
                     (List.length !new_shared_files);
           new_shared_files := []
         end
@@ -216,15 +216,15 @@ let check_shared_files () =
                 lprintf_nl () "Shared file doesn't exist";
                 raise Not_found;
               end;
-            if Unix32.getsize sh.shared_name false <> sh.shared_size then begin
+            if Unix32.getsize sh.shared_name <> sh.shared_size then begin
                 lprintf_nl () "Bad shared file size";
                 raise Not_found;
               end;
             computation := true;
-            let end_pos = Int64.add sh.shared_pos block_size in
+            let end_pos = sh.shared_pos ++ block_size in
             let end_pos = if end_pos > sh.shared_size then sh.shared_size
               else end_pos in
-            let len = Int64.sub end_pos sh.shared_pos in
+            let len = end_pos -- sh.shared_pos in
 (*          lprintf "compute next md4"; lprint_newline (); *)
 
             M.compute_md4 (Unix32.filename sh.shared_fd) sh.shared_pos len
@@ -263,7 +263,7 @@ let check_shared_files () =
 let _ =
   network.op_network_share <- (fun fullname codedname size ->
       if !verbose_share then
-        lprintf_nl () "Sharing %s" fullname;
+        lprintf_nl () "op_network_share: Sharing %s" fullname;
       try
 (*
 lprintf "Searching %s" fullname; lprint_newline ();
@@ -272,9 +272,8 @@ lprintf "Searching %s" fullname; lprint_newline ();
         let s = Hashtbl.find shared_files_info
             (fullname, size, mtime) in
         (* if s.sh_mtime = mtime && s.sh_size = size then begin *)
-            if !verbose_share then begin
+            if !verbose_share then
                 lprintf_nl () "donkeyShare: Using old MD4s for %s" fullname;
-              end;
             new_file_to_share s codedname None
 (*          end else begin
             if !verbose_share then begin
@@ -334,8 +333,8 @@ let remember_shared_info file new_name =
           let disk_name = file_disk_name file in
           Unix32.mtime disk_name
         with _ ->
-            if !verbose_hidden_errors then
-              lprintf_nl () "Share: Trying mtime on new name %s, disk_name %s, too much files in shared_files_new.ini"
+            if !verbose then
+              lprintf_nl () "Share: Trying mtime on new name %s, disk_name %s, TODO: too many files in shared_files_new.ini?"
 	        new_name (file_disk_name file);
             Unix32.mtime new_name
       in
