@@ -41,6 +41,16 @@ open CommonNetwork
 open FileTPTypes
 open FileTPOptions
 
+(* prints a new logline with date, module and starts newline *)
+let lprintf_nl () =
+  lprintf "%s[FileTP] "
+    (log_time ()); lprintf_nl2
+
+(* prints a new logline with date, module and does not start newline *)
+let lprintf_n () =
+  lprintf "%s[FileTP] "
+    (log_time ()); lprintf
+
 let network = new_network "FTP" "FileTP"
     [ ]
 
@@ -71,7 +81,7 @@ let as_file file = as_file file.file_file
 let file_size file = file.file_file.impl_file_size
 let file_downloaded file = file_downloaded (as_file file)
 let file_age file = file.file_file.impl_file_age
-let file_fd file = file.file_file.impl_file_fd
+let file_fd file = file_fd (as_file file)
 let file_disk_name file = file_disk_name (as_file file)
 let file_best_name file = file_best_name (as_file file)
 
@@ -102,12 +112,10 @@ let find_proto (name : string) =
 let min_range_size = megabyte
 
 let set_file_size file size =
-  lprintf "set_file_size %s : %s\n" (Unix32.filename (file_fd file)) (Int64.to_string size);
   if file_size file = zero && size <> zero then begin
-      lprintf "Setting SWARMER\n";
       let file_chunk_size =
         max megabyte (
-          size // (max (Int64.of_int 5) (size // (megabytes 5)))
+          size // (max 5L (size // (megabytes 5)))
         )
       in
       file.file_file.impl_file_size <- size;
@@ -116,7 +124,6 @@ let set_file_size file size =
       let swarmer = Int64Swarmer.create kernel (as_file file)
           file_chunk_size in
       file.file_swarmer <- Some swarmer;
-      lprintf "Swarmer set\n";
       Int64Swarmer.set_verified swarmer (fun _ _ ->
           file_must_update (as_file file);
       );
@@ -150,7 +157,7 @@ let new_file file_id file_name file_size =
       file_nconnected_clients = 0;
     } and file_impl =  {
       dummy_file_impl with
-      impl_file_fd = t;
+      impl_file_fd = Some t;
       impl_file_size = zero;
       impl_file_downloaded = zero;
       impl_file_val = file;
