@@ -64,14 +64,18 @@ let lprintf_n () =
 
 let http_send_range_request c range sock d =
   let url = d.download_url in
+
+  let real_url =
+    (Str.global_replace (Str.regexp " ") "%20" url.Url.full_file) in
+
   let referer = d.download_referer in
   
   let (x,y) = range in
-  let range = Printf.sprintf "%Ld-%Ld" x (y -- (Int64.one)) in
+  let range = Printf.sprintf "%Ld-%Ld" x (Int64.pred y) in
 
   let buf = Buffer.create 100 in
 
-  Printf.bprintf buf "GET %s HTTP/1.0\r\n" url.Url.full_file;
+  Printf.bprintf buf "GET %s HTTP/1.0\r\n" real_url;
 
 (*
             (match d.download_uri with
@@ -208,13 +212,13 @@ let rec client_parse_header c gconn sock header =
               String.sub range (dash_pos+1) (slash_pos - dash_pos - 1))
           in
           if slash_pos = star_pos - 1 then
-            x,y ++ Int64.one (* "bytes x-y/*" *)
+            x, Int64.succ y (* "bytes x-y/*" *)
           else
           let z = Int64.of_string (
               String.sub range (slash_pos+1) (len - slash_pos -1) )
           in
-          if y = z then x -- Int64.one, size else
-            x,y ++ Int64.one
+          if y = z then Int64.pred x, size else
+            x, Int64.succ y
         with
         | e ->
             lprintf_nl () "Exception %s for range [%s]"
@@ -240,7 +244,7 @@ let rec client_parse_header c gconn sock header =
                 (start_pos, end_pos)
           with _ ->
 (* A bit dangerous, no ??? *)
-              if !verbose_hidden_errors then
+              if !verbose_unknown_messages then
                 lprintf_nl () "ERROR: Could not find/parse range header (exception %s), disconnect\nHEADER: %s"
                     (Printexc2.to_string e)
                     (String.escaped header);
@@ -253,9 +257,8 @@ let rec client_parse_header c gconn sock header =
         if !verbose then lprintf_nl () "Specified length: %Ld" len;
         if len <> end_pos -- start_pos then
           begin
-            failwith "\n\nERROR: bad computed range: %Ld-%Ld/%Ld \n%s\n"
-              start_pos end_pos len
-              (String.escaped header);
+            failwith (Printf.sprintf "ERROR: bad computed range: %Ld-%Ld/%Ld \n%s\n"
+              start_pos end_pos len (String.escaped header))
           end
       with _ ->
           lprintf_nl () "[WARNING]: no Content-Length field\n%s\n"
@@ -280,7 +283,7 @@ let rec client_parse_header c gconn sock header =
           end;
 
         let b = TcpBufferedSocket.buf sock in
-        let to_read = (*min (end_pos -- !counter_pos) *)
+        let to_read = min (end_pos -- !counter_pos)
           (Int64.of_int b.len) in
 
         if !verbose then lprintf_nl () "Reading: end_pos %Ld counter_pos %Ld len %d = to_read %Ld"
@@ -293,8 +296,6 @@ end_pos !counter_pos b.len to_read;
         let swarmer = match file.file_swarmer with
             None -> assert false | Some sw -> sw
         in
-        let old_downloaded =
-          Int64Swarmer.downloaded swarmer in
 (*        List.iter (fun (_,_,r) -> Int64Swarmer.free_range r)
         d.download_ranges; *)
 
@@ -352,7 +353,7 @@ lprintf "READ: buf_used %d\n" to_read_int;
           end)
 
   with e ->
-      if !verbose_hidden_errors then
+      if !verbose_unknown_messages then
         begin
           lprintf_nl () "Exception %s in client_parse_header" (Printexc2.to_string e);
           AnyEndian.dump header
