@@ -68,6 +68,7 @@ module P = GuiTypes
 
 let _ =
   file_ops.op_file_cancel <- (fun file ->
+      Int64Swarmer.remove_swarmer file.file_swarmer;
       file.file_swarmer <- None;
       remove_file file;
   );
@@ -100,6 +101,7 @@ let _ =
         P.file_last_seen = BasicSocket.last_time ();
         P.file_priority = file_priority (as_file file);
         P.file_uids = [Uid.create (FileTP file.file_id)];
+        P.file_sub_files = [];
       }
   )
 
@@ -129,6 +131,7 @@ let _ =
         P.client_downloaded = zero;
         P.client_uploaded = zero;
         P.client_upload = None;
+        P.client_sui_verified = None;
 (*		P.client_sock_addr = (; *)
       }
   );
@@ -179,7 +182,9 @@ let _ =
           ("", "sr ar", Printf.sprintf "%d"
               (((last_time ()) - cinfo.GuiTypes.client_connect_time) / 60));
           ("", "sr", "D");
+          ("", "sr", "N");
           ("", "sr", (string_of_client_addr c));
+          ] @ (if !Geoip.active then [("?", "sr", "?")] else []) @ [
           ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_uploaded));
           ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_downloaded));
           ("", "sr", info.GuiTypes.file_name); ]);
@@ -309,7 +314,7 @@ let download_file url referer =
   let file = new_file (Md4.random ()) u.Url.full_file zero in
   
   if !verbose then
-    lprintf_nl "[FILETP]: DOWNLOAD FILE %s" (file_best_name  file); 
+    lprintf_nl () "DOWNLOAD FILE %s" (file_best_name  file); 
   if not (List.memq file !current_files) then begin
       current_files := file :: !current_files;
     end;
@@ -343,34 +348,28 @@ let get_regexp_string text r =
    and false otherwise.
  *)
 let rec op_network_parse_url url =
-  if !verbose then
-    lprintf_nl "[FILETP] op_network_parse_url";
   let location_regexp = "Location: \\(.*\\)" in
   let real_url = get_regexp_string url (Str.regexp location_regexp) in
   if !verbose then
     lprintf "real url: %s\n" real_url;
-  if (is_http_torrent url real_url) then false
+  if (is_http_torrent url real_url) then "", false
   else
     if (String2.check_prefix real_url "http://") then (
-      if !verbose then
-        lprintf_nl "[FILETP]: http download";
       let length_regexp = "Content-Length: \\(.*\\)" in
        try let length = get_regexp_int url (Str.regexp length_regexp) in
          if (length > 0) then begin
-           download_file real_url ""; true
+           download_file real_url ""; "started FileTP download", true
          end
-         else raise Not_found
+         else "test1", false
        with Not_found -> 
-           if !verbose then
-             lprintf_nl "[FILETP]: Unknown file length. Use a web browser";
-           false
+           "Unknown file length. Use a web browser", false
     )
     else if (String2.check_prefix url "ftp://") || (String2.check_prefix url "ssh://") then (
       download_file url "";
-      true
+      "started FileTP download", true
     )
     else
-      false
+      "test2", false
 
 let _ =
   network.op_network_parse_url <- op_network_parse_url
@@ -393,15 +392,22 @@ let commands = [
           Printf.bprintf buf "download started";
         _s ""
         with Not_found ->
-            if !verbose then
-              lprintf_nl "[FILETP]: Not enough parameters";
             let buf = o.conn_buf in
             if o.conn_output = HTML then
               html_mods_table_one_row buf "serversTable" "servers" [
                 ("", "srh", "Not enough parameters"); ]
             else
               Printf.bprintf buf "Not enough parameters";
-            _s ""  
+            _s ""
+	| e ->
+	    let error = Printf.sprintf "%s" (Printexc2.to_string e) in
+            let buf = o.conn_buf in
+            if o.conn_output = HTML then
+              html_mods_table_one_row buf "serversTable" "servers" [
+                ("", "srh", error); ]
+            else
+              Printf.bprintf buf "%s" error;
+            _s ""
     ), " <url> <referer> :\t\t\t\tstart downloading this URL";
 
     "mirror", "Network/FileTP", Arg_multiple (fun args o ->
@@ -415,7 +421,7 @@ let commands = [
           | _ -> raise Not_found);
 
           if !verbose then
-            lprintf_nl "[FILETP]: MIRROR [%s] [%s]" !num !url;
+            lprintf_nl () "MIRROR [%s] [%s]" !num !url;
           let u = Url.of_string !url in
           let r = Url.of_string !referer in
 
@@ -426,10 +432,10 @@ let commands = [
           let num = int_of_string !num in
           Hashtbl.iter (fun _ file ->
               if !verbose then
-                lprintf_nl "[FILETP]: COMPARE %d/%d" (file_num file) num;
+                lprintf_nl () "COMPARE %d/%d" (file_num file) num;
               if file_num file = num then begin
                   if !verbose then
-                    lprintf_nl "[FILETP]: Try HEAD from mirror";
+                    lprintf_nl () "Try HEAD from mirror";
 
 
                   download_file_from_mirror file u r;
@@ -456,7 +462,7 @@ let commands = [
             _s ""
         | Not_found ->
             if !verbose then
-              lprintf_nl "[FILETP]: Not enough parameters";
+              lprintf_nl () "Not enough parameters";
             let buf = o.conn_buf in
             if o.conn_output = HTML then
               html_mods_table_one_row buf "serversTable" "servers" [
@@ -469,10 +475,10 @@ let commands = [
 
 let _ =
   CommonNetwork.register_commands commands;
-  (* Shut up "Network.share not implemented by FileTP" *)
   network.op_network_share <- (fun fullname codedname size -> ());
-  (* Same with Network.search and Network.forget_search... *)
   network.op_network_search <- (fun ss buf -> ());
+  network.op_network_download <- (fun r -> dummy_file);
+  file_ops.op_file_commit <- (fun file new_name -> ());
   network.op_network_forget_search <- (fun s -> ());
-  (* and Network.recover_temp *)
+  network.op_network_connect_servers <- (fun s -> ());
   network.op_network_recover_temp <- (fun s -> ())
