@@ -90,8 +90,8 @@ let clean_sources () =
 let download_finished file = 
   if List.memq file !current_files then begin
       file_completed (as_file file);
+      Int64Swarmer.remove_swarmer file.file_swarmer;
       file.file_swarmer <- None;
-      
 (* TODO: maybe this remove file may have a bad effect on sharing *)
       GnutellaGlobals.remove_file file;
       old_files =:= (file.file_name, file_size file) :: !!old_files;
@@ -264,7 +264,6 @@ and read_some d c counter_pos b to_read_int =
   let up = match d.download_uploader with
       None -> assert false
     | Some up -> up in
-  let swarmer = Int64Swarmer.uploader_swarmer up in
   
   let file = d.download_file in
   
@@ -273,12 +272,6 @@ and read_some d c counter_pos b to_read_int =
       raise Exit;
     end;
 
-(*
-  lprintf "CHUNK: %s\n" 
-          (String.escaped (String.sub b.buf b.pos to_read_int)); *)
-  let old_downloaded = 
-    Int64Swarmer.downloaded swarmer in
-  
   begin
     try
       Int64Swarmer.received up
@@ -288,16 +281,7 @@ and read_some d c counter_pos b to_read_int =
           (Printexc2.to_string e);
 (* TODO: we should pause the download !!! *)
   end;
-  c.client_reconnect <- true;
-  
-  let new_downloaded = 
-    Int64Swarmer.downloaded swarmer in
-  
-(*
-lprintf "READ %Ld\n" (new_downloaded -- old_downloaded);
-lprintf "READ: buf_used %d\n" to_read_int;
-  *)
-  ()
+  c.client_reconnect <- true
 
 (*************************************************************************)
 (*                                                                       *)
@@ -325,8 +309,6 @@ and client_parse_header c gconn sock (first_line, headers) =
 
 (* The reply should be  "HTTP/1.1 200 OK" *)
     let space_pos = String.index first_line ' ' in
-    let slash_pos = String.index first_line '/' in
-    let proto = String.sub first_line (slash_pos+1) (space_pos - slash_pos) in
     let code = String.sub first_line (space_pos+1) 3 in
     let code = int_of_string code in
     
@@ -360,15 +342,15 @@ and client_parse_header c gconn sock (first_line, headers) =
               String.sub range (dash_pos+1) (slash_pos - dash_pos - 1))
           in
           if slash_pos = star_pos - 1 then 
-            x,y ++ Int64.one (* "bytes x-y/*" *)
+            x, Int64.succ y (* "bytes x-y/*" *)
           else
           let z = Int64.of_string (
               String.sub range (slash_pos+1) (len - slash_pos -1) )
           in
           if y = z then 
             failwith "Cannot parse range"
-(* x -- Int64.one, size *)
-          else x,y ++ Int64.one
+(* Int64.pred x, size *)
+          else x, Int64.succ y
         with 
         | e ->
             lprintf "[GDO] Exception %s for range [%s]\n" 
@@ -399,7 +381,7 @@ iff Content-Length = Requested Length and
                 (start_pos, end_pos)
           with _ ->  *)
 (* A bit dangerous, no ??? *)
-          if !verbose_hidden_errors then
+          if !verbose_unknown_messages then
             begin
               lprintf "[GDO] ERROR: Could not find/parse range header (exception %s), disconnect\n" 
                 (Printexc2.to_string e);
@@ -456,7 +438,7 @@ iff Content-Length = Requested Length and
           end)
   
   with e ->
-      if !verbose_hidden_errors then
+      if !verbose_unknown_messages then
         begin
           lprintf "[GDO] Exception %s in client_parse_header\n" (Printexc2.to_string e);
           print_head first_line headers;
@@ -562,7 +544,7 @@ end;
                           end;
                         d.download_ranges <- d.download_ranges @ 
                           [RANGEReq (x,y,r)];
-                        Printf.sprintf "%Ld-%Ld" x (y -- Int64.one)
+                        Printf.sprintf "%Ld-%Ld" x (Int64.pred y)
                       with Not_found ->
                           if !verbose_swarming then 
                             lprintf "Could not find range in current block\n";
@@ -571,8 +553,8 @@ end;
                 in
                 iter ()
               with Not_found -> 
-		if !verbose_hidden_errors then
-                  lprintf_nl "Unable to get a block !!";
+		if !verbose_unknown_messages then
+                  lprintf_nl () "Unable to get a block !!";
                   check_finished swarmer file;
                   raise Not_found
             in
@@ -600,13 +582,12 @@ end;
 
 and disconnect_client c r =
   if !verbose then
-    lprintf "DISCONNECT CLIENT\n";
+    lprintf_nl () "DISCONNECT CLIENT";
   match c.client_sock with
   | Connection sock -> 
       (try
-          if !verbose_msg_clients then begin
-              lprintf "Disconnected from source for %s\n" (string_of_reason r); 
-            end;
+          if !verbose_msg_clients then
+              lprintf_nl () "Disconnected from source for %s" (string_of_reason r); 
           c.client_requests <- [];
           connection_failed c.client_connection_control;
           set_client_disconnected c r;
@@ -682,7 +663,7 @@ should clearly be called). *)
       begin
         
         match c.client_user.user_kind with
-          Indirect_location (_, uid) -> 
+          Indirect_location (_, uid, _, _) -> 
             GnutellaProto.ask_for_push uid 
         | Known_location (ip, port) ->
             let token =
@@ -766,19 +747,18 @@ let push_handler cc gconn sock (first_line, headers) =
         lprintf "PARSING GIV HEADER\n"; 
       end;
     let colon_pos = String.index first_line ':' in
-    let slash_pos = String.index first_line '/' in
     let uid = Md4.of_string (String.sub first_line (colon_pos+1) 32) in
     let index = int_of_string (String.sub first_line 4 (colon_pos-4)) in
     if !verbose_msg_clients then begin
         lprintf "PARSED\n";
       end;
     let c = try
-        Hashtbl.find clients_by_uid (Indirect_location ("", uid)) 
+        Hashtbl.find clients_by_uid (Indirect_location ("", uid, ip, port)) 
       with _ ->
           try
             Hashtbl.find clients_by_uid (Known_location (ip,port))
           with _ ->
-              new_client  (Indirect_location ("", uid)) 
+              new_client  (Indirect_location ("", uid, ip, port))
     in
     c.client_host <- Some (ip, port);
     match c.client_sock with
@@ -842,11 +822,11 @@ let read_request url headers gconn sock =
         x, None, _ -> x, size
       | x, Some y, Some z ->
           if y = z then (* some vendor bug *)
-            x -- Int64.one, y
+            Int64.pred x, y
           else
-            x, y ++ Int64.one
+            x, Int64.succ y
       | x, Some y, None ->
-          x, y ++ Int64.one
+          x, Int64.succ y
     with _ -> false,  (Int64.zero, size)
   
   in
@@ -869,7 +849,7 @@ let read_request url headers gconn sock =
           ("Accept-Ranges", "bytes") ::
           ("Content-range", 
             Printf.sprintf "bytes %Ld-%Ld/%Ld"
-              chunk_pos (chunk_end -- Int64.one) size) ::
+              chunk_pos (Int64.pred chunk_end) size) ::
           headers
         end else headers in
     let headers =
@@ -1111,7 +1091,7 @@ let listen () =
 (*************************************************************************)
       
 let push_connection guid index ip port =
-  let token =
+  let _ =
     add_pending_connection connection_manager (fun token ->
         let sh =Hashtbl.find shareds_by_id index in
         let sock = connect token "gnutella download" 
@@ -1144,7 +1124,7 @@ let push_connection guid index ip port =
         write_string sock 
           (Printf.sprintf "GIV %d:%s/%s\n\n" 
             index (Md4.to_string guid) sh.shared_codedname)
-    )    
+    )
 
   in
   ()
