@@ -272,26 +272,20 @@ module P = GuiTypes
   
 let _ =
   file_ops.op_file_cancel <- (fun file ->
+      Int64Swarmer.remove_swarmer file.file_swarmer;
+      file.file_swarmer <- None;
       remove_file file;
       GnutellaProto.cancel_recover_files file
   );
+  file_ops.op_file_commit <- (fun file new_name ->
+      Int64Swarmer.remove_swarmer file.file_swarmer;
+      file.file_swarmer <- None;
+  );
   file_ops.op_file_info <- (fun file ->
-      {
-        P.file_fields = P.Fields_file_info.all;
-        
-        P.file_comment = file_comment (as_file file);
+      { (impl_file_info file.file_file) with
         P.file_name = file.file_name;
-        P.file_num = (file_num file);
         P.file_network = network.network_num;
         P.file_names = [file.file_name, P.noips()];
-        P.file_md4 = Md4.null;
-        P.file_size = file_size file;
-        P.file_downloaded = file_downloaded file;
-        P.file_all_sources = 0;
-        P.file_active_sources = 0;
-        P.file_state = file_state file;
-        P.file_sources = None;
-        P.file_download_rate = file_download_rate file.file_file;
         
          P.file_chunks = (match file.file_swarmer with
              None -> "" | Some swarmer ->
@@ -301,11 +295,8 @@ let _ =
            None -> "" | Some swarmer ->
                  Int64Swarmer.availability swarmer)];
         
-        P.file_format = FormatNotComputed 0;
         P.file_chunks_age = [|0|];
-        P.file_age = file_age file;
         P.file_last_seen = BasicSocket.last_time ();
-        P.file_priority = file_priority (as_file file);
         P.file_uids = file.file_uids;
       }    
   )
@@ -313,24 +304,17 @@ let _ =
 let _ =
   server_ops.op_server_info <- (fun s ->
       if !!enable_gnutella then
-        {
-          P.server_num = (server_num s);
+        { (impl_server_info s.server_server) with
+
           P.server_network = network.network_num;
           P.server_addr = s.server_host.host_addr;
           P.server_port = s.server_host.host_port;
-	  P.server_realport = 0;
-          P.server_score = 0;
-          P.server_tags = [];
           P.server_nusers = s.server_nusers;
           P.server_nfiles = s.server_nfiles;
-          P.server_state = server_state s;
           P.server_name = s.server_agent;
-          P.server_description = "";
-          P.server_users = None;
-          P.server_banner = "";
-          P.server_preferred = false;
-          } else
-        raise Not_found
+
+        } 
+        else raise Not_found
   );
   server_ops.op_server_connect <- (fun s ->
       GnutellaServers.connect_server s.server_host);
@@ -371,8 +355,8 @@ let _ =
       | "gnut://" :: "server" :: ip :: port :: _ ->  
           let ip = Ip.addr_of_string ip in
           let port = int_of_string port in
-          let s = new_server ip port in
-          true
+          let _ = new_server ip port in
+          "added Gnutella server", true
       | "gnut://" :: "friend" :: uid :: ip :: port :: _ ->  
           let ip = Ip.of_string ip in
           let port = int_of_string port in
@@ -380,12 +364,12 @@ let _ =
           let c = new_client (Known_location (ip, port)) in
           c.client_user.user_uid <- md4;
           friend_add (as_client c);
-          true
+          "added Gnutella friend", true
       | "gnut://" :: "friend" :: uid :: _ ->
           let md4 = Md4.of_string uid in
-          let c = new_client (Indirect_location ("", md4)) in
+          let c = new_client (Indirect_location ("", md4, Ip.null, 0)) in
           friend_add (as_client c);
-          true
+          "added Gnutella friend", true
       
       | _ -> 
           let (name, size, uids) =
@@ -400,9 +384,9 @@ let _ =
               let r = IndexedResults.get_result rs in
               let file = download_file r in
               CommonInteractive.start_download file;
-              true
+              "started Gnutella download", true
             end
-          else false
+          else "", false
   )
   
 let browse_client c = 
@@ -411,27 +395,13 @@ let browse_client c =
   
 let _ =
   client_ops.op_client_info <- (fun c ->
-      {
+      { (impl_client_info c.client_client) with
+
         P.client_network = network.network_num;
         P.client_kind = c.client_user.user_kind;
-        P.client_state = client_state (as_client c);
         P.client_type = client_type c;
-        P.client_tags = [];
         P.client_name = c.client_user.user_nick;
-        P.client_files = None;
-        P.client_num = (client_num (as_client c));
-        P.client_rating = 0;
-        P.client_chat_port = 0 ;
-        P.client_connect_time = BasicSocket.last_time ();
-        P.client_software = "";
-        P.client_release = "";
-        P.client_emulemod = "";
-        P.client_downloaded = zero;
-        P.client_uploaded = zero;
-        P.client_upload = None;
-(*        P.client_sock_addr = (match c.client_user.user_kind with
-                        | Known_location (ip,port) -> Ip.to_string ip
-                        | _ -> ""); *)
+
       }
   );
   client_ops.op_client_browse <- (fun c immediate ->
@@ -485,7 +455,9 @@ let _ =
           ("", "sr ar", Printf.sprintf "%d" 
               (((last_time ()) - cinfo.GuiTypes.client_connect_time) / 60));
           ("", "sr", "D");
+          ("", "sr", "N");
           ("", "sr", (string_of_kind cinfo.GuiTypes.client_kind));
+          ] @ (if !Geoip.active then [("?", "sr", "?")] else []) @ [
           ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_uploaded));
           ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_downloaded));
           ("", "sr", info.GuiTypes.file_name); ]);
