@@ -262,7 +262,7 @@ module Print = struct
     
     let buf = Buffer.create 100
     let print_payload msg = 
-      Buffer.clear buf;
+      Buffer.reset buf;
       let name = 
         let module Buffer = Buffer2 in
         match msg with 
@@ -417,7 +417,7 @@ let buf_uid buf s = match Uid.to_uid s with
       
 let g2_encode_payload msg = 
   let module M = G2_LittleEndian in
-  Buffer.clear buf;
+  Buffer.reset buf;
   let name = 
     match msg with 
     | TO md4 -> buf_md4 buf md4; "TO"
@@ -611,8 +611,8 @@ lprintf "\n";
   try
     if be then 
       (
-       if !verbose_hidden_errors then
-         lprintf "Big Endian not supported yet\n";
+       if !verbose then
+         lprintf_nl () "Big Endian not supported yet";
        raise Exit
       );
     let module M = G2_LittleEndian in
@@ -766,7 +766,7 @@ lprintf "\n";
         UPROD_XML xml
     | _ -> raise Not_found
   with e ->
-      if !verbose_hidden_errors then
+      if !verbose_unknown_messages then
         begin
           lprintf "Cannot parse: %s\n   " (Printexc2.to_string e);
           List.iter (fun name -> lprintf "%s/" name) names;
@@ -933,7 +933,6 @@ let udp_send_ack ip port counter =
           (char_of_int (counter land 0xff))
           (char_of_int ((counter lsr 8) land 0xff))
         in
-        let len = String.length s in
 (*        lprintf "ack sent\n"; *)
         UdpSocket.write sock false s ip port
       with e ->
@@ -1045,7 +1044,7 @@ let parse_udp_packet ip port buf =
   let nSequence = LittleEndian.get_int16 buf 4 in
   let nCount = get_uint8 buf 7 in
   let nFlags = get_uint8 buf 3 in
-  let ack_me = nFlags land 2 <> 0 in
+(*  let ack_me = nFlags land 2 <> 0 in *)
 (* Contribute:
   - deflating
   - multi-parts message
@@ -1129,8 +1128,8 @@ let parse_udp_packet ip port buf =
         | 2, false -> LittleEndian.get_int16 buf (pos+1), 3 
         | 3, false -> LittleEndian.get_int24 buf (pos+1), 4
         | _ ->
-            if !verbose_hidden_errors then
-              lprintf "no correct pkt_len, pkt_pos\n";
+            if !verbose then
+              lprintf_nl () "no correct pkt_len, pkt_pos";
             0, 1
       in
       let name_len = ((cb lsr 3) land 7) + 1 in
@@ -1140,7 +1139,7 @@ let parse_udp_packet ip port buf =
             len_len name_len pkt_len
           ;
         );
-      let msg_len = 
+      let _ = 
         if len < pos + msg_len then 
           (lprintf "RECOVER: %d < %d + %d (1 + %d + %d + %d)\n" len pos msg_len
               len_len name_len pkt_len
@@ -1271,22 +1270,22 @@ let decompress s =
   let (_,used_in, used_out) = inflate z !buf !pos len s2 0 100000 f in
   pos := !pos + len;
   
-  Printf.printf "\n----------NEW PAQUET (%d/%d)----------------------------------------\n" used_in len;  
+  lprintf "\n----------NEW PAQUET (%d/%d)----------------------------------------\n" used_in len;  
   for i = 0 to used_out - 1 do
     let c = s2.[i] in
     let x = int_of_char c in
     if x > 31 && x < 127 then 
       print_char c
     else
-      Printf.printf "(%d)" x;
+      lprintf "(%d)" x;
   done;
-  Printf.printf "\n------------------------------------------------------------\n";  
+  lprintf "\n------------------------------------------------------------\n";  
   for i = 0 to used_out - 1 do
     let c = s2.[i] in
     let x = int_of_char c in
-    Printf.printf "(%d)" x;
+    lprintf "(%d)" x;
   done;
-  Printf.printf "\n------------------------------------------------------------\n";
+  lprintf "\n------------------------------------------------------------\n";
 *)
 
 (*
@@ -1426,7 +1425,6 @@ let create_qrt_table words table_size =
       let pos = Int64.to_int pos in
       if !verbose then
         lprintf "ADDING WORD %d\n" pos; 
-      let index = pos / 8 in
       let bit = (1 lsl (pos land 7)) in
       array.(pos) <- array.(pos) lor bit;
   ) words;
@@ -1443,8 +1441,8 @@ let send_qrt_sequence s update_table =
   
   if update_table then cached_qrt_table := "";
   let table_size = 20 in
-  let infinity = 7 in
-  let table_length = 1 lsl table_size in
+(*  let infinity = 7 in
+  let table_length = 1 lsl table_size in *)
   server_send_qrt_reset s;
   
   if !cached_qrt_table = "" then 
@@ -1452,10 +1450,7 @@ let send_qrt_sequence s update_table =
   let table = !cached_qrt_table in
   
   let compressor, table =
-    if Autoconf.has_zlib then
       1, Autoconf.zlib__compress_string table
-    else
-      0, table 
   in
   
   server_send_qrt_patch s {
@@ -1478,10 +1473,10 @@ let print_string s buf =
   if String.sub buf 0 3 <> "GND" then
     lprintf "Not a GNUTELLA2 packet (no GND)\n"
   else
-  let nSequence = LittleEndian.get_int16 buf 4 in
+(*  let nSequence = LittleEndian.get_int16 buf 4 in *)
   let nCount = get_uint8 buf 7 in
   let nFlags = get_uint8 buf 3 in
-  let ack_me = nFlags land 2 <> 0 in
+(*  let ack_me = nFlags land 2 <> 0 in *)
 (* Contribute:
   - deflating
   - multi-parts message
@@ -1517,8 +1512,8 @@ let print_string s buf =
         | 2, false -> LittleEndian.get_int16 buf (pos+1), 3 
         | 3, false -> LittleEndian.get_int24 buf (pos+1), 4
         | _ ->
-            if !verbose_hidden_errors then
-              lprintf "no correct pkt_len, pkt_pos\n";
+            if !verbose then
+              lprintf_nl () "no correct pkt_len, pkt_pos";
             0, 1
       in
       let name_len = ((cb lsr 3) land 7) + 1 in
@@ -1528,7 +1523,7 @@ let print_string s buf =
             len_len name_len pkt_len
           ;
         );
-      let msg_len = 
+      let _ = 
         if len < pos + msg_len then 
           (lprintf "RECOVER: %d < %d + %d (1 + %d + %d + %d)\n" len pos msg_len
             len_len name_len pkt_len
