@@ -430,7 +430,6 @@ let connect_server s =
 (*      lprintf "CONNECTING ONE SERVER\n"; *)
             DG.connection_try s.server_connection_control;
             incr nservers;
-            DG.printf_char 's'; 
             let sock = TcpBufferedSocket.connect token "opennap to server" 
                 (Ip.to_inet_addr s.server_ip) s.server_port 
                 (server_handler s) (* Mftp_comm.server_msg_to_string*)  in
@@ -519,6 +518,7 @@ let _ =
   
   network.op_network_connected <- (fun _ -> !connected_servers != []);
   network.op_network_save_complex_options <- (fun _ -> ());
+  network.op_network_update_options <- (fun _ -> ());
   network.op_network_save_sources <- (fun _ -> ())
 
   (*
