@@ -176,7 +176,7 @@ let cprintf kont fmt =
   let rec doprn i =
     if i >= len then begin
       let res = Buffer.contents dest in
-      Buffer.clear dest;  (* just in case kprintf is partially applied *)
+      Buffer.reset dest;  (* just in case kprintf is partially applied *)
       Obj.magic (kont res)
     end else
     match String.unsafe_get fmt i with
@@ -207,9 +207,9 @@ let lprintf_nl2 fmt =
   (fmt : ('a,unit, unit) format )
 
 let lprint_newline () = lprintf "\n"
-let lprint_char c = lprintf "%c" c
-let lprint_int c = lprintf "%d" c
-let lprint_string c = lprintf "%s" c
+let lprint_char = lprintf "%c"
+let lprint_int = lprintf "%d"
+let lprint_string = lprintf "%s"
 
 let set_lprintf_handler f =
   lprintf_handler := f
@@ -264,3 +264,121 @@ let log_to_buffer buf =
 
 let set_logging b =
   lprintf_to_stdout := b
+
+(* html_mods *)
+
+let html_mods_commands buf n c l =
+    (* Name Class List *)
+    Printf.bprintf buf "\\<div class=\\\"%s\\\"\\>\\<table id=\\\"%s\\\"
+    name=\\\"%s\\\" class=\\\"%s\\\" cellspacing=0 cellpadding=0
+    width=\\\"100pc\\\"\\>\\<tbody\\>\\<tr\\>"
+    c n n c;
+    List.iter (fun (w,x,y,z)  ->
+     (* Class Title Onclick Value *)
+     Printf.bprintf buf "\\<td class=\\\"%s\\\"
+     title=\\\"%s\\\" onMouseOver=\\\"mOvr(this,'mOvr1');\\\" onMouseOut=\\\"mOut(this);\\\"
+     onClick=\\\"%s\\\" \\>%s\\</td\\>"
+     w x y z;
+    ) l;
+    Printf.bprintf buf "\\</tr\\>\\</tbody\\>\\</table\\>\\</div\\>"
+
+(*
+  html_mods_commands buf "commandsTable" "commands" [
+    ( "bu bbig", "Extend search to more servers and view results", "mSub('output','vr');", "Extend search" ) ;
+*)
+
+
+let html_mods_table_header buf n c l =
+    (* Name Class List *)
+    Printf.bprintf buf "\\<div class=\\\"%s\\\"\\>\\<table id=\\\"%s\\\" name=\\\"%s\\\" class=\\\"%s\\\" cellspacing=0 cellpadding=0\\>"
+    c n n c;
+    if List.length l > 0 then begin
+        Printf.bprintf buf "\\<tr\\>";
+        List.iter (fun (w,x,y,z)  ->
+         (* Sort Class Title Value *)
+         Printf.bprintf buf "\\<td onClick=\\\"_tabSort(this,%s);\\\" class=\\\"%s\\\" title=\\\"%s\\\"\\>%s\\</td\\>"
+         w x y z;
+        ) l;
+        Printf.bprintf buf "\\</tr\\>"
+      end
+(* Add colspan functionality to html_mods_table_header *)
+
+let html_mods_table_header_colspan buf n c l =
+    (* Name Class List *)
+    Printf.bprintf buf "\\<div class=\\\"%s\\\"\\>\\<table id=\\\"%s\\\" name=\\\"%s\\\" class=\\\"%s\\\" cellspacing=0 cellpadding=0\\>\\<tr\\>"
+    c n n c;
+    List.iter (fun (v,w,x,y,z)  ->
+     (* Sort Class Title Value *)
+     Printf.bprintf buf "\\<td colspan=%s onClick=\\\"_tabSort(this,%s);\\\" class=\\\"%s\\\" title=\\\"%s\\\"\\>%s\\</td\\>"
+     v w x y z;
+    ) l;
+    Printf.bprintf buf "\\</tr\\>"
+
+let html_mods_table_no_header buf n c l =
+    (* 1 row * n cols *)
+    (* Name Class List *)
+    Printf.bprintf buf "\\<div class=\\\"%s\\\"\\>\\<table id=\\\"%s\\\" name=\\\"%s\\\" class=\\\"%s\\\" cellspacing=0 cellpadding=0\\>\\<tr\\>"
+    c n n c;
+    List.iter (fun (t,c,d)  ->
+    (* Title Class Value *)
+     Printf.bprintf buf "\\<td class=\\\"%s\\\" %s\\>%s\\</td\\>"
+     c (if t <> "" then "title=\\\"" ^ t ^ "\\\"" else "") d;
+    ) l;
+    Printf.bprintf buf "\\</tr\\>\\</table\\>\\</div\\>"
+
+let html_mods_table_one_row buf n c l =
+    (* 1 row * n cols *)
+    (* Name Class List *)
+    Printf.bprintf buf "\\<div class=\\\"%s\\\"\\>\\<table id=\\\"%s\\\" name=\\\"%s\\\" class=\\\"%s\\\" cellspacing=0 cellpadding=0\\>\\<tr\\>"
+    c n n c;
+    List.iter (fun (t,c,d)  ->
+    (* Title Class Value *)
+     Printf.bprintf buf "\\<td class=\\\"%s\\\" %s\\>%s\\</td\\>"
+     c (if t <> "" then "title=\\\"" ^ t ^ "\\\"" else "") d;
+    ) l;
+    Printf.bprintf buf "\\</tr\\>\\</table\\>\\</div\\>"
+
+let html_mods_table_one_col buf n c l =
+    (* n rows * 1 col *)
+    (* Name Class List *)
+    Printf.bprintf buf "\\<div class=\\\"%s\\\"\\>\\<table id=\\\"%s\\\" name=\\\"%s\\\" class=\\\"%s\\\" cellspacing=0 cellpadding=0\\>\\<tr\\>"
+    c n n c;
+    List.iter (fun (t,c,d)  ->
+    (* Title Class Value *)
+     Printf.bprintf buf "\\<tr\\>\\<td class=\\\"%s\\\" %s\\>%s\\</td\\>\\</tr\\>"
+     c (if t <> "" then "title=\\\"" ^ t ^ "\\\"" else "") d;
+    ) l;
+    Printf.bprintf buf "\\</tr\\>\\</table\\>\\</div\\>"
+
+let html_mods_td buf l =
+    (* List *)
+    List.iter (fun (t,c,d)  ->
+    (* Title Class Value *)
+     Printf.bprintf buf "\\<td class=\\\"%s\\\" %s\\>%s\\</td\\>"
+     c (if t <> "" then "title=\\\"" ^ t ^ "\\\"" else "") d;
+    ) l
+
+
+let html_mods_big_header_start buf c l =
+  Printf.bprintf buf "\\<div class=\\\"%s\\\"\\>\\<table class=main border=0 cellspacing=0 cellpadding=0\\>\\<tr\\>\\<td\\>" c;
+  Printf.bprintf buf "\\<table cellspacing=0 cellpadding=0 width=100%%\\>\\<tr\\>";
+  Printf.bprintf buf "\\<td width=100%%\\>\\</td\\>";
+  let len = List.length l in
+  let cnt = ref 0 in
+  List.iter (fun s ->
+    incr cnt;
+    Printf.bprintf buf "\\<td nowrap class=\\\"fbig%s\\\"\\>%s\\</td\\>" (if !cnt = len then " pr" else "") s;
+  ) l;
+  Printf.bprintf buf "\\</tr\\>\\</table\\>\\</td\\>\\</tr\\>\\<tr\\>\\<td\\>"
+
+let html_mods_big_header_end buf =
+  Printf.bprintf buf "\\</td\\>\\</tr\\>\\</table\\>\\</div\\>"
+
+let html_mods_counter = ref true
+
+let html_mods_cntr () =
+    html_mods_counter := not !html_mods_counter;
+    if !html_mods_counter then 1 else 2
+
+let html_mods_cntr_init () =
+  html_mods_counter := true
