@@ -54,6 +54,9 @@ let is_directory filename =
 let is_link filename =
   try let s = Unix.lstat filename in s.st_kind = S_LNK with _ -> false
 
+let chmod f o = 
+  try Unix.chmod f o with e -> lprintf_nl "warning: chmod failed on %s: %s" f (Printexc2.to_string e)
+
 let rec safe_mkdir dir =  
   if Sys.file_exists dir then begin
       if not (is_directory dir) then 
@@ -62,16 +65,20 @@ let rec safe_mkdir dir =
   else 
   if is_link dir then
     begin try
-      let dir = opendir dir in () 
+      ignore (opendir dir)
       with
-        Unix.Unix_error (EACCES, _, _) -> failwith (Printf.sprintf "access denied for directory %s" dir)
-      | Unix.Unix_error (ENOENT, _, _) -> failwith (Printf.sprintf "directory %s not found, orphaned link?" dir)
-      | e -> failwith (Printf.sprintf "error %s for directory %s" (Printexc2.to_string e) dir)
+        Unix.Unix_error (EACCES, _, _) -> lprintf_nl "access denied for directory %s" dir; exit 73
+      | Unix.Unix_error (ENOENT, _, _) -> lprintf_nl "directory %s not found, orphaned link?" dir; exit 73
+      | e -> lprintf_nl "error %s for directory %s" (Printexc2.to_string e) dir; exit 73
     end
   else begin
       let predir = Filename.dirname dir in
       if predir <> dir then safe_mkdir predir;
-(*      if dir <> "." then *) Unix.mkdir dir 0o775
+      begin try
+        Unix.mkdir dir 0o775
+      with
+        e -> lprintf_nl "error %s for directory %s" (Printexc2.to_string e) dir; exit 73
+      end
     end    
     
     
@@ -108,9 +115,9 @@ let copy oldname newname =
 let rename oldname newname =
   if oldname <> newname then
   try Unix.rename oldname newname with
-    Unix_error(EXDEV,_,_) as e ->
+    Unix_error(EXDEV,_,_) ->
 (* renaming is not enough, we must COPY *)
-      lprintf "COPY %s TO %s\n" oldname newname; 
+      lprintf_nl "COPY %s TO %s" oldname newname; 
       let copied = ref false in
       try
         copy oldname newname; 
@@ -125,11 +132,14 @@ external c_seek64 : Unix.file_descr -> int64 -> Unix.seek_command -> int64 =
   "unix_lseek_64" 
 external c_getsize64 : string -> int64 = "ml_getsize64"
 external c_getfdsize64 : Unix.file_descr -> int64 = "ml_getfdsize64"
-external c_ftruncate64 : Unix.file_descr -> int64 -> unit =
+(* c_ftruncate64 sets size, optionally using a sparse file *)
+external c_ftruncate64 : Unix.file_descr -> int64 -> bool -> unit =
   "mld_ftruncate_64"
 external c_getdtablesize : unit -> int = "ml_getdtablesize"
 external c_sizeofoff_t : unit -> int = "ml_sizeofoff_t"
 
+external endianness : unit -> string = "ml_check_endianness"
+
 let rec remove_all_directory dirname =
   let files = list_directory dirname in
   List.iter (fun file ->
@@ -148,7 +158,7 @@ let random () =
   done;
   s
 
-let can_write_to_directory dirname =
+let rec can_write_to_directory dirname =
   let temp_file = Filename.concat dirname "tmp_" ^ random () ^ "_mld.tmp" in
   try
     (let oc = open_out_gen [Open_creat; Open_wronly; Open_append] 0o600 temp_file in
@@ -157,6 +167,9 @@ let can_write_to_directory dirname =
     (try Sys.remove temp_file with _ -> ())
   with
     Sys_error s when s = temp_file ^ ": " ^ (Unix.error_message Unix.EACCES) ->
-      failwith (Printf.sprintf "can not create files in directory %s, check rights..." dirname)
-  | Sys_error s -> failwith (Printf.sprintf "%s for directory %s" s dirname)
-  | e -> failwith (Printf.sprintf "%s for directory %s" (Printexc2.to_string e) dirname)
+      lprintf_nl "can not create files in directory %s, check rights..." dirname; exit 73
+  | Sys_error s when s = temp_file ^ ": " ^ (Unix.error_message Unix.ENOENT) ->
+      (try safe_mkdir dirname; can_write_to_directory dirname with _ ->
+        lprintf_nl "%s does not exist and can not be created, exiting..." dirname; exit 73)
+  | Sys_error s -> lprintf_nl "%s for directory %s" s dirname; exit 73
+  | e -> lprintf_nl "%s for directory %s" (Printexc2.to_string e) dirname; exit 73
