@@ -172,7 +172,7 @@ external is_utf8 : string -> bool = "ml_utf8_validate"
 (**********************************************************************************)
 
 (* taken from camomile *)
-(* $Id: charset.ml,v 1.5 2005/08/01 20:09:13 spiralvoice Exp $ *)
+(* $Id: charset.ml,v 1.6 2005/11/19 17:19:45 spiralvoice Exp $ *)
 (* Copyright 2002, 2003 Yamagata Yoriyuki. distributed with LGPL *)
 
 let utf8_look s i =
@@ -254,7 +254,7 @@ let utf8_get s n = utf8_look s (nth s n)
 (**********************************************************************************)
 
 (* taken from camomile *)
-(* $Id: charset.ml,v 1.5 2005/08/01 20:09:13 spiralvoice Exp $ *)
+(* $Id: charset.ml,v 1.6 2005/11/19 17:19:45 spiralvoice Exp $ *)
 (* Copyright 2002, 2003 Yamagata Yoriyuki. distributed with LGPL *)
 
 let rec length_aux s c i =
@@ -281,7 +281,7 @@ let utf8_length s = length_aux s 0 0
 
 
 (* taken from camomile *)
-(* $Id: charset.ml,v 1.5 2005/08/01 20:09:13 spiralvoice Exp $ *)
+(* $Id: charset.ml,v 1.6 2005/11/19 17:19:45 spiralvoice Exp $ *)
 (* Copyright 2002, 2003 Yamagata Yoriyuki. distributed with LGPL *)
 
 external uint_code : uchar -> int = "%identity"
@@ -1791,11 +1791,10 @@ let set_default_charset_list (lang : string) =
   (* Let's get rid of charset aliases *)
   let l = List.map (fun li -> List.hd li) (charset_list_from_language lang) in
   enc_list := List.map (fun c -> charset_to_string c ) l;
-(*  Printf.printf "List of charmap used to convert the strings:\n";
+(* Printf2.lprintf "List of charmap used to convert the strings:\n";
   List.iter (fun enc ->
-    Printf.printf "  Use encoding %s\n" enc; 
-  ) !enc_list;
-  flush stdout; *)
+    Printf2.lprintf "  Use encoding %s\n" enc; 
+  ) !enc_list; *)
   nenc := List.length !enc_list
 
 (**********************************************************************************)
