@@ -341,7 +341,7 @@ module Make(M: sig
     
     let file s =
       let digest = String.create hash_length in
-      let file_size = Unix32.getsize s false in
+      let file_size = Unix32.getsize s in
       unsafe_file digest s file_size;
       digest
     
@@ -387,8 +387,9 @@ module Make(M: sig
     
     
     let up s = int_of_char s.[0]
-    let up2 s = (int_of_char s.[0])*256+(int_of_char s.[1])
-    let up3 s = (int_of_char s.[0])*65536+(int_of_char s.[1])*256+(int_of_char s.[2])
+    let up2 s = ((int_of_char s.[0]) lsl 8) lor (int_of_char s.[1])
+    let up3 s = ((int_of_char s.[0]) lsl 16) lor 
+                ((int_of_char s.[1]) lsl 8) lor (int_of_char s.[2])
   
     let enabled = true
   end
@@ -469,11 +470,11 @@ module Tiger = Make(struct
         
       let unsafe_file digest filename = 
         Printf2.lprintf "Tiger.unsafe_file not implemented\n";
-        exit 2
+        exit 70
         
       let digest_subfile digest fd pos len = 
         Printf2.lprintf "Tiger.digest_subfile not implemented\n";
-        exit 2
+        exit 70
     
       module Base = Base32
         
@@ -488,7 +489,7 @@ module PreTigerTree = Make(struct
         "tigertree_unsafe64_fd"
       
       let unsafe_file digest filename file_size = 
-        let fd = Unix32.create_diskfile filename [Unix.O_RDONLY] 0o444 in
+        let fd = Unix32.create_diskfile filename false in
         Unix32.apply_on_chunk fd Int64.zero file_size 
           (fun fd pos ->
             digest_subfile digest fd pos file_size)
