@@ -22,6 +22,11 @@
 
 #include <string.h> 
 #include <ctype.h>
+#include <caml/config.h>
+#include <caml/signals.h>
+#ifdef __MORPHOS__
+#include <inttypes.h>
+#endif
 
 #define lseek XXXXXXXXX
 #define read XXXXXXXXX
@@ -29,9 +34,6 @@
 
 #define UNIX_BUFFER_SIZE 16384
 
-extern void enter_blocking_section(); 
-extern void leave_blocking_section();
-
 
 /*******************************************************************
 
@@ -126,8 +128,7 @@ value try_poll(value fdlist, value timeout) /* ML */
 /*        printf("SETTING %d TO %d\n", fd, nfds); */
         pfds[nfds] = v;
         nfds++;
-      } else
-        ;
+      } // else ;
     }
   }
 /*  printf("POLL: %d/%d\n", nfds, ufds_size); */
@@ -273,9 +274,8 @@ value ml_getdtablesize(value unit)
       
 #else
 
-      printf("Your shell allows %d file descriptors, but select only allows %d file descriptors\n", dtablesize, maxselectfds);
       maxfd = maxselectfds;
-      printf("Limit has been set to %d\n", maxfd);
+		  // printf("[Info] File descriptor limit: MIN(process: %d, select(): %d) = %d\n", dtablesize, maxselectfds, maxfd);
 
 #endif
   }
@@ -351,7 +351,7 @@ value ml_setsock_iptos_throughput(value sock_v)
 
 value ml_sizeofoff_t(value unit)
 {
-  return Val_int(sizeof(off_t));
+  return Val_int(sizeof(OFF_T));
 }
 
 /*******************************************************************
@@ -393,12 +393,13 @@ value ml_getfdsize64(value fd_v)
 *******************************************************************/
 
 #define ZEROS_LEN 1024
-value mld_ftruncate_64(value fd_v, value len_v)
+value mld_ftruncate_64(value fd_v, value len_v, value sparse)
 {
-  off_t len = Int64_val(len_v);
+  OFF_T len = Int64_val(len_v);
   OS_FD fd = Fd_val(fd_v);  
+	 int use_sparse = Bool_val(sparse);
 
-  os_ftruncate(fd, len);
+  os_ftruncate(fd, len, use_sparse);
     
   return Val_unit;
 }
@@ -500,17 +501,17 @@ unsigned char hash_buffer[HASH_BUFFER_LEN];
 value HASH_NAME##_unsafe64_fd (value digest_v, value fd_v, value pos_v, value len_v) \
 { \
   OS_FD fd = Fd_val(fd_v); \
-  off_t pos = Int64_val(pos_v); \
-  off_t len = Int64_val(len_v); \
+  OFF_T pos = Int64_val(pos_v); \
+  OFF_T len = Int64_val(len_v); \
   unsigned char *digest = String_val(digest_v); \
   HASH_CONTEXT context; \
-  int nread; \
+  ssize_t nread; \
  \
   HASH_INIT (&context); \
   os_lseek(fd, pos, SEEK_SET); \
  \
   while (len!=0){ \
-    int max_nread = HASH_BUFFER_LEN > len ? len : HASH_BUFFER_LEN; \
+    size_t max_nread = HASH_BUFFER_LEN > len ? len : HASH_BUFFER_LEN; \
  \
     nread = os_read (fd, hash_buffer, max_nread); \
  \
@@ -552,7 +553,7 @@ value HASH_NAME##_unsafe_file (value digest_v, value filename_v, value file_size
   unsigned char *digest = String_val(digest_v); \
   FILE *file; \
   HASH_CONTEXT context; \
-  int len; \
+  size_t len; \
  \
   if ((file = fopen (filename, "rb")) == NULL) \
     raise_not_found(); \
@@ -586,19 +587,19 @@ ML_HASH(md4,MD4_CTX,MD4Init,MD4Update,md4_finish)
 *******************************************************************/
 #include "tiger.h"
 
-static void tiger_tree_fd(OS_FD fd, int len, int pos, int block_size, char *digest)
+static void tiger_tree_fd(OS_FD fd, size_t len, size_t pos, size_t block_size, char *digest)
 {
   static char tiger_buffer[BLOCK_SIZE+1];
   if(block_size == BLOCK_SIZE){
-    int length = (len - pos > BLOCK_SIZE) ? BLOCK_SIZE : len - pos;
+    size_t length = (len - pos > BLOCK_SIZE) ? BLOCK_SIZE : len - pos;
     char *s = tiger_buffer+1;
-    int toread = length;
+    size_t toread = length;
     char *curs = s;
       while (toread!=0){
       int max_nread = toread;
 /* HASH_BUFFER_LEN > toread ? toread : HASH_BUFFER_LEN; */
 
-      int nread = os_read (fd, curs, max_nread);
+      ssize_t nread = os_read (fd, curs, max_nread);
 
         if(nread <= 0) {
         unix_error(errno, "tiger_safe_fd: Read", Nothing);
@@ -624,8 +625,8 @@ static void tiger_tree_fd(OS_FD fd, int len, int pos, int block_size, char *dige
 value tigertree_unsafe64_fd (value digest_v, value fd_v, value pos_v, value len_v)
 {
   OS_FD fd = Fd_val(fd_v);
-  off_t pos = Int64_val(pos_v);
-  off_t len = Int64_val(len_v);
+  OFF_T pos = Int64_val(pos_v);
+  OFF_T len = Int64_val(len_v);
   unsigned char *digest = String_val(digest_v);
 /*  int nread; */
 
@@ -676,7 +677,7 @@ value ml_setlcnumeric(value no)
 
 // #include "socketaddr.h"
 #ifndef _WIN32
-#include <sys/types.h>
+// #include <sys/types.h>
 #include <netdb.h>
 #endif
 
@@ -786,7 +787,7 @@ static void store_in_job(value job_v)
   End_roots();
 }
 
-#if !defined(HAVE_PTHREAD) || !(HAS_GETHOSTBYNAME_R || GETHOSTBYNAME_IS_REENTRANT)
+#if !defined(HAVE_PTHREAD) || (!(HAS_GETHOSTBYNAME_R || GETHOSTBYNAME_IS_REENTRANT) && !defined(HAS_SIGWAIT))
 
 value ml_ip_job_start(value job_v)
 {
@@ -845,6 +846,8 @@ static void * hasher_thread(void * arg)
 {
   struct timeval now;
   struct timespec timeout;
+
+#if !defined(PTW32_STATIC_LIB)	
   sigset_t mask;
 
   /* Block all signals so that we don't try to execute a Caml signal handler */
@@ -852,6 +855,7 @@ static void * hasher_thread(void * arg)
   pthread_sigmask(SIG_BLOCK, &mask, NULL);
   
   nice(19);
+#endif
   
   pthread_mutex_lock(&mutex);
 
@@ -919,7 +923,7 @@ value ml_has_pthread(value unit)
 #if defined(__MINGW32__)
 /* taken from www.greatchief.plus.com/gpc/os-hacks.h */
 #define _fullpath(res,path,size) \
-  (GetFullPathName ((path), (size), (res), NULL) ? (res) : NULL)
+  (GetFullPathNameW ((path), (size), (res), NULL) ? (res) : NULL)
 
 #define realpath(path,resolved_path) _fullpath(resolved_path, path, MAX_PATH)
 
@@ -942,13 +946,15 @@ struct statfs {
    long    f_spare[6]; /* spare for later */
 };
 
-static int statfs (const char *path, struct statfs *buf)
+static int statfs (const unsigned char *path, struct statfs *buf)
   {
     HINSTANCE h;
     FARPROC f;
     int retval = 0;
-    char tmp [MAX_PATH], resolved_path [MAX_PATH];
-    realpath(path, resolved_path);
+    WCHAR tmp [MAX_PATH], resolved_path [MAX_PATH];
+    WCHAR * wpath=utf8_to_utf16(path);
+    realpath(wpath, resolved_path);
+    free(wpath);
     if (!resolved_path)
       retval = - 1;
     else
@@ -956,7 +962,7 @@ static int statfs (const char *path, struct statfs *buf)
         /* check whether GetDiskFreeSpaceExA is supported */
         h = LoadLibraryA ("kernel32.dll");
         if (h)
-          f = GetProcAddress (h, "GetDiskFreeSpaceExA");
+          f = GetProcAddress (h, "GetDiskFreeSpaceExW");
         else
           f = NULL;
         if (f)
@@ -979,7 +985,7 @@ static int statfs (const char *path, struct statfs *buf)
           {
             DWORD sectors_per_cluster, bytes_per_sector;
             if (h) FreeLibrary (h);
-            if (!GetDiskFreeSpaceA (resolved_path, &sectors_per_cluster,
+            if (!GetDiskFreeSpaceW (resolved_path, &sectors_per_cluster,
                    &bytes_per_sector, &buf -> f_bavail, &buf -> f_blocks))
               {
                 errno = ENOENT;
@@ -997,11 +1003,11 @@ static int statfs (const char *path, struct statfs *buf)
       }
 
     /* get the FS volume information */
-    if (strspn (":", resolved_path) > 0) resolved_path [3] = '\0'; /* we want only the root */    
-    if (GetVolumeInformation (resolved_path, NULL, 0, &buf -> f_fsid, &buf -> f_namelen, NULL, tmp, MAX_PATH))
+    if (wcsspn (L":", resolved_path) > 0) resolved_path [3] = '\0'; /* we want only the root */    
+    if (GetVolumeInformationW (resolved_path, NULL, 0, &buf -> f_fsid, &buf -> f_namelen, NULL, tmp, MAX_PATH))
     /* http://msdn.microsoft.com/library/default.asp?url=/library/en-us/fileio/fs/getvolumeinformation.asp */
      {
-     	if (strcasecmp ("NTFS", tmp) == 0)
+     	if (_wcsicmp (L"NTFS", tmp) == 0)
      	 {
      	   buf -> f_type = NTFS_SUPER_MAGIC;
      	 }
@@ -1042,7 +1048,7 @@ statfs_statfs (value pathv)
 {
   CAMLparam1 (pathv);
   CAMLlocal1 (bufv);
-  const char *path = String_val (pathv);
+  const unsigned char *path = String_val (pathv);
   struct statfs buf;
   if (statfs (path, &buf) == -1)
     raise_constant(*(value *)caml_named_value("error"));
@@ -1064,7 +1070,7 @@ statfs_statfs (value pathv)
 
 #ifdef HAVE_STATS
 static value
-#if ((defined (sun) || defined (__sun__)))
+#if ((defined (sun) || defined (__sun__))) || (defined(__NetBSD__) && (__NetBSD_Version__ > 299000000)) || defined (__hpux__)
 copy_statfs (struct statvfs *buf)
 #else
 copy_statfs (struct statfs *buf)
@@ -1084,7 +1090,7 @@ copy_statfs (struct statfs *buf)
   v = copy_int64 (buf->f_bavail); caml_modify (&Field (bufv, 4), v);
   v = copy_int64 (buf->f_files); caml_modify (&Field (bufv, 5), v);
   v = copy_int64 (buf->f_ffree); caml_modify (&Field (bufv, 6), v);
-#if ((defined (sun) || defined (__sun__)))
+#if ((defined (sun) || defined (__sun__))) || defined (__hpux__)
   v = copy_int64 (-1); caml_modify (&Field (bufv, 7), v);
   v = copy_int64 (buf->f_namemax); caml_modify (&Field (bufv, 8), v);
   v = copy_string (buf->f_basetype); caml_modify (&Field (bufv, 9), v);
@@ -1109,7 +1115,7 @@ copy_statfs (struct statfs *buf)
 #endif /* defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(__APPLE__) */
   caml_modify (&Field (bufv, 7), Val_unit);
   v = copy_int64 (-1); caml_modify (&Field (bufv, 10), v);
-#endif /*  ((defined (sun) || defined (__sun__))) */
+#endif /*  ((defined (sun) || defined (__sun__))) || defined (__hpux__) */
   CAMLreturn (bufv);
 }
 #endif
@@ -1121,7 +1127,7 @@ statfs_statfs (value pathv)
   CAMLparam1 (pathv);
   CAMLlocal1 (bufv);
   const char *path = String_val (pathv);
-#if ((defined (sun) || defined (__sun__)))
+#if ((defined (sun) || defined (__sun__))) || (defined(__NetBSD__) && (__NetBSD_Version__ > 299000000)) || defined (__hpux__)
   struct statvfs buf;
   if (statvfs (path, &buf) == -1)
 #else
@@ -1136,3 +1142,155 @@ statfs_statfs (value pathv)
 #endif
 }
 #endif /* defined(__MINGW32__) */
+
+
+#if defined(__MINGW32__)
+static HWND myHWND = NULL;
+#endif
+
+value
+external_start (value version)
+{
+
+// Disable close button on console
+#if defined(__MINGW32__)
+  char *buf = "[MLDonkey TitleSearch]\0"; // if multiple instances
+  char *title = String_val (version);
+  SetConsoleTitle((LPCTSTR)buf);
+  myHWND = FindWindowEx(NULL, NULL, NULL, (LPCTSTR)buf);
+  SetConsoleTitle((LPCTSTR)title);
+
+  if (myHWND != NULL) {
+    HMENU hmenu = GetSystemMenu(myHWND, FALSE);
+    MENUITEMINFO CloseItem;
+    CloseItem.cbSize = sizeof(MENUITEMINFO);
+    CloseItem.fMask = MIIM_ID;
+    CloseItem.wID = SC_MINIMIZE+1;
+    EnableMenuItem(hmenu, SC_CLOSE, MF_BYCOMMAND | MF_GRAYED);
+    SetMenuItemInfo(hmenu, SC_CLOSE, FALSE, &CloseItem);
+    DrawMenuBar(myHWND);
+  }
+#endif
+
+#if defined(HAVE_PTHREAD) && defined(PTW32_STATIC_LIB)
+	pthread_win32_process_attach_np();
+#endif
+	return Val_unit;
+
+}
+
+// Crypto 
+void crypto_exit();
+void createKey(char buf[]);
+unsigned long loadKey(char privateKeyBase64[], char buf[]);
+int createSignature(byte *buf, int maxLen, byte *key, int keyLen, uint32_t cInt, uint8_t ipType, uint32_t ip);
+int verifySignature(byte *key, int keyLen, byte *sig, int sigLen, uint32_t cInt, uint8_t ipType, uint32_t ip);
+
+value
+external_exit (void) 
+{
+// Revert console system menu
+#if defined(__MINGW32__)
+  if (myHWND != NULL) {
+    HMENU hmenu = GetSystemMenu(myHWND, FALSE);
+    MENUITEMINFO CloseItem;
+    CloseItem.cbSize = sizeof(MENUITEMINFO);
+    CloseItem.fMask = MIIM_ID;
+    CloseItem.wID = SC_CLOSE;
+    SetMenuItemInfo(hmenu, SC_MINIMIZE+1, FALSE, &CloseItem);
+    EnableMenuItem(hmenu, SC_CLOSE, MF_BYCOMMAND | MF_ENABLED);
+    DrawMenuBar(myHWND);
+  }
+#endif
+
+#if defined(HAVE_PTHREAD) && defined(PTW32_STATIC_LIB)
+	pthread_win32_process_detach_np();
+#endif
+
+	crypto_exit();
+	return Val_unit;
+}
+
+value
+ml_uname(void) {
+	char buf[4096];
+	buf[0] = '\0';
+	os_uname(buf);
+	return caml_copy_string(buf);
+}
+
+
+// return private key Base64Encoded
+value 
+ml_createKey() {
+	char buf[4096];
+	createKey(buf);
+	return caml_copy_string(buf);
+}
+
+// return public key
+value 
+ml_loadKey(value privatekey) {
+  char *s = String_val(privatekey);
+	char buf[4096];
+	unsigned long len =	loadKey(s, buf);
+
+  value res;
+  res = caml_alloc_string(len);
+  memmove(String_val(res), buf, len);
+
+	return res;
+}
+
+value
+ml_createSignature(value m_key, value m_keyLen, value m_cInt, value m_ipType, value m_ip) {
+
+	byte *key = (byte*) String_val(m_key);
+	int keyLen = Int_val(m_keyLen);
+	uint32_t cInt = Int64_val(m_cInt);
+	int ipType = Int_val(m_ipType);
+	uint32_t ip = Int64_val(m_ip);
+
+	byte buf[4096];
+
+	int len = createSignature(buf, 200, key, keyLen, cInt, ipType, ip);
+
+  value res;
+  res = caml_alloc_string(len);
+  memmove(String_val(res), buf, len);
+
+	return res;
+}
+
+value 
+ml_verifySignature(value m_key, value m_keyLen, value m_sig, value m_sigLen, value m_cInt, value m_ipType, value m_ip) {
+
+	byte* key = (byte*) String_val(m_key);
+	int keyLen = Int_val(m_keyLen);
+	byte* sig = (byte*) String_val(m_sig);
+	int sigLen = Int_val(m_sigLen);
+	uint32_t cInt = Int64_val(m_cInt);
+	int ipType = Int_val(m_ipType);
+	uint32_t ip = Int64_val(m_ip);
+	
+	return Val_bool( 
+			verifySignature(key, keyLen, sig, sigLen, cInt, ipType, ip)
+	);
+}
+
+value
+ml_verifySignature_bytecode(value *argv, int argn) {
+	return ml_verifySignature(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6]);
+}
+
+value ml_check_endianness(void)
+{
+  CAMLparam0 ();
+  CAMLlocal1 (v);
+#ifdef ARCH_BIG_ENDIAN
+  v = copy_string ("big endian");
+#else
+  v = copy_string ("little endian");
+#endif
+  CAMLreturn (v);
+}
