@@ -24,13 +24,13 @@ open BasicSocket
 
 let latencies = Hashtbl.create 2131
 
-let max_opened_connections = ref (fun () -> maxi 20 (MlUnix.max_sockets - 50))
+let max_opened_connections = ref (fun () -> 20)
 let max_connections_per_second = ref (fun () -> 50)
 
 let opened_connections = ref 0
 let opened_connections_this_second = ref 0
 
-let max_buffer_size = ref 1000000
+let max_buffer_size = ref 10000000
 
 let bind_address = ref Unix.inet_addr_any
 let ip_packet_size = ref 40
@@ -179,7 +179,6 @@ let can_open_connection manager =
 
 let schedule_connections () =
   let max_wanted = !max_opened_connections () in
-  let current_connections = !opened_connections in
   let max_connections_per_second = !max_connections_per_second () in
 
   let rec iter todo_managers done_managers =
@@ -553,12 +552,11 @@ let write t s pos1 len =
 (*          if t.monitored then begin
               lprintf "write: direct written %d\n" nw;
 end; *)
-          tcp_uploaded_bytes := Int64.add !tcp_uploaded_bytes (Int64.of_int nw);
+          tcp_uploaded_bytes := !tcp_uploaded_bytes ++ (Int64.of_int nw);
           (match t.write_control with
               None -> ()
             | Some bc ->
-                bc.moved_bytes <-
-                  Int64.add bc.moved_bytes (Int64.of_int nw));
+                bc.moved_bytes <- bc.moved_bytes ++ (Int64.of_int nw));
           if t.nwrite = 0 then begin
 (*              if t.connecting then
                 lprintf "WRITE BEFORE CONNECTION.......\n";
@@ -623,8 +621,7 @@ let can_read_handler t sock max_len =
             t.event_handler t BUFFER_OVERFLOW;
             lprintf "[OVERFLOW] in %s" (info sock);
             close t Closed_for_overflow;
-            raise exn_exit;
-            0
+            raise exn_exit
           )
         else
         if b.len + min_read_size < buf_len then
@@ -698,11 +695,10 @@ let can_read_handler t sock max_len =
     end;
 *)
 
-    tcp_downloaded_bytes := Int64.add !tcp_downloaded_bytes (Int64.of_int nread);
+    tcp_downloaded_bytes := !tcp_downloaded_bytes ++ (Int64.of_int nread);
     (match t.read_control with
         None -> () | Some bc ->
-          bc.moved_bytes <-
-          Int64.add bc.moved_bytes (Int64.of_int nread));
+          bc.moved_bytes <- bc.moved_bytes ++ (Int64.of_int nread));
 
     t.nread <- t.nread + nread;
     if nread > 0 then begin
@@ -755,12 +751,11 @@ let can_write_handler t sock max_len =
 
 (*            if t.monitored then
 (lprintf "written %d\n" nw; ); *)
-        tcp_uploaded_bytes := Int64.add !tcp_uploaded_bytes (Int64.of_int nw);
+        tcp_uploaded_bytes := !tcp_uploaded_bytes ++ (Int64.of_int nw);
         (match t.write_control with
             None -> ()
           | Some bc ->
-              bc.moved_bytes <-
-                    Int64.add bc.moved_bytes (Int64.of_int nw));
+              bc.moved_bytes <- bc.moved_bytes ++ (Int64.of_int nw));
             if t.nwrite = 0 then begin
 (*                lprintf "add_connect_latency at %f\n" (current_time ()); *)
                 add_connect_latency t.host t.connect_time;
@@ -960,9 +955,9 @@ let compute_lost_byte bc =
   if bc.total_bytes = 0 then -1 else
   let sum = ref Int64.zero in
   for i = 0 to 3600-1 do
-    sum := Int64.add !sum (Int64.of_int bc.lost_bytes.(i));
+    sum := !sum ++ (Int64.of_int bc.lost_bytes.(i));
   done;
-  Int64.to_int (Int64.div !sum (Int64.of_int 3600))
+  Int64.to_int (!sum // 3600L)
 
 let moved_bytes bc = bc.moved_bytes
 
@@ -1194,9 +1189,8 @@ let create token name fd handler =
         ;
     );
   set_dump_info sock (dump_socket t);
-  if !debug then begin
-      lprintf "[fd %d %s]\n" (sock_num sock) name;
-    end;
+  if !debug then
+      lprintf_nl "[tBS] fd %d %s" (sock_num sock) name;
   t.sock_in <- sock;
   t.sock_out <- sock;
   t
@@ -1249,9 +1243,8 @@ let create_pipe token name fd_in fd_out handler =
 
   let sock_in = BasicSocket.create name fd_in (fun _ event ->
         tcp_handler t event) in
-  if !debug then begin
-      lprintf "[fd %d %s]\n" (sock_num sock_in) name;
-    end;
+  if !debug then
+      lprintf_nl "[tBS] fd %d %s" (sock_num sock_in) name;
   set_printer sock_in fname;
   set_dump_info sock_in (dump_socket t);
 
@@ -1345,8 +1338,7 @@ let connect token name host port handler =
 (*Printf.bprintf buf "User-Agent: Mozilla/4.0 (compatible; MSIE 5.01; Windows NT; Hotbar 2.0)\n";*)
         Printf.bprintf buf "User-Agent: MLdonkey/%s\n" Autoconf.current_version;
         Printf.bprintf buf "\n";
-        let nw = MlUnix.write s (Buffer.contents buf) 0 (Buffer.length buf) in
-        ()
+        ignore (MlUnix.write s (Buffer.contents buf) 0 (Buffer.length buf))
       end;
     let t = create token name s handler in
 
@@ -1382,17 +1374,20 @@ let connect token name host port handler =
         forecast_upload t 0;    (* The TCP ACK packet *)
         t
     | Unix.Unix_error (Unix.ENETUNREACH,_,_) as e ->
+        (* log nothing here, but later in donkeyClient.ml *)
 	close t Closed_connect_failed;
         raise e
     | e ->
-        lprintf "For host %s:%d   "
-            (Unix.string_of_inet_addr host) port;
         close t Closed_connect_failed;
         raise e
   with
-    Unix.Unix_error (Unix.ENETUNREACH,_,_) as e -> raise e
+    Unix.Unix_error (Unix.ENETUNREACH,_,_) as e -> raise e (* avoid logging *)
+  | Unix.Unix_error (Unix.ENOBUFS,_,_) as e ->
+      if Autoconf.windows then lprintf_nl
+        "No more free buffers, read http://support.microsoft.com/kb/q196271/ to fix this problem";
+      raise e
   | e ->
-      lprintf "EXCEPTION %s  before connect to host %s:%d\n"
+      lprintf_nl "Exception (%s) before connect to host %s:%d"
           (Printexc2.to_string e) (Unix.string_of_inet_addr host) port;
       raise e
 
@@ -1460,7 +1455,7 @@ let simple_send_buf buf sock =
   write sock s 0 len
 
 let value_send sock m =
-  (* Buffer.clear internal_buf; *)
+  (* Buffer.reset internal_buf; *)
   Buffer.reset internal_buf;
   Buffer.add_string internal_buf (Marshal.to_string m []);
   simple_send_buf internal_buf sock
@@ -1565,7 +1560,7 @@ let deflate_timer _ =
   to_deflate := [];
   to_deflate_len := 0
 
-let to_deflate conn =
+let _to_deflate conn =
   if not (List.memq conn !to_deflate) then
     to_deflate := conn :: !to_deflate;
   if !to_deflate_len > 1000000 then
@@ -1605,7 +1600,7 @@ let write t s pos len =
   | Some (_,_, _,wbuf) ->
 
       to_deflate_len := !to_deflate_len + len;
-      to_deflate t;
+      _to_deflate t;
       buf_add t wbuf s pos len
 
 (*************************************************************************)
@@ -1689,7 +1684,6 @@ to an option 'packet_unit' (default 250). Ask Simon if it makes sense.
                     let nconnections = maxi bc.nconnections 1 in
                     let can_read = maxi 1 (bc.remaining_bytes / nconnections) in
                     let can_read = maxi !ip_packet_size (can_read * t.read_power) in
-                    let old_nread = t.nread in
                     (try
 (*                    lprintf "allow to read %d\n" can_read; *)
                         can_read_handler t t.sock_in can_read
@@ -1698,8 +1692,6 @@ to an option 'packet_unit' (default 250). Ask Simon if it makes sense.
                             (Printexc2.to_string e)
                           t.name (sock_num t.sock_in) *) ()
                     );
-                    bc.remaining_bytes <- bc.remaining_bytes -
-                    t.nread + old_nread;
                     bc.nconnections <- bc.nconnections - t.read_power
                   else
                   if !verbose_bandwidth > 2 then begin
@@ -1848,6 +1840,13 @@ let _ =
   Heap.add_memstat "tcpBufferedSocket" (fun level buf ->
       Printf.bprintf buf "  %d latencies\n" (Hashtbl.length latencies);
       Printf.bprintf buf "  %d entries in net_stats\n" (Hashtbl.length net_stats);
+      Printf.bprintf buf "  String.length big_buffer: %d\n" (String.length big_buffer);
+      Printf.bprintf buf "  connection_managers: %d\n" (List.length !connection_managers);
+      Printf.bprintf buf "  read_bandwidth_controlers: %d\n" (List.length !read_bandwidth_controlers);
+      Printf.bprintf buf "  write_bandwidth_controlers: %d\n" (List.length !write_bandwidth_controlers);
+      Printf.bprintf buf "  to_deflate: %d\n" (List.length !to_deflate);
+      Printf.bprintf buf "  max_opened_connections: %d\n" (!max_opened_connections ());
+      Printf.bprintf buf "  max_connections_per_second: %d\n" (!max_connections_per_second ());
   );
   add_infinite_timer 1.0 proc_net_timer
 
